using System;
using System.Collections.Generic;
using Microsoft.Practices.EnterpriseLibrary.Caching;
using Microsoft.Practices.EnterpriseLibrary.Caching.Expirations;
using Microsoft.Practices.EnterpriseLibrary.Common;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Caching.Configuration;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
namespace CachingCoreTests
{
    /// <summary>
    /// Summary description for CachingIsolatedStoreDicTestFixture
    /// </summary>
    [TestClass]
    public class CachingIsolatedStoreDicTestFixture
    {
        private CacheManager itemCache;
        private CacheManagerFactory factory;
        private DictionaryConfigurationSource configSource;
        public CachingIsolatedStoreDicTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        /// <summary>
        /// Initialize() is called once during test execution before
        /// test methods in this test class are executed.
        /// </summary>
        [TestInitialize()]
        public void Initialize()
        {
            //  TODO: Add test initialization code
            CachingDictionarySource sourceSection = new CachingDictionarySource();
            configSource = sourceSection.BuildDictionarySourceSection();

            factory = new CacheManagerFactory(configSource);
            itemCache = factory.Create("IsolatedCacheManagerDic");
            if (itemCache != null)
                itemCache.Flush();

            itemCache = factory.Create("IsolatedCacheManagerEncDic");
            if (itemCache != null)
                itemCache.Flush();
        }

        /// <summary>
        /// Cleanup() is called once during test execution after
        /// test methods in this class have executed unless the
        /// corresponding Initialize() call threw an exception.
        /// </summary>
        [TestCleanup()]
        public void Cleanup()
        {
            //  TODO: Add test cleanup code
        }

        #region "Default Add overload"
        [TestMethod]
        public void IsolatedStoreAddDefaultDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void IsolatedStoreAddRemoveDefaultDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("1");
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

            itemCache.Remove("3");
            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.IsNull(writtenItem);

            itemCache.Remove("4");
            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.IsNull(writtenItem);

            itemCache.Remove("5");
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.IsNull(writtenItem);

            itemCache.Remove("6");
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

            Assert.AreEqual(0, itemCache.Count);
        }

        [TestMethod]
        public void IsolatedStoreAddDupKeyDefaultDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(1, "Toy11", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy11", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);
            Assert.AreEqual(1, itemCache.Count);
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreAddNullKeyDefaultDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(null, itemDetails);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreReadNullKeyDefaultDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            writtenItem = (ItemDetails)itemCache.GetData(null);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreRemoveNullKeyDefaultDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemCache.Remove(null);

        }

        [TestMethod]
        public void IsolatedStoreNonExistKeyDefaultDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void IsolatedStoreAddRemoveCountDefaultDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemCache.Remove("1");

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(7, "Toy7", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("3");

            Assert.AreEqual(5, itemCache.Count);

            itemCache.Remove("5");
            itemCache.Remove("4");

            Assert.AreEqual(3, itemCache.Count);

            itemCache.Remove("2");
            itemCache.Remove("4");
            Assert.AreEqual(2, itemCache.Count);
        }

        [TestMethod]
        public void IsolatedStoreContainsDefaultDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);
            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);

            Assert.AreEqual(true, itemCache.Contains("1"));
            Assert.AreEqual(true, itemCache.Contains("2"));
            Assert.AreEqual(true, itemCache.Contains("3"));
            Assert.AreEqual(true, itemCache.Contains("4"));
            Assert.AreEqual(true, itemCache.Contains("5"));
            Assert.AreEqual(true, itemCache.Contains("6"));
            Assert.AreEqual(false, itemCache.Contains("7"));


        }

        [TestMethod]
        public void IsolatedStoreAddNullObjectDefaultDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        public void IsolatedStoreAddNullValueDefaultDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, null, 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreCheckContainsNullDefaultDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual(false, itemCache.Contains(null));
        }

        #endregion

        #region "Absolute Time Expiry, Add overload"
        [TestMethod]
        public void IsolatedStoreAbsTimeExpiryAddDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void IsolatedStoreAbsTimeExpiryAddRemoveDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("1");
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

            itemCache.Remove("3");
            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.IsNull(writtenItem);

            itemCache.Remove("4");
            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.IsNull(writtenItem);

            itemCache.Remove("5");
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.IsNull(writtenItem);

            itemCache.Remove("6");
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

            Assert.AreEqual(0, itemCache.Count);
        }

        [TestMethod]
        public void IsolatedStoreAbsTimeExpiryAddDupKeyDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(1, "Toy11", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy11", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);
            Assert.AreEqual(1, itemCache.Count);
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreAbsTimeExpiryAddNullKeyDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(null, itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreAbsTimeExpiryReadNullKeyDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            writtenItem = (ItemDetails)itemCache.GetData(null);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreAbsTimeExpiryRemoveNullKeyDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemCache.Remove(null);

        }

        [TestMethod]
        public void IsolatedStoreAbsTimeExpiryNonExistKeyDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void IsolatedStoreAbsTimeExpiryAddRemoveCountDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemCache.Remove("1");

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(7, "Toy7", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("3");

            Assert.AreEqual(5, itemCache.Count);

            itemCache.Remove("5");
            itemCache.Remove("4");

            Assert.AreEqual(3, itemCache.Count);

            itemCache.Remove("2");
            itemCache.Remove("4");
            Assert.AreEqual(2, itemCache.Count);
        }

        [TestMethod]
        public void IsolatedStoreAbsTimeExpiryContainsDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            Assert.AreEqual(true, itemCache.Contains("1"));
            Assert.AreEqual(true, itemCache.Contains("2"));
            Assert.AreEqual(true, itemCache.Contains("3"));
            Assert.AreEqual(true, itemCache.Contains("4"));
            Assert.AreEqual(true, itemCache.Contains("5"));
            Assert.AreEqual(true, itemCache.Contains("6"));
            Assert.AreEqual(false, itemCache.Contains("7"));


        }

        [TestMethod]
        public void IsolatedStoreAbsTimeExpiryAddNullObjectDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        public void IsolatedStoreAbsTimeExpiryAddNullValueDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, null, 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreAbsTimeExpiryCheckContainsNullDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual(false, itemCache.Contains(null));
        }

        #endregion

        #region "Sliding Time Expiry, Add overload"
        [TestMethod]
        public void IsolatedStoreSlidingTimeExpiryAddDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void IsolatedStoreSlidingTimeExpiryAddRemoveDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("1");
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

            itemCache.Remove("3");
            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.IsNull(writtenItem);

            itemCache.Remove("4");
            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.IsNull(writtenItem);

            itemCache.Remove("5");
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.IsNull(writtenItem);

            itemCache.Remove("6");
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

            Assert.AreEqual(0, itemCache.Count);
        }

        [TestMethod]
        public void IsolatedStoreSlidingTimeExpiryAddDupKeyDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(1, "Toy11", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy11", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);
            Assert.AreEqual(1, itemCache.Count);
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreSlidingTimeExpiryAddNullKeyDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(null, itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreSlidingTimeExpiryReadNullKeyDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            writtenItem = (ItemDetails)itemCache.GetData(null);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreSlidingTimeExpiryRemoveNullKeyDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemCache.Remove(null);

        }

        [TestMethod]
        public void IsolatedStoreSlidingTimeExpiryNonExistKeyDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void IsolatedStoreSlidingTimeExpiryAddRemoveCountDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemCache.Remove("1");

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(7, "Toy7", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("3");

            Assert.AreEqual(5, itemCache.Count);

            itemCache.Remove("5");
            itemCache.Remove("4");

            Assert.AreEqual(3, itemCache.Count);

            itemCache.Remove("2");
            itemCache.Remove("4");
            Assert.AreEqual(2, itemCache.Count);
        }

        [TestMethod]
        public void IsolatedStoreSlidingTimeExpiryContainsDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(20)));

            Assert.AreEqual(true, itemCache.Contains("1"));
            Assert.AreEqual(true, itemCache.Contains("2"));
            Assert.AreEqual(true, itemCache.Contains("3"));
            Assert.AreEqual(true, itemCache.Contains("4"));
            Assert.AreEqual(true, itemCache.Contains("5"));
            Assert.AreEqual(true, itemCache.Contains("6"));
            Assert.AreEqual(false, itemCache.Contains("7"));

        }

        [TestMethod]
        public void IsolatedStoreSlidingTimeExpiryAddNullObjectDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        public void IsolatedStoreSlidingTimeExpiryAddNullValueDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, null, 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreSlidingTimeExpiryCheckContainsNullDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(20)));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual(false, itemCache.Contains(null));
        }

        #endregion

        #region "File Dependency Expiry, Add overload"
        [TestMethod]
        public void IsolatedStoreFileExpiryAddDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCaching.txt"));

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void IsolatedStoreFileExpiryAddRemoveDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCaching.txt"));

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("1");
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

            itemCache.Remove("3");
            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.IsNull(writtenItem);

            itemCache.Remove("4");
            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.IsNull(writtenItem);

            itemCache.Remove("5");
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.IsNull(writtenItem);

            itemCache.Remove("6");
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

            Assert.AreEqual(0, itemCache.Count);
        }

        [TestMethod]
        public void IsolatedStoreFileExpiryAddDupKeyDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(1, "Toy11", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new FileDependency("FileForCaching.txt"));

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy11", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);
            Assert.AreEqual(1, itemCache.Count);
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreFileExpiryAddNullKeyDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(null, itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreFileExpiryReadNullKeyDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));

            writtenItem = (ItemDetails)itemCache.GetData(null);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreFileExpiryRemoveNullKeyDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));

            itemCache.Remove(null);

        }

        [TestMethod]
        public void IsolatedStoreFileExpiryNonExistKeyDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void IsolatedStoreFileExpiryAddRemoveCountDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCaching.txt"));

            itemCache.Remove("1");

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCaching.txt"));

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(7, "Toy7", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCaching.txt"));

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("3");

            Assert.AreEqual(5, itemCache.Count);

            itemCache.Remove("5");
            itemCache.Remove("4");

            Assert.AreEqual(3, itemCache.Count);

            itemCache.Remove("2");
            itemCache.Remove("4");
            Assert.AreEqual(2, itemCache.Count);
        }

        [TestMethod]
        public void IsolatedStoreFileExpiryContainsDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCaching.txt"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCaching.txt"));

            Assert.AreEqual(true, itemCache.Contains("1"));
            Assert.AreEqual(true, itemCache.Contains("2"));
            Assert.AreEqual(true, itemCache.Contains("3"));
            Assert.AreEqual(true, itemCache.Contains("4"));
            Assert.AreEqual(true, itemCache.Contains("5"));
            Assert.AreEqual(true, itemCache.Contains("6"));
            Assert.AreEqual(false, itemCache.Contains("7"));


        }

        [TestMethod]
        public void IsolatedStoreFileExpiryAddNullObjectDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        public void IsolatedStoreFileExpiryAddNullValueDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, null, 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreFileExpiryCheckContainsNullDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual(false, itemCache.Contains(null));
        }

        #endregion

        #region "Extended Format, Add overload"
        [TestMethod]
        public void IsolatedStoreExtFormatExpiryAddDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void IsolatedStoreExtFormatExpiryAddRemoveDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("1");
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

            itemCache.Remove("3");
            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.IsNull(writtenItem);

            itemCache.Remove("4");
            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.IsNull(writtenItem);

            itemCache.Remove("5");
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.IsNull(writtenItem);

            itemCache.Remove("6");
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

            Assert.AreEqual(0, itemCache.Count);
        }

        [TestMethod]
        public void IsolatedStoreExtFormatExpiryAddDupKeyDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(1, "Toy11", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new ExtendedFormatTime("0 0 * * *"));


            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy11", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);
            Assert.AreEqual(1, itemCache.Count);
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreExtFormatExpiryAddNullKeyDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(null, itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreExtFormatExpiryReadNullKeyDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            writtenItem = (ItemDetails)itemCache.GetData(null);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreExtFormatExpiryRemoveNullKeyDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            itemCache.Remove(null);

        }

        [TestMethod]
        public void IsolatedStoreExtFormatExpiryNonExistKeyDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void IsolatedStoreExtFormatExpiryAddRemoveCountDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            itemCache.Remove("1");

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(7, "Toy7", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("3");

            Assert.AreEqual(5, itemCache.Count);

            itemCache.Remove("5");
            itemCache.Remove("4");

            Assert.AreEqual(3, itemCache.Count);

            itemCache.Remove("2");
            itemCache.Remove("4");
            Assert.AreEqual(2, itemCache.Count);
        }

        [TestMethod]
        public void IsolatedStoreExtFormatExpiryContainsDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));

            Assert.AreEqual(true, itemCache.Contains("1"));
            Assert.AreEqual(true, itemCache.Contains("2"));
            Assert.AreEqual(true, itemCache.Contains("3"));
            Assert.AreEqual(true, itemCache.Contains("4"));
            Assert.AreEqual(true, itemCache.Contains("5"));
            Assert.AreEqual(true, itemCache.Contains("6"));
            Assert.AreEqual(false, itemCache.Contains("7"));


        }

        [TestMethod]
        public void IsolatedStoreExtFormatExpiryAddNullObjectDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        public void IsolatedStoreExtFormatExpiryAddNullValueDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, null, 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreExtFormatExpiryCheckContainsNullDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual(false, itemCache.Contains(null));
        }

        #endregion

        #region "Never Expired, Add overload"
        [TestMethod]
        public void IsolatedStoreNeverExpiredAddDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, null);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, null);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, null);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void IsolatedStoreNeverExpiredAddRemoveDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, null);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, null);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, null);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy2", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.AreEqual("Toy3", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(17), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.AreEqual("Toy4", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(11), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("Toy5", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(33), writtenItem.Price);

            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.AreEqual("Toy6", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("1");
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

            itemCache.Remove("3");
            writtenItem = (ItemDetails)itemCache.GetData("3");
            Assert.IsNull(writtenItem);

            itemCache.Remove("4");
            writtenItem = (ItemDetails)itemCache.GetData("4");
            Assert.IsNull(writtenItem);

            itemCache.Remove("5");
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.IsNull(writtenItem);

            itemCache.Remove("6");
            writtenItem = (ItemDetails)itemCache.GetData("6");
            Assert.IsNull(writtenItem);

            Assert.AreEqual(0, itemCache.Count);
        }

        [TestMethod]
        public void IsolatedStoreNeverExpiredAddDupKeyDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemDetails = new ItemDetails(1, "Toy11", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, null);

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy11", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(13), writtenItem.Price);
            Assert.AreEqual(1, itemCache.Count);
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreNeverExpiredAddNullKeyDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(null, itemDetails, CacheItemPriority.High, null, null);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreNeverExpiredReadNullKeyDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            writtenItem = (ItemDetails)itemCache.GetData(null);

        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreNeverExpiredRemoveNullKeyDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemCache.Remove(null);

        }

        [TestMethod]
        public void IsolatedStoreNeverExpiredNonExistKeyDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemCache.Remove("2");
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void IsolatedStoreNeverExpiredAddRemoveCountDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, null);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, null);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, null);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            itemCache.Remove("1");

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            Assert.AreEqual(5, itemCache.Count);

            itemDetails = new ItemDetails(7, "Toy7", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            Assert.AreEqual(6, itemCache.Count);

            itemCache.Remove("3");

            Assert.AreEqual(5, itemCache.Count);

            itemCache.Remove("5");
            itemCache.Remove("4");

            Assert.AreEqual(3, itemCache.Count);

            itemCache.Remove("2");
            itemCache.Remove("4");
            Assert.AreEqual(2, itemCache.Count);
        }

        [TestMethod]
        public void IsolatedStoreNeverExpiredContainsDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);

            itemDetails = new ItemDetails(2, "Toy2", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, null);

            itemDetails = new ItemDetails(3, "Toy3", 17);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, null);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, null);

            itemDetails = new ItemDetails(5, "Toy5", 33);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            itemDetails = new ItemDetails(6, "Toy6", 12);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);

            Assert.AreEqual(true, itemCache.Contains("1"));
            Assert.AreEqual(true, itemCache.Contains("2"));
            Assert.AreEqual(true, itemCache.Contains("3"));
            Assert.AreEqual(true, itemCache.Contains("4"));
            Assert.AreEqual(true, itemCache.Contains("5"));
            Assert.AreEqual(true, itemCache.Contains("6"));
            Assert.AreEqual(false, itemCache.Contains("7"));


        }

        [TestMethod]
        public void IsolatedStoreNeverExpiredAddNullObjectDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, null);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        public void IsolatedStoreNeverExpiredAddNullValueDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, null, 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);
            Assert.AreEqual(true, itemCache.Contains("1"));
        }

        [TestMethod]
        [ExpectedException(typeof(System.ArgumentException))]
        public void IsolatedStoreNeverExpiredCheckContainsNullDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), null, CacheItemPriority.High, null, null);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual(false, itemCache.Contains(null));
        }

        #endregion

        #region "Isolated Store with Encryption"
        [TestMethod]
        public void IsolatedStoreAbsTimeExpiryEncryptAddDicTest()
        {
            itemCache = factory.Create("IsolatedCacheManagerEncDic");
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(20)));

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);

            Assert.AreEqual(1, itemCache.Count);
            itemCache.Flush();
            Assert.AreEqual(0, itemCache.Count);

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

        }
        #endregion
    }
}
