using System;
using Microsoft.Practices.EnterpriseLibrary.Caching;
using Microsoft.Practices.EnterpriseLibrary.Caching.Expirations;
using System.IO;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
namespace CachingCoreTests
{
    /// <summary>
    /// Summary description for AddwithdiffcombiOfParameter
    /// </summary>
    [TestClass]
    public class DiffParametersTestFixture
    {
       private CacheManager itemCache;
        public DiffParametersTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        /// <summary>
        /// Initialize() is called once during test execution before
        /// test methods in this test class are executed.
        /// </summary>
        [TestInitialize()]
        public void Initialize()
        {
            itemCache = CacheFactory.GetCacheManager("DefaultCacheManager");
            if (itemCache != null)
                itemCache.Flush();
        }

        /// <summary>
        /// Cleanup() is called once during test execution after
        /// test methods in this class have executed unless the
        /// corresponding Initialize() call threw an exception.
        /// </summary>
        [TestCleanup()]
        public void Cleanup()
        {
            //  TODO: Add test cleanup code
        }

        #region "Combination of different priorities"

        [TestMethod]
        public void AddtoCacheLowPriorityDefaultTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "one", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("one", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("1"));
        }
        [TestMethod]
        public void AddtoCacheHighPriorityDefaultTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(2, "two", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);
            writtenItem = (ItemDetails)itemCache.GetData("2");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("two", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("2"));

        }
        [TestMethod]
        public void AddtoCacheNormalPriorityDefaultTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(3, "three", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);
            writtenItem = (ItemDetails)itemCache.GetData("3");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("three", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("3"));
        }
        [TestMethod]
        public void AddtoCacheNonePriorityDefaultTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(4, "four", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);
            writtenItem = (ItemDetails)itemCache.GetData("4");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("four", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("4"));

        }
        [TestMethod]
        public void AddtoCacheNotRemovablePriorityDefaultTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(5, "five", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails);
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("five", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("5"));
        }
        #endregion
        
        #region "Combination of different priorities and Absolutetime expiry"

        [TestMethod]
        public void AddtoCacheLowPriorityAbsoluteTimeTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "one", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(1)));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("one", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("1"));
        }
        [TestMethod]
        public void AddtoCacheHighPriorityAbsoluteTimeTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(2, "two", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));
            writtenItem = (ItemDetails)itemCache.GetData("2");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("two", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("2"));

        }
        [TestMethod]
        public void AddtoCacheNormalPriorityAbsoluteTimeTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(3, "three", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));
            writtenItem = (ItemDetails)itemCache.GetData("3");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("three", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("3"));
        }
        [TestMethod]
        public void AddtoCacheNonePriorityAbsoluteTimeTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(4, "four", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));
            writtenItem = (ItemDetails)itemCache.GetData("4");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("four", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("4"));

        }
        [TestMethod]
        public void AddtoCacheNotRemovablePriorityAbsoluteTimeTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(5, "five", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("five", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("5"));
        }
        #endregion

        #region "Combination of different priorities and slidingtime" 
        [TestMethod]
        public void AddtoCacheLowPrioritySlidingtimeTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "one", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new SlidingTime(TimeSpan.FromMinutes(2)));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("one", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("1"));
        }
        [TestMethod]
        public void AddtoCacheHighPrioritySlidingtimeTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(2, "two", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new SlidingTime(TimeSpan.FromMinutes(2)));
            writtenItem = (ItemDetails)itemCache.GetData("2");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("two", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("2"));
       }
        [TestMethod]
        public void AddtoCacheNormalPrioritySlidingtimeTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(3, "three", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new SlidingTime(TimeSpan.FromMinutes(2)));
            writtenItem = (ItemDetails)itemCache.GetData("3");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("three", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("3"));
        }
        [TestMethod]
        public void AddtoCacheNonePrioritySlidingtimeTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(4, "four", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new SlidingTime(TimeSpan.FromMinutes(2)));
            writtenItem = (ItemDetails)itemCache.GetData("4");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("four", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("4"));

        }
        [TestMethod]
        public void AddtoCacheNotRemovablePrioritySlidingtimeTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(5, "five", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new SlidingTime(TimeSpan.FromMinutes(2)));
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("five", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("5"));
        }
        #endregion

        #region "Combination of different priorities and File depenedency"

        [TestMethod]
        public void AddtoCacheLowPriorityFileDependencyTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "one", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new FileDependency("FileForCaching.txt"));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("one", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("1"));
        }
        [TestMethod]
        public void AddtoCacheHighPriorityFileDependencyTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(2, "two", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new FileDependency("FileForCaching.txt"));
            writtenItem = (ItemDetails)itemCache.GetData("2");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("two", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("2"));
        }
        [TestMethod]
        public void AddtoCacheNormalPriorityFileDependencyTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(3, "three", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new FileDependency("FileForCaching.txt"));
            writtenItem = (ItemDetails)itemCache.GetData("3");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("three", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("3"));
        }
        [TestMethod]
        public void AddtoCacheNonePriorityFileDependencyTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(4, "four", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new FileDependency("FileForCaching.txt"));
            writtenItem = (ItemDetails)itemCache.GetData("4");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("four", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("4"));

        }
        [TestMethod]
        public void AddtoCacheNotRemovablePriorityFileDependencyTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(5, "five", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new FileDependency("FileForCaching.txt"));
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("five", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("5"));
        }
        #endregion

        #region "Combination of different priorities and ExtendedFormat" 
        [TestMethod]
        public void AddtoCacheLowPriorityExtendedFormatTimeTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "one", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new ExtendedFormatTime("0 0 * * *"));
            writtenItem = (ItemDetails)itemCache.GetData("1");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("one", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("1"));
        }
        [TestMethod]
        public void AddtoCacheHighPriorityExtendedFormatTimeTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(2, "two", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new ExtendedFormatTime("0 0 * * *"));
            writtenItem = (ItemDetails)itemCache.GetData("2");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("two", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("2"));
        }
        [TestMethod]
        public void AddtoCacheNormalPriorityExtendedFormatTimeTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(3, "three", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new ExtendedFormatTime("0 0 * * *"));
            writtenItem = (ItemDetails)itemCache.GetData("3");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("three", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("3"));
        }
        [TestMethod]
        public void AddtoCacheNonePriorityExtendedFormatTimeTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(4, "four", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new ExtendedFormatTime("0 0 * * *"));
            writtenItem = (ItemDetails)itemCache.GetData("4");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("four", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("4"));

        }
        [TestMethod]
        public void AddtoCacheNotRemovablePriorityExtendedFormatTimeTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(5, "five", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new ExtendedFormatTime("0 0 * * *"));
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("five", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("5"));
        }

        #endregion

	    #region "Combination of different priorities and Never Expired Items"

        [TestMethod]
        public void AddtoCacheLowPriorityNeverExpiredTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "one", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, null);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("one", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("1"));
        }
        [TestMethod]
        public void AddtoCacheHighPriorityNeverExpiredTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(2, "two", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, null);
            writtenItem = (ItemDetails)itemCache.GetData("2");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("two", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("2"));

        }
        [TestMethod]
        public void AddtoCacheNormalPriorityNeverExpiredTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(3, "three", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, null);
            writtenItem = (ItemDetails)itemCache.GetData("3");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("three", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("3"));
        }
        [TestMethod]
        public void AddtoCacheNonePriorityNeverExpiredTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(4, "four", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, null);
            writtenItem = (ItemDetails)itemCache.GetData("4");
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            Assert.AreEqual("four", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("4"));

        }
        [TestMethod]
        public void AddtoCacheNotRemovablePriorityNeverExpiredTest()
        {
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(5, "five", 100);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, null);
            writtenItem = (ItemDetails)itemCache.GetData("5");
            Assert.AreEqual("five", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(100), writtenItem.Price);
            Assert.IsTrue(itemCache.Contains(writtenItem.ItemId.ToString()));
            int count = itemCache.Count;
            Assert.AreEqual(1, count);
            itemCache.Remove(itemDetails.ItemId.ToString());
            Assert.AreEqual(null, (ItemDetails)itemCache.GetData("5"));
        }
        #endregion

    }
}
