using System;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
using IgnoreTest = NUnit.Framework.IgnoreAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Caching;
using Microsoft.Practices.EnterpriseLibrary.Caching.Expirations;
using System.Threading;
using System.IO;
using System.Data;
using System.Data.SqlClient;
using Microsoft.Practices.EnterpriseLibrary.Common;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Caching.Configuration;

namespace CachingDBThreadWaitTests
{
    /// <summary>
    /// Summary description for ExpirationDBDicTestFixture
    /// </summary>
    [TestClass]
    public class ExpirationDBDicTestFixture
    {
        private CacheManager itemCache;
        private CacheManagerFactory factory;
        private DictionaryConfigurationSource configSource;
        string connStr;
        public ExpirationDBDicTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        /// <summary>
        /// Initialize() is called once during test execution before
        /// test methods in this test class are executed.
        /// </summary>
        [TestInitialize()]
        public void Initialize()
        {
            CachingDBDictionarySource sourceSection = new CachingDBDictionarySource();
            configSource = sourceSection.BuildDBDictionarySourceSection();
            
            connStr = "server=(local)\\SQLEXPRESS;initial catalog=caching;integrated security=SSPI";

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("begin delete from CacheData " +
                                            "end", conn);
            cmd.ExecuteNonQuery();

            conn.Close();

            factory = new CacheManagerFactory(configSource);
            itemCache = factory.Create("DBCacheManagerDic");
            if (itemCache != null)
                itemCache.Flush();
        }

        /// <summary>
        /// Cleanup() is called once during test execution after
        /// test methods in this class have executed unless the
        /// corresponding Initialize() call threw an exception.
        /// </summary>
        [TestCleanup()]
        public void Cleanup()
        {
            //  TODO: Add test cleanup code
        }

        #region "Expiration - DB Cache Manager"
        [TestMethod]
        public void DBAbsoluteTimeExpiryDicTest()
        {
            int count = 0;
            ItemDetails itemDetails = new ItemDetails(1, "Toy", 25);
            ItemDetails writtenItem = new ItemDetails();
            DateTime currentTime = DateTime.Now;
            DateTime newTime = new DateTime();
            newTime = currentTime.AddSeconds(5);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(newTime));

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);
            
            Thread.Sleep(6000);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(0, count);

            dr.Close();
            conn.Close();

        }

        [TestMethod]
        public void DBSlidingTimeExpiryDicTest()
        {
            int count = 0;
            ItemDetails itemDetails = new ItemDetails(2, "Toy1", 27);
            ItemDetails writtenItem = new ItemDetails();
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new SlidingTime(TimeSpan.FromSeconds(3)));

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(27), writtenItem.Price);

            //This is to show that cache will expire only if it is not accessed in the next two minutes
            Thread.Sleep(2500);
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Thread.Sleep(2500);
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNotNull(writtenItem);
            
            Thread.Sleep(4000);
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('2')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(0, count);

            dr.Close();
            conn.Close();

        }

        [TestMethod]
        public void DBFileDependencyExpiryDicTest()
        {
            int count = 0;
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 12);
            ItemDetails writtenItem = new ItemDetails();
            
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new FileDependency("FileForCachinginDB.txt"));

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            File.SetLastWriteTime("FileForCachinginDB.txt", DateTime.Now);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(0, count);

            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void DBNeverExpiredDicTest()
        {
            int count = 0;
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 12);
            ItemDetails writtenItem = new ItemDetails();
            

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, null);

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);


            Thread.Sleep(1000);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNotNull(writtenItem);
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(1, count);

            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void DBAbsoluteTimeExpiryReadDicTest()
        {
            int count = 0;
            ItemDetails itemDetails = new ItemDetails(1, "Toy", 25);
            ItemDetails writtenItem = new ItemDetails();
            DateTime currentTime = DateTime.Now;
            DateTime newTime = new DateTime();
            newTime = currentTime.AddSeconds(3);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(newTime));

            Thread.Sleep(4000);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);


            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(0, count);

            dr.Close();
            conn.Close();

        }

        [TestMethod]
        public void DBSlidingTimeExpiryReadDicTest()
        {
            int count = 0;
            ItemDetails itemDetails = new ItemDetails(2, "Toy1", 27);
            ItemDetails writtenItem = new ItemDetails();
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new SlidingTime(TimeSpan.FromSeconds(3)));

            Thread.Sleep(4000);
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);


            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('2')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(0, count);

            dr.Close();
            conn.Close();

        }

        [TestMethod]
        public void DBFileDependencyExpiryReadDicTest()
        {
            int count = 0;
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 12);
            ItemDetails writtenItem = new ItemDetails();
            
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new FileDependency("FileForCachinginDB.txt"));

            File.SetLastWriteTime("FileForCachinginDB.txt", DateTime.Now);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(0, count);

            dr.Close();
            conn.Close();
        }

        #endregion
    }
}
