using System;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Caching;
using Microsoft.Practices.EnterpriseLibrary.Caching.Expirations;
using System.IO;
using System.Threading;
using System.Data;
using System.Data.SqlClient;

namespace CachingDBThreadWaitTests
{
    /// <summary>
    /// Summary description for RefreshActionTestFixture
    /// </summary>
    [TestClass]
    public class RefreshActionTestFixture
    {
        public static bool removed = false;
        String connStr;
        private CacheManager itemCache;
        public RefreshActionTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        /// <summary>
        /// Initialize() is called once during test execution before
        /// test methods in this test class are executed.
        /// </summary>
        [TestInitialize()]
        public void Initialize()
        {
            connStr = "server=(local)\\SQLEXPRESS;initial catalog=caching;integrated security=SSPI";

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("begin delete from CacheData " +
                                            "end", conn);
            cmd.ExecuteNonQuery();

            conn.Close();

            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            if (itemCache != null)
                itemCache.Flush();
            
        }

        /// <summary>
        /// Cleanup() is called once during test execution after
        /// test methods in this class have executed unless the
        /// corresponding Initialize() call threw an exception.
        /// </summary>
        [TestCleanup()]
        public void Cleanup()
        {
            //  TODO: Add test cleanup code
        }
        #region "Refresh action for Expiration"

        [TestMethod]
        public void DBExpiryRefreshActionTest()
        {
            int count = 0;
            removed = false;
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, new ItemRefreshAction(), new FileDependency("FileForCachinginDB.txt"));
            Assert.AreEqual(false, removed);
            File.SetLastWriteTime("FileForCachinginDB.txt", DateTime.Now);
            Thread.Sleep(1500);
            Assert.AreEqual(true, removed); 
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);
            
            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('1')", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(0, count);

            dr.Close();
            conn.Close();

        }

        #endregion

        #region "Refresh action for Scavenging"
        [TestMethod]
        public void DBRefreshActionScavengingTest()
        {
            removed = false;
            ItemDetails writtenItem = new ItemDetails();
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, new ItemRefreshAction(), null);

            itemDetails = new ItemDetails(2, "Toy2", 22);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, new ItemRefreshAction(), null);

            itemDetails = new ItemDetails(3, "Toy3", 21);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, new ItemRefreshAction(), null);

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, new ItemRefreshAction(), null);

            itemDetails = new ItemDetails(5, "Toy5", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, new ItemRefreshAction(), null);

            itemDetails = new ItemDetails(6, "Toy6", 16);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, new ItemRefreshAction(), null);

            itemDetails = new ItemDetails(7, "Toy7", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, new ItemRefreshAction(), null);

            itemDetails = new ItemDetails(8, "Toy8", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, new ItemRefreshAction(), null);

            itemDetails = new ItemDetails(9, "Toy9", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, new ItemRefreshAction(), null);

            itemDetails = new ItemDetails(10, "Toy10", 21);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, new ItemRefreshAction(), null);

            Assert.AreEqual(false, removed);

            itemDetails = new ItemDetails(11, "Toy11", 23);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, new ItemRefreshAction(), null);

            Thread.Sleep(1000); 
            Assert.AreEqual(true, removed);

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

        }

        
        #endregion

        [Serializable]
        public class ItemRefreshAction : ICacheItemRefreshAction
        {
            public void Refresh(string removedKey, object expiredValue, CacheItemRemovedReason removalReason)
            {
                removed = true;

            }
        }
    }

   
}
