using System;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Caching;
using Microsoft.Practices.EnterpriseLibrary.Caching.Expirations;
using System.Data;
using System.Data.SqlClient;
using System.Xml;

namespace CachingDatabaseTests
{
    /// <summary>
    /// Summary description for CachingRuntimeTestFixture
    /// </summary>
    [TestClass]
    public class CachingRuntimeTestFixture
    {
        String connStr;
        private CacheManager itemCache;
        public CachingRuntimeTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        [TestInitialize()]
        public void Initialize()
        {
            connStr = "server=(local)\\SQLEXPRESS;initial catalog=caching;integrated security=SSPI";

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("begin delete from CacheData " +
                                            "end", conn);
            cmd.ExecuteNonQuery();

            conn.Close();

            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            if (itemCache != null)
                itemCache.Flush();

        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        // Caching does not detect changes to runtime.
        [TestMethod]
        public void RuntimeChangesTest()
        {
            CacheManager itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            Assert.IsNotNull(itemCache);
            UpdatePartitionName("Partition2"); 
            System.Threading.Thread.Sleep(15000);
            itemCache = CacheFactory.GetCacheManager("DBCacheManager");
            itemCache.Add("25", "testing");

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();
            int count = 0;
            SqlCommand cmd = new SqlCommand("select * from CacheData where [key] in ('25') and partitionname = 'Partition1'", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                count = count + 1;
            }
            Assert.AreEqual(1, count);
            dr.Close();
            conn.Close();
            UpdatePartitionName("Partition1"); 
        }

        private void UpdatePartitionName(string partitionName)
        {
            string configurationFile = "CachingDatabaseTests.dll.config";
            XmlDocument doc = new XmlDocument();
            doc.Load(configurationFile);
            XmlNodeList nodeList = doc.SelectNodes("/configuration/cachingConfiguration/backingStores/add");
            foreach (XmlNode xNode in nodeList)
            {
                if (((XmlAttribute)xNode.Attributes["name"]).Value == "Data Cache Storage")
                {
                    ((XmlAttribute)xNode.Attributes["partitionName"]).Value = partitionName;
                    break;
                }
            }
            doc.Save(configurationFile);
        }
    }
}
