using System;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Caching;
using Microsoft.Practices.EnterpriseLibrary.Caching.Expirations;
using System.Threading;
using System.IO;

namespace CachingThreadWaitTests
{
    /// <summary>
    /// Summary description for DefaultStoreExpirationTest
    /// </summary>
    [TestClass]
    public class ExpirationTestFixture
    {
        private CacheManager itemCache;
        public ExpirationTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        /// <summary>
        /// Initialize() is called once during test execution before
        /// test methods in this test class are executed.
        /// </summary>
        [TestInitialize()]
        public void Initialize()
        {
            itemCache = CacheFactory.GetCacheManager("DefaultCacheManager");
            if (itemCache != null)
                itemCache.Flush();

        }

        /// <summary>
        /// Cleanup() is called once during test execution after
        /// test methods in this class have executed unless the
        /// corresponding Initialize() call threw an exception.
        /// </summary>
        [TestCleanup()]
        public void Cleanup()
        {
            //  TODO: Add test cleanup code
        }
        #region "Expiration - Default Cache Manager"
        [TestMethod]
        public void AbsoluteTimeExpiryTest()
        {
            ItemDetails itemDetails = new ItemDetails(1, "Toy", 25);
            ItemDetails writtenItem = new ItemDetails();
            DateTime currentTime = DateTime.Now;
            DateTime newTime = new DateTime();
            newTime = currentTime.AddSeconds(5);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(newTime));

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(25), writtenItem.Price);
            

            Thread.Sleep(6000);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void SlidingTimeExpiryTest()
        {

            ItemDetails itemDetails = new ItemDetails(2, "Toy1", 27);
            ItemDetails writtenItem = new ItemDetails();
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new SlidingTime(TimeSpan.FromSeconds(3)));

            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(27), writtenItem.Price);

            //This is to show that cache will expire only if it is not accessed in the next two minutes
            Thread.Sleep(2500);
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Thread.Sleep(2500);
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNotNull(writtenItem); 
            
            Thread.Sleep(4000);
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void FileDependencyExpiryTest()
        {
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 12);
            ItemDetails writtenItem = new ItemDetails();

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new FileDependency("FileForCaching.txt"));

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

            File.SetLastWriteTime("FileForCaching.txt", DateTime.Now);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void NeverExpiredTest()
        {
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 12);
            ItemDetails writtenItem = new ItemDetails();
            DateTime currentTime = DateTime.Now;
            DateTime newTime = new DateTime();
            newTime = currentTime.AddSeconds(1);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, null);

            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);


            Thread.Sleep(2000);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNotNull(writtenItem);
            Assert.AreEqual("Toy1", writtenItem.Name);
            Assert.AreEqual(Convert.ToDecimal(12), writtenItem.Price);

        }


        [TestMethod]
        public void AbsoluteTimeExpiryReadTest()
        {
            ItemDetails itemDetails = new ItemDetails(1, "Toy", 25);
            ItemDetails writtenItem = new ItemDetails();
            DateTime currentTime = DateTime.Now;
            DateTime newTime = new DateTime();
            newTime = currentTime.AddSeconds(3);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(newTime));

            Thread.Sleep(4000);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void SlidingTimeExpiryReadTest()
        {
            ItemDetails itemDetails = new ItemDetails(2, "Toy1", 27);
            ItemDetails writtenItem = new ItemDetails();
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new SlidingTime(TimeSpan.FromSeconds(3)));

            Thread.Sleep(4000);
            writtenItem = (ItemDetails)itemCache.GetData("2");
            Assert.IsNull(writtenItem);

        }

        [TestMethod]
        public void FileDependencyExpiryReadTest()
        {
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 12);
            ItemDetails writtenItem = new ItemDetails();
            DateTime currentTime = DateTime.Now;
            DateTime newTime = new DateTime();
            newTime = currentTime.AddMinutes(2);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new FileDependency("FileForCaching.txt"));

            File.SetLastWriteTime("FileForCaching.txt", DateTime.Now);
            writtenItem = (ItemDetails)itemCache.GetData("1");
            Assert.IsNull(writtenItem);

        }


        #endregion


    }
}
