using System;
using System.Collections.Generic;
using System.Text;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Security.Cryptography;
using Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
namespace CryptoFuncTests
{
	class CryptoHelper
	{
		public static byte[] Encrypt(string instanceName,object configSource, byte[] dataToEncrypt)
		{
			if(configSource != null)
			{
				SymmetricCryptoProviderFactory factory = new SymmetricCryptoProviderFactory((IConfigurationSource)configSource);
				ISymmetricCryptoProvider provider = (ISymmetricCryptoProvider)factory.Create(instanceName);
				return provider.Encrypt(dataToEncrypt);   
			}
			return Cryptographer.EncryptSymmetric(instanceName, dataToEncrypt);    
		}
		public static byte[] Decrypt(string instanceName, object configSource, byte[] dataToDecrypt)
		{
			if (configSource != null)
			{
                SymmetricCryptoProviderFactory factory = new SymmetricCryptoProviderFactory((IConfigurationSource)configSource);
				ISymmetricCryptoProvider provider = (ISymmetricCryptoProvider)factory.Create(instanceName);
				return provider.Decrypt(dataToDecrypt);
			}
			return Cryptographer.DecryptSymmetric(instanceName, dataToDecrypt);    
		}
        
		public static string Encrypt(string instanceName,string dataToEncrypt)
		{
			return Cryptographer.EncryptSymmetric (instanceName, dataToEncrypt);
		}
		public static string Decrypt(string instanceName, string dataToDecrypt)
		{
			return Cryptographer.DecryptSymmetric(instanceName, dataToDecrypt);
                      
		}
        public static string CreateHash(string instanceName, string dataToHash)
        {
            //if (configsource != null)
            //{
            //    HashProviderFactory factory = new HashProviderFactory();
            //    IHashProvider provider = (IHashProvider)factory.CreateObject(instanceName, configsource);
            //    provider.CreateHash(dataToHash);
            //}
            return Cryptographer.CreateHash(instanceName, dataToHash);
        }
        public static byte[] CreateHash(string instanceName, object configsource, byte[] dataToHash)
        {
            if (configsource != null)
            {
                HashProviderFactory factory = new HashProviderFactory((IConfigurationSource)configsource);
                IHashProvider provider = (IHashProvider)factory.Create(instanceName);
                return provider.CreateHash(dataToHash);
            }
            return Cryptographer.CreateHash(instanceName, dataToHash);
            
        }
        public static bool CompareHash(string instanceName,object configsource, byte[] plainText, byte[] hashedText)
        {
            if (configsource != null)
            {
                HashProviderFactory factory = new HashProviderFactory((IConfigurationSource)configsource);
                IHashProvider provider = (IHashProvider)factory.Create(instanceName);
                return provider.CompareHash(plainText, hashedText);
            }
            return Cryptographer.CompareHash(instanceName, plainText, hashedText);
        }
        public static bool CompareHash(string instancename,  string plaintext, string hashedtext)
        {
            //if (configsource != null)
            //{
            //    HashProviderFactory factory = new HashProviderFactory();
            //    IHashProvider provider = (IHashProvider)factory.CreateObject(instanceName, new DictionaryConfigurationSource((Dictionary<string, object>)configsource));
            //    return provider.CompareHash(plainText, hashedText);
            //}
            return Cryptographer.CompareHash(instancename, plaintext, hashedtext);
        }
	}
}
