using System;
using System.Collections.Generic;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Security.Cryptography;
using Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using System.Configuration;
using System.Security.Cryptography;
namespace CryptoFuncTests
{
    /// <summary>
    /// Summary description for CryptographerHashTestFixture
    /// Test using different hash providers for creating and comparing hash, returns the comparison false. 
    /// Use CreateHash method for creating hash for data in form of string or byte array, using various Hash and Keyed Hash Algorithms. With salt
    /// Use CompareHash method for comparing data in form of string or byte array with existing hash, using various Hash and Keyed Hash Algorithms. With Salt 
    /// Use CompareHash method for comparing data in form of string or byte array with existing hash, using various Hash and Keyed Hash Algorithms. Without Salt 
    /// </summary>
    [TestClass]
    public class CryptographerHashTestFixtureDictionarySource
    {
        private DictionaryConfigurationSource configSource = new DictionaryConfigurationSource();
        private byte[] PlainByteMessage;
        private String PlainStrMessage;
        private const String HMACMD5KeyedHashNoSalt = "HMACMD5KeyedHashNoSalt";
        private const String HMACMD5KeyedHash = "HMACMD5KeyedHash";
        private const String SHA1ManagedNoSalt = "SHA1ManagedNoSalt";
        private const String SHA1Managed = "SHA1Managed";
        private const String DPAPISymmetricCryptographyProvider = "DPAPISymmetricCryptographyProvider";
        public CryptographerHashTestFixtureDictionarySource()
        {
            String PlainStrMessageNotInBase64 = "This is Plain Message";
            PlainByteMessage = System.Text.Encoding.Unicode.GetBytes(PlainStrMessageNotInBase64);
            PlainStrMessage = System.Convert.ToBase64String(PlainByteMessage);
        }
        [TestInitialize()]
        public void Initialize()
        {
            configSource = BuildSymmetrySourceSection();
        }

        #region TestCases for hashing
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestHashingtForNullCryptoInstance()
        {
            CryptoHelper.CreateHash((String)null, configSource, PlainByteMessage);
        }
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestHashingForEmptyCryptoInstance()
        {
            CryptoHelper.CreateHash(String.Empty, configSource, PlainByteMessage);
        }
        [TestMethod]
        [ExpectedException(typeof(System.NullReferenceException))]
        public void TestHashingForNullPlainMessageInByte()
        {
            CryptoHelper.CreateHash(SHA1Managed, configSource, (byte[])null);
        }
        
        [TestMethod]
        [ExpectedException(typeof(System.Configuration.ConfigurationErrorsException))]
        public void TestHahingForCryptoInstanceNotPresentInConfig()
        {
             CryptoHelper.CreateHash("InstanceNotpresent", configSource, PlainByteMessage);
        }
        #endregion

        # region Tests for CompareHash
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCompareHashForNullHashInstance()
        {
            CryptoHelper.CompareHash(null, configSource, PlainByteMessage, Cryptographer.CreateHash(SHA1Managed, PlainByteMessage));
        }
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCompareHashForEmptyCryptoInstance()
        {
            CryptoHelper.CompareHash(String.Empty, configSource, PlainByteMessage, Cryptographer.CreateHash(SHA1Managed, PlainByteMessage));
        }
        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestCompareHashForNullPlainMessageInByte()
        {
            CryptoHelper.CompareHash(SHA1Managed, configSource, (byte[])null, PlainByteMessage);
        }
        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestCompareHashForNullPlainMessage()
        {
            CryptoHelper.CompareHash(SHA1Managed, (String)null, PlainStrMessage);
        }
        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestCompareHashForNullHashMessage()
        {
            CryptoHelper.CompareHash(SHA1Managed, configSource, PlainByteMessage, (byte[])null);
        }
        [TestMethod]
        public void TestCompareHashForValidInput()
        {
            byte[] hash = CryptoHelper.CreateHash(SHA1Managed, configSource, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash(SHA1Managed, configSource, PlainByteMessage, hash) == true);
        }
        #endregion

        # region Tests for all the Hash Providers
        [TestMethod]
        public void TestHashingForCryptoSHA512ManagedWithNoSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("SHA512ManagedWithNoSalt", configSource, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("SHA512ManagedWithNoSalt", configSource, PlainByteMessage, hash));
        }
        [TestMethod]
        public void TestHashingForCryptoSHA512ManagedWithSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("SHA512ManagedWithSalt", configSource, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("SHA512ManagedWithSalt", configSource, PlainByteMessage, hash));
        }
        [TestMethod]
        public void TestHashingForCryptoSHA384ManagedWithNoSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("SHA384ManagedWithNoSalt", configSource, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("SHA384ManagedWithNoSalt", configSource, PlainByteMessage, hash));
        }
        [TestMethod]
        public void TestHashingForCryptoSHA384ManagedWithSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("SHA384ManagedWithSalt", configSource, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("SHA384ManagedWithSalt", configSource, PlainByteMessage, hash));
        }
        [TestMethod]
        public void TestHashingForCryptoSHA256ManagedWithNoSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("SHA256ManagedWithNoSalt", configSource, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("SHA256ManagedWithNoSalt", configSource, PlainByteMessage, hash));
        }
        [TestMethod]
        public void TestHashingForCryptoSHA256ManagedWithSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("SHA256ManagedWithSalt", configSource, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("SHA256ManagedWithSalt", configSource, PlainByteMessage, hash));
        }
        [TestMethod]
        public void TestHashingForCryptoSHA1ManagedWithNoSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("SHA1ManagedWithNoSalt", configSource, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("SHA1ManagedWithNoSalt", configSource, PlainByteMessage, hash));
        }
        [TestMethod]
        public void TestHashingForCryptoSHA1ManagedWithSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("SHA1ManagedWithSalt", configSource, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("SHA1ManagedWithSalt", configSource, PlainByteMessage, hash));
        }
        [TestMethod]
        public void TestHashingForCryptoRIPEMD160ManagedWithNoSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("RIPEMD160ManagedWithNoSalt", configSource, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("RIPEMD160ManagedWithNoSalt", configSource, PlainByteMessage, hash));
        }
        [TestMethod]
        public void TestHashingForCryptoRIPEMD160ManagedWithSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("RIPEMD160ManagedWithSalt", configSource, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("RIPEMD160ManagedWithSalt", configSource, PlainByteMessage, hash));
        }
        [TestMethod]
        public void TestHashingForCryptoMD5CryptoServiceProviderWithNoSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("MD5CryptoServiceProviderWithNoSalt", configSource, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("MD5CryptoServiceProviderWithNoSalt", configSource, PlainByteMessage, hash));
        }
        [TestMethod]
        public void TestHashingForCryptoMD5CryptoServiceProviderWithSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("MD5CryptoServiceProviderWithSalt", configSource, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("MD5CryptoServiceProviderWithSalt", configSource, PlainByteMessage, hash));
        }
        [TestMethod]
        public void TestHashingForCryptoMACTripleDESWithNoSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("MACTripleDESWithNoSalt", configSource, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("MACTripleDESWithNoSalt", configSource, PlainByteMessage, hash));
        }
        [TestMethod]
        public void TestHashingForCryptoMACTripleDESWithSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("MACTripleDESWithSalt", configSource, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("MACTripleDESWithSalt", configSource, PlainByteMessage, hash));
        }
        [TestMethod]
        public void TestHashingForCryptoHMACSHA512WithNoSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("HMACSHA512WithNoSalt", configSource, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("HMACSHA512WithNoSalt", configSource, PlainByteMessage, hash));
        }
        [TestMethod]
        public void TestHashingForCryptoHMACSHA512WithSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("HMACSHA512WithSalt", configSource, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("HMACSHA512WithSalt", configSource, PlainByteMessage, hash));
        }
        [TestMethod]
        public void TestHashingForCryptoHMACSHA384WithNoSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("HMACSHA384WithNoSalt", configSource, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("HMACSHA384WithNoSalt", configSource, PlainByteMessage, hash));
        }
        [TestMethod]
        public void TestHashingForCryptoHMACSHA384WithSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("HMACSHA384WithSalt", configSource, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("HMACSHA384WithSalt", configSource, PlainByteMessage, hash));
        }
        [TestMethod]
        public void TestHashingForCryptoHMACSHA256WithNoSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("HMACSHA256WithNoSalt", configSource, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("HMACSHA256WithNoSalt", configSource, PlainByteMessage, hash));
        }
        [TestMethod]
        public void TestHashingForCryptoHMACSHA256WithSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("HMACSHA256WithSalt", configSource, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("HMACSHA256WithSalt", configSource, PlainByteMessage, hash));
        }
        [TestMethod]
        public void TestHashingForCryptoHMACMD5WithNoSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("HMACMD5WithNoSalt", configSource, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("HMACMD5WithNoSalt", configSource, PlainByteMessage, hash));
        }
        [TestMethod]
        public void TestHashingForCryptoHMACMD5WithSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("HMACMD5WithSalt", null, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("HMACMD5WithSalt", null, PlainByteMessage, hash));
        }
        [TestMethod]
        public void TestHashingForCryptoHMACRIPEMD160WithNoSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("HMACRIPEMD160WithNoSalt", configSource, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("HMACRIPEMD160WithNoSalt", configSource, PlainByteMessage, hash));
        }
        [TestMethod]
        public void TestHashingForCryptoHMACRIPEMD160WithSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("HMACRIPEMD160WithSalt", configSource, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("HMACRIPEMD160WithSalt", configSource, PlainByteMessage, hash));
        }
       
        [TestMethod]
        public void TestHashingForCryptoHMACSHA1WithNoSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("HMACSHA1WithNoSalt",configSource, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("HMACSHA1WithNoSalt",configSource, PlainByteMessage, hash));
        }
        [TestMethod]
        public void TestHashingForCryptoHMACSHA1WithSalt()
        {
            byte[] hash = CryptoHelper.CreateHash("HMACSHA1WithSalt",configSource, PlainByteMessage);
            Assert.IsTrue(CryptoHelper.CompareHash("HMACSHA1WithSalt",configSource, PlainByteMessage, hash));
        }  
         
        #endregion

        # region  Testing CreateHash and Compare method for creating hash for data in form of  byte array, using various Hash and Keyed Hash Algorithms. With salt
        
        
            [TestMethod]
            public void TestingCreateHashCompareHashMethodUsingDataInTheFormOfBytesUsingVariousAlgorithms()
            {
                byte[] Hash1 = CryptoHelper.CreateHash(SHA1Managed,null, PlainByteMessage);
                byte[] Hash2 = CryptoHelper.CreateHash(SHA1ManagedNoSalt,null, Hash1);
                byte[] Hash3 = CryptoHelper.CreateHash(HMACMD5KeyedHash,null, Hash2);
                byte[] Hash4 = CryptoHelper.CreateHash(HMACMD5KeyedHashNoSalt,null, Hash3);
                // Test for CreateHash
                Assert.IsFalse(this.CompareByteArrays(Hash1, PlainByteMessage));
                Assert.IsFalse(this.CompareByteArrays(Hash2, Hash1));
                Assert.IsFalse(this.CompareByteArrays(Hash3, Hash2));
                Assert.IsFalse(this.CompareByteArrays(Hash4, Hash3));
            }
            [TestMethod]
            public void TestCompareHashMethodUsingDataInTheFormOfBytesUsingVariousAlgorithmsForValidComparision()
            {
                byte[] Hash1 = CryptoHelper.CreateHash(SHA1Managed,configSource, PlainByteMessage);
                byte[] Hash2 = CryptoHelper.CreateHash(SHA1ManagedNoSalt,configSource, Hash1);
                byte[] Hash3 = CryptoHelper.CreateHash(HMACMD5KeyedHash,configSource, Hash2);
                byte[] Hash4 = CryptoHelper.CreateHash(HMACMD5KeyedHashNoSalt,configSource, Hash3);
                Assert.IsTrue(CryptoHelper.CompareHash(SHA1Managed,configSource, PlainByteMessage, Hash1));
                Assert.IsTrue(CryptoHelper.CompareHash(SHA1ManagedNoSalt,configSource, Hash1, Hash2));
                Assert.IsTrue(CryptoHelper.CompareHash(HMACMD5KeyedHash,configSource, Hash2, Hash3));
                Assert.IsTrue(CryptoHelper.CompareHash(HMACMD5KeyedHashNoSalt,configSource, Hash3, Hash4));
            }
            //In this test MessageDigest is  computed with one Method And Compared Using Another Method
            [TestMethod]
            public void TestCompareHashMethodUsingDataInTheFormOfBytesUsingVariousAlgorithmsForInValidComparision()
            {
                byte[] Hash1 = CryptoHelper.CreateHash(SHA1Managed,null, PlainByteMessage);
                byte[] Hash2 = CryptoHelper.CreateHash(SHA1ManagedNoSalt,null, Hash1);
                byte[] Hash3 = CryptoHelper.CreateHash(HMACMD5KeyedHash,null, Hash2);
                byte[] Hash4 = CryptoHelper.CreateHash(HMACMD5KeyedHashNoSalt,null, Hash3);
                Assert.IsFalse(Cryptographer.CompareHash(HMACMD5KeyedHashNoSalt, PlainByteMessage, Hash1));
                Assert.IsFalse(Cryptographer.CompareHash(HMACMD5KeyedHash, Hash1, Hash2));
                Assert.IsFalse(Cryptographer.CompareHash(SHA1ManagedNoSalt, Hash2, Hash3));
                Assert.IsFalse(Cryptographer.CompareHash(SHA1Managed, Hash3, Hash4));
            
            }   
        # endregion
        
        # region Hash Provider Default Test
        [TestMethod]
        [ExpectedException(typeof(ConfigurationErrorsException))]
        public void NullSectionDictionarySourceTest()
        {
          //  try
           // {
                
                DictionaryConfigurationSource DictSource = new DictionaryConfigurationSource();
                CryptographySettings settings = null;
                DictSource.Add("securityCryptographyConfiguration", settings);
                HashProviderFactory factory = new HashProviderFactory(DictSource);
                IHashProvider provider = (IHashProvider)factory.Create(SHA1Managed);
          //  }
            //catch (InvalidOperationException e)
            //{
            //    Assert.IsInstanceOfType(e.InnerException, typeof(System.Configuration.ConfigurationErrorsException));
            //}
        }
        [TestMethod]
       [ExpectedException(typeof(ArgumentNullException))]
        public void NullDictionarySource()
        {
         //   try
         //   {
            HashProviderFactory factory = new HashProviderFactory((IConfigurationSource)null);
                IHashProvider provider = (IHashProvider)factory.Create(SHA1Managed);
        //    }
         //   catch (InvalidOperationException e)
        //    {
               // Assert.IsInstanceOfType(e.InnerException, typeof(ArgumentNullException));
          //  }
        }
        [TestMethod]
        public void TestDefaultInstance()
        {
            DictionaryConfigurationSource DictSource = new DictionaryConfigurationSource();
            CryptographySettings setting = new CryptographySettings();
            setting.DefaultHashProviderName = SHA1Managed;
            HashAlgorithmProviderData Sha1Provider = new HashAlgorithmProviderData();
            Sha1Provider.Name = SHA1Managed;
            Sha1Provider.AlgorithmType = Type.GetType("System.Security.Cryptography.SHA1Managed, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            Sha1Provider.Type = typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.HashAlgorithmProvider);
            Sha1Provider.SaltEnabled = false;
            setting.HashProviders.Add(Sha1Provider);
            DictSource.Add("securityCryptographyConfiguration", setting);
          
            HashProviderFactory factory = new HashProviderFactory(DictSource);
            IHashProvider provider = (IHashProvider)factory.Create(SHA1Managed );
            IHashProvider providerDefault = (IHashProvider)factory.CreateDefault();
            byte[] HashedText = providerDefault.CreateHash(PlainByteMessage);
            Assert.IsTrue(true==provider.CompareHash(PlainByteMessage,HashedText));
        }
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDefaultInstanceWhichIsNotSet()
        {
            DictionaryConfigurationSource DictSource = new DictionaryConfigurationSource();
            CryptographySettings setting = new CryptographySettings();
         // Defaul instance not set   setting.DefaultHashProviderName = SHA1Managed;
            HashAlgorithmProviderData Sha1Provider = new HashAlgorithmProviderData();
            Sha1Provider.Name = SHA1Managed;
            Sha1Provider.AlgorithmType = Type.GetType("System.Security.Cryptography.SHA1Managed, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            Sha1Provider.Type = typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.HashAlgorithmProvider);
            Sha1Provider.SaltEnabled = false;
            setting.HashProviders.Add(Sha1Provider);
            DictSource.Add("securityCryptographyConfiguration", setting);
            HashProviderFactory factory = new HashProviderFactory(DictSource);
            IHashProvider provider = (IHashProvider)factory.Create(SHA1Managed );
            IHashProvider providerDefault = (IHashProvider)factory.CreateDefault();
            byte[] HashedText = providerDefault.CreateHash(PlainByteMessage);
            Assert.IsTrue(true == provider.CompareHash(PlainByteMessage, HashedText));
        
        }
        //[TestMethod]
        //[ExpectedException(typeof(ArgumentException))]
        //public void TestDefaultInstanceWhichIsSetToNull()
        //{
        //    DictionaryConfigurationSource DictSource = new DictionaryConfigurationSource();
        //    CryptographySettings setting = new CryptographySettings();
        //    setting.DefaultHashProviderName = null;
        //    HashAlgorithmProviderData Sha1Provider = new HashAlgorithmProviderData();
        //    Sha1Provider.Name = SHA1Managed;
        //    Sha1Provider.AlgorithmType = Type.GetType("System.Security.Cryptography.SHA1Managed, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
        //    Sha1Provider.Type = typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.HashAlgorithmProvider);
        //    Sha1Provider.SaltEnabled = false;
        //    setting.HashProviders.Add(Sha1Provider);
        //    DictSource.Add("securityCryptographyConfiguration", setting);
        //    HashProviderFactory factory = new HashProviderFactory();
        //    IHashProvider provider = (IHashProvider)factory.Create(Sha1Provider,DictSource);
        //    IHashProvider providerDefault = (IHashProvider)factory.CreateDefault(DictSource);
        //    byte[] HashedText = providerDefault.CreateHash(PlainByteMessage);
        //    Assert.IsTrue(true == provider.CompareHash(PlainByteMessage, HashedText));
          
        //}
        [TestMethod]
       [ExpectedException(typeof(System.Configuration.ConfigurationErrorsException))]
        public void TestDefaultInstanceWhichIsNotPresentInConfigurationSource()
        {
            //try
            //{
                DictionaryConfigurationSource DictSource = new DictionaryConfigurationSource();
                CryptographySettings setting = new CryptographySettings();
                setting.DefaultHashProviderName = "NotPresentInConfig";
                HashAlgorithmProviderData Sha1Provider = new HashAlgorithmProviderData();
                Sha1Provider.Name = SHA1Managed;
                Sha1Provider.AlgorithmType = Type.GetType("System.Security.Cryptography.SHA1Managed, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
                Sha1Provider.Type = typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.HashAlgorithmProvider);
                Sha1Provider.SaltEnabled = false;
                setting.HashProviders.Add(Sha1Provider);
                DictSource.Add("securityCryptographyConfiguration", setting);
                HashProviderFactory factory = new HashProviderFactory(DictSource);
                IHashProvider provider = (IHashProvider)factory.Create(SHA1Managed );
                IHashProvider providerDefault = (IHashProvider)factory.CreateDefault();
                byte[] HashedText = providerDefault.CreateHash(PlainByteMessage);
                Assert.IsTrue(true == provider.CompareHash(PlainByteMessage, HashedText));
            //}
            //catch (InvalidOperationException e)
            //{
            //    Assert.IsInstanceOfType(e.InnerException, typeof(System.Configuration.ConfigurationErrorsException));
            //}
        }
        #endregion

        #region Build the DictionaryConfiguration Source
        private DictionaryConfigurationSource BuildSymmetrySourceSection()
        {
            DictionaryConfigurationSource section = new DictionaryConfigurationSource();
            CryptographySettings setting = new CryptographySettings();
            setting.HashProviders.Add(GenerateHashAlgorithmProviderData("SHA512ManagedWithNoSalt", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.HashAlgorithmProvider), "System.Security.Cryptography.SHA512Managed, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", false));
            setting.HashProviders.Add(GenerateHashAlgorithmProviderData("SHA512ManagedWithSalt", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.HashAlgorithmProvider), "System.Security.Cryptography.SHA512Managed, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", true));
            setting.HashProviders.Add(GenerateHashAlgorithmProviderData("SHA384ManagedWithNoSalt", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.HashAlgorithmProvider), "System.Security.Cryptography.SHA384Managed, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", false));
            setting.HashProviders.Add(GenerateHashAlgorithmProviderData("SHA384ManagedWithSalt", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.HashAlgorithmProvider), "System.Security.Cryptography.SHA384Managed, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", true));
            setting.HashProviders.Add(GenerateHashAlgorithmProviderData("SHA256ManagedWithNoSalt", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.HashAlgorithmProvider), "System.Security.Cryptography.SHA256Managed, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", false));
            setting.HashProviders.Add(GenerateHashAlgorithmProviderData("SHA256ManagedWithSalt", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.HashAlgorithmProvider), "System.Security.Cryptography.SHA256Managed, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", true));
            setting.HashProviders.Add(GenerateHashAlgorithmProviderData("SHA1ManagedWithNoSalt", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.HashAlgorithmProvider), "System.Security.Cryptography.SHA1Managed, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", false));
            setting.HashProviders.Add(GenerateHashAlgorithmProviderData("SHA1ManagedWithSalt", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.HashAlgorithmProvider), "System.Security.Cryptography.SHA1Managed, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", true));
            setting.HashProviders.Add(GenerateHashAlgorithmProviderData("RIPEMD160ManagedWithNoSalt", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.HashAlgorithmProvider), "System.Security.Cryptography.RIPEMD160Managed, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", false));
            setting.HashProviders.Add(GenerateHashAlgorithmProviderData("RIPEMD160ManagedWithSalt", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.HashAlgorithmProvider), "System.Security.Cryptography.RIPEMD160Managed, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", false));
            setting.HashProviders.Add(GenerateHashAlgorithmProviderData("MD5CryptoServiceProviderWithNoSalt", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.HashAlgorithmProvider), "System.Security.Cryptography.MD5CryptoServiceProvider, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", false));
            setting.HashProviders.Add(GenerateHashAlgorithmProviderData("MD5CryptoServiceProviderWithSalt", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.HashAlgorithmProvider), "System.Security.Cryptography.MD5CryptoServiceProvider, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", true));
            setting.HashProviders.Add(GenerateHashAlgorithmProviderData("hashAlgorithm1", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.HashAlgorithmProvider), "System.Security.Cryptography.SHA1Managed, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", false));
            setting.HashProviders.Add(GenerateHashAlgorithmProviderData("SHA1Managed", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.HashAlgorithmProvider), "System.Security.Cryptography.SHA1Managed, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", true));
            setting.HashProviders.Add(GenerateHashAlgorithmProviderData("SHA1ManagedNoSalt", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.HashAlgorithmProvider), "System.Security.Cryptography.SHA1Managed, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", false));

            setting.HashProviders.Add(GenerateKeyedHashAlgorithmProviderData("MACTripleDESWithNoSalt", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.KeyedHashAlgorithmProvider), "System.Security.Cryptography.MACTripleDES, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", @"MacTripleDes.key",DataProtectionScope.CurrentUser, false));
            setting.HashProviders.Add(GenerateKeyedHashAlgorithmProviderData("MACTripleDESWithSalt", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.KeyedHashAlgorithmProvider), "System.Security.Cryptography.MACTripleDES, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", @"MacTripleDes.key", DataProtectionScope.CurrentUser, true));

            setting.HashProviders.Add(GenerateKeyedHashAlgorithmProviderData("HMACSHA512WithNoSalt", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.KeyedHashAlgorithmProvider), "System.Security.Cryptography.HMACSHA512, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", @"HMAC512.key", DataProtectionScope.CurrentUser, false));
            setting.HashProviders.Add(GenerateKeyedHashAlgorithmProviderData("HMACSHA512WithSalt", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.KeyedHashAlgorithmProvider), "System.Security.Cryptography.HMACSHA512, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", @"HMAC512.key", DataProtectionScope.CurrentUser, true));

            setting.HashProviders.Add(GenerateKeyedHashAlgorithmProviderData("HMACSHA384WithNoSalt", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.KeyedHashAlgorithmProvider), "System.Security.Cryptography.HMACSHA384, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", @"HMAC300.key", DataProtectionScope.CurrentUser, false));
            setting.HashProviders.Add(GenerateKeyedHashAlgorithmProviderData("HMACSHA384WithSalt", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.KeyedHashAlgorithmProvider), "System.Security.Cryptography.HMACSHA384, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", @"HMAC300.key", DataProtectionScope.CurrentUser, true));

            setting.HashProviders.Add(GenerateKeyedHashAlgorithmProviderData("HMACSHA256WithNoSalt", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.KeyedHashAlgorithmProvider), "System.Security.Cryptography.HMACSHA256, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", @"HMAC256.key", DataProtectionScope.CurrentUser, false));
            setting.HashProviders.Add(GenerateKeyedHashAlgorithmProviderData("HMACSHA256WithSalt", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.KeyedHashAlgorithmProvider), "System.Security.Cryptography.HMACSHA256, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", @"HMAC256.key", DataProtectionScope.CurrentUser, true));

            setting.HashProviders.Add(GenerateKeyedHashAlgorithmProviderData("HMACMD5WithNoSalt", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.KeyedHashAlgorithmProvider), "System.Security.Cryptography.HMACSHA1, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", @"Hmac1.key", DataProtectionScope.CurrentUser, false));
            setting.HashProviders.Add(GenerateKeyedHashAlgorithmProviderData("HMACMD5WithSalt", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.KeyedHashAlgorithmProvider), "System.Security.Cryptography.HMACSHA1, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", @"Hmac1.key", DataProtectionScope.CurrentUser, true));

            setting.HashProviders.Add(GenerateKeyedHashAlgorithmProviderData("HMACRIPEMD160WithNoSalt", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.KeyedHashAlgorithmProvider), "System.Security.Cryptography.HMACRIPEMD160, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", @"HMACRIPEMD160.key", DataProtectionScope.CurrentUser, false));
            setting.HashProviders.Add(GenerateKeyedHashAlgorithmProviderData("HMACRIPEMD160WithSalt", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.KeyedHashAlgorithmProvider), "System.Security.Cryptography.HMACRIPEMD160, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", @"HMACRIPEMD160.key", DataProtectionScope.CurrentUser, true));

            setting.HashProviders.Add(GenerateKeyedHashAlgorithmProviderData("HMACSHA1WithNoSalt", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.KeyedHashAlgorithmProvider), "System.Security.Cryptography.HMACSHA1, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", @"Hmac1.key", DataProtectionScope.CurrentUser, false));
            setting.HashProviders.Add(GenerateKeyedHashAlgorithmProviderData("HMACSHA1WithSalt", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.KeyedHashAlgorithmProvider), "System.Security.Cryptography.HMACSHA1, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", @"Hmac1.key", DataProtectionScope.CurrentUser, true));

            setting.HashProviders.Add(GenerateKeyedHashAlgorithmProviderData("hmac1", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.KeyedHashAlgorithmProvider), "System.Security.Cryptography.HMACSHA1, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", @"Hmac1.key", DataProtectionScope.CurrentUser, true));

            setting.HashProviders.Add(GenerateKeyedHashAlgorithmProviderData("HMACMD5KeyedHash", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.KeyedHashAlgorithmProvider), "System.Security.Cryptography.HMACMD5, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", @"HMacMd5.key", DataProtectionScope.CurrentUser, true));

            setting.HashProviders.Add(GenerateKeyedHashAlgorithmProviderData("HMACMD5KeyedHashNoSalt", typeof(Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.KeyedHashAlgorithmProvider), "System.Security.Cryptography.HMACMD5, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089", @"HMacMd5.key", DataProtectionScope.CurrentUser, false));
			section.Add("securityCryptographyConfiguration", setting);
            return section;
        }
        private HashAlgorithmProviderData GenerateHashAlgorithmProviderData(String Name, Type HashProviderType, String AlgorithmType, bool salt)
        {
            HashAlgorithmProviderData data = new HashAlgorithmProviderData();
            data.Name = Name;
            data.Type = HashProviderType;
            data.AlgorithmType = Type.GetType(AlgorithmType);
            data.SaltEnabled = salt;
            return (data);
        }
        private KeyedHashAlgorithmProviderData GenerateKeyedHashAlgorithmProviderData(String Name, Type HashProviderType, String AlgorithmType, String KeyFileName,DataProtectionScope scope, bool salt)
        {
            KeyedHashAlgorithmProviderData data = new KeyedHashAlgorithmProviderData();
            data.Name = Name;
            data.Type = HashProviderType;
            data.AlgorithmType = Type.GetType(AlgorithmType);
            data.SaltEnabled = salt;
            data.ProtectedKeyFilename = KeyFileName;
            data.ProtectedKeyProtectionScope = scope;
           // data.Key = System.Convert.FromBase64String(Key);
            return (data);
        }
        #endregion

        # region HelperMethod
        private bool CompareByteArrays(byte[] Array1, byte[] Array2)
        {
            if (Array1.GetUpperBound(0) != Array2.GetUpperBound(0))
            {
                return (false);
            }
            for (int index1 = 0; index1 < Array1.GetUpperBound(0); index1++)
            {
                if (Array1[index1] != Array2[index1])
                {
                    return (false);
                }
            }
            return (true);
        }
        # endregion
    }
}
