using System;
using System.Text;
using System.Collections.Generic;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Data.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Data;
using System.Data.SqlClient;
using System.Xml;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using System.Configuration;
using System.Diagnostics;
using System.Management;
using System.Messaging;

using Microsoft.Practices.EnterpriseLibrary.Data.Sql;
using System.Data.Common;

namespace DataFuncTests
{
    /// <summary>
    /// Summary description for CustomDatabase
    /// </summary>
    [TestClass]
    public class CustomDatabaseTests
    {
        ManagementScope scope = new ManagementScope(@"\\.\root\EnterpriseLibrary");

        public CustomDatabaseTests()
        {
            //
            // TODO: Add constructor logic here
            //
        }


        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        [TestInitialize()]
        public void MyTestInitialize() 
        {
        }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void CreateCustomDatabaseTest()
        {
         Database db=   DatabaseFactory.CreateDatabase("CustomDatabase");
         Assert.IsNotNull(db);
        }

        [TestMethod]
        public void ConnectionFailedEventTest()
        {
            scope.Options.EnablePrivileges = true;
            EventQuery eq = new EventQuery("SELECT * FROM ConnectionFailedEvent");
            ManagementEventWatcher watcher = new ManagementEventWatcher(scope, eq);
            watcher.EventArrived += new EventArrivedEventHandler(WatcherEventArrived);
            watcher.Start();
            try
            {
                SqlDatabase sql = (SqlDatabase)DatabaseFactory.CreateDatabase("InvalidSqlConnectionString");

                sql.CreateConnection();
                sql.ExecuteNonQuery(sql.GetStoredProcCommand("AddInItems"));
            }
            catch { }
            watcher.EventArrived -= new EventArrivedEventHandler(WatcherEventArrived);
            watcher.Stop();  
        }

        private void WatcherEventArrived(object sender, EventArrivedEventArgs args)
        {

            /// Validate the message here
            string wmiText = args.NewEvent.GetText(TextFormat.Mof);

        }


        [TestMethod]
        public void TestingForRuntimeChanges()
        {
            Database db = DatabaseFactory.CreateDatabase("CustomDatabase");
            Assert.IsNotNull(db);

            string configurationFile = "daabfunctionaltests.dll.config";
           XmlDocument doc = new XmlDocument();
           doc.Load(configurationFile);
           XmlNodeList nodeList = doc.SelectNodes("/configuration/connectionStrings/add");
           foreach (XmlNode xNode in nodeList)
           {
               if (((XmlAttribute)xNode.Attributes["name"]).Value == "CustomDatabase")
               {
                   ((XmlAttribute)xNode.Attributes["connectionString"]).Value = "ThisIsChangedValue";
                   break;
               }
           }
            doc.Save(configurationFile);
            System.Threading.Thread.Sleep(15000);
            Database db1 = DatabaseFactory.CreateDatabase("CustomDatabase");
            Assert.AreEqual(((CustomMockDatabase)db1).ConnectionString, "ThisIsChangedValue");


        }

        [TestMethod]
        public void TestingForRuntimeChangesUsingFileSource()
        {
            string configurationFile = "daabfunctionaltests.dll.config";
            DatabaseProviderFactory dpf = new DatabaseProviderFactory(new FileConfigurationSource(configurationFile));
            Database db = dpf.Create("CustomDatabase");
            Assert.IsNotNull(db);

            
            XmlDocument doc = new XmlDocument();
            doc.Load(configurationFile);
            XmlNodeList nodeList = doc.SelectNodes("/configuration/connectionStrings/add");
            foreach (XmlNode xNode in nodeList)
            {
                if (((XmlAttribute)xNode.Attributes["name"]).Value == "CustomDatabase")
                {
                    ((XmlAttribute)xNode.Attributes["connectionString"]).Value = "ThisIsChangedValue";
                    break;
                }
            }
            doc.Save(configurationFile);
            System.Threading.Thread.Sleep(15000);
            Database db1 = dpf.Create("CustomDatabase");
            Assert.AreEqual(((CustomMockDatabase)db1).ConnectionString, "ThisIsChangedValue");


        }

        [TestMethod]
        [ExpectedException(typeof(SqlException))]
        public void DBTimeOutError()
        {
            Database db = DatabaseFactory.CreateDatabase("DataSQLTest");
            DbCommand dbCmd = db.GetSqlStringCommand("waitfor delay '0:0:30'");
            dbCmd.CommandTimeout = 10;
            db.ExecuteNonQuery(dbCmd);

        }

        [TestMethod]
        public void DBInvalidInstanceEventLogTest()
        {
            using (EventLog eventLog = new EventLog("Application"))
            {
                eventLog.Clear();
                try
                {
                    Database db = DatabaseFactory.CreateDatabase("InvalidInstanceNotInConfig");
                }
                catch (ConfigurationErrorsException)
                {
                }
                Assert.AreEqual(1, eventLog.Entries.Count);
            }

        }
        
    }

    [DatabaseAssembler(typeof(CustomDatabaseCustomAssembler))]
    public class CustomMockDatabase : Database
    {
        public string ConnectionString;
        public CustomMockDatabase(string connectionString):base(connectionString,SqlClientFactory.Instance)
        {
            this.ConnectionString = connectionString;
        }

        protected override void DeriveParameters(System.Data.Common.DbCommand discoveryCommand)
        {
            throw new Exception("The method or operation is not implemented.");
        }
    }

    public class CustomDatabaseCustomAssembler : IDatabaseAssembler
    {
        #region IDatabaseAssembler Members

        public Database Assemble(string name, System.Configuration.ConnectionStringSettings connectionStringSettings, Microsoft.Practices.EnterpriseLibrary.Common.Configuration.IConfigurationSource configurationSource)
        {
            return new CustomMockDatabase(connectionStringSettings.ConnectionString);
        }

        #endregion


    }

    


    

}