using System;
//using Microsoft.VisualStudio.TestTools.UnitTesting;
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Data.Sql;
using System.Text;
using System.Data;
using System.Data.SqlClient;
using System.Data.OleDb;
using System.Data.Odbc;
using System.Data.Sql;
using System.Xml;
using System.Data.Common;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using System.Configuration;


namespace DataFuncTests.GenericDatabaseFixturesFixtures
{
    /// <summary>
    /// Tests ExecuteNonQuery of the Database class
    /// </summary>
    [TestClass]
    public class ExecuteNonQueryTestFixture
    {
        string connStr;
        public ExecuteNonQueryTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        /// <summary>
        /// Initialize() is called once during test execution before
        /// test methods in this test class are executed.
        /// </summary>
        [TestInitialize()]
        public void Initialize()
        {
            connStr = ConfigurationManager.ConnectionStrings["DataSQLTest"].ConnectionString;
            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("begin Delete from Items where itemId > 3 " +
                                                   "Update Items set ItemDescription='Digital Image Pro',QtyInHand = 25, QtyRequired=100 where ItemId = 1 " +
                                                   "Update Items set ItemDescription='Excel 2003',QtyInHand = 95, QtyRequired=100 where ItemId = 2 " +
                                                   "Update Items set ItemDescription='Infopath',QtyInHand = 34, QtyRequired=100 where ItemId = 3 " +
                                                   "delete from CustomersOrders where CustomerId > 3 " +
                                                   "Update CustomersOrders set QtyOrdered=100 " +
                                            "end", conn);
            cmd.ExecuteNonQuery();

            conn.Close();
        }

        /// <summary>
        /// Cleanup() is called once during test execution after
        /// test methods in this class have executed unless the
        /// corresponding Initialize() call threw an exception.
        /// </summary>
        [TestCleanup()]
        public void Cleanup()
        {
            //  TODO: Add test cleanup code
        }

        #region "DB Command"

        [TestMethod]
        public void NonQueryDBCmdQueryTest()
        {
            bool isPresent = false;
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            string sqlCommand = "Insert into CustomersOrders values(13, 'John',3, 214)";
            DbCommand dbCommandWrapper = db.GetSqlStringCommand(sqlCommand);
            db.ExecuteNonQuery(dbCommandWrapper);
            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("select * from CustomersOrders where CustomerName = 'John'", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(3, Convert.ToInt32(dr["ItemId"].ToString()));
                Assert.AreEqual(214, Convert.ToInt32(dr["QtyOrdered"].ToString()));
                isPresent = true;
            }

            dr.Close();
            Assert.AreEqual(true, isPresent);
        }

        [TestMethod]
        public void NonQueryDBCmdSpNoParmTest()
        {
            bool isPresent = false;
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            string sqlCommand = "CustomerOrdersAdd";
            DbCommand dbCommandWrapper = db.GetStoredProcCommand(sqlCommand);
            db.ExecuteNonQuery(dbCommandWrapper);
            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("select * from CustomersOrders where CustomerName = 'David'", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(3, Convert.ToInt32(dr["ItemId"].ToString()));
                Assert.AreEqual(555, Convert.ToInt32(dr["QtyOrdered"].ToString()));
                isPresent = true;
            }

            dr.Close();
            cmd = new SqlCommand("delete from CustomersOrders where CustomerName = 'David'", conn);
            dr = cmd.ExecuteReader();
            dr.Close();
            conn.Close();
            Assert.AreEqual(true, isPresent);
        }

        [TestMethod]
        public void NonQueryDBCmdSpINParmTest()
        {
            bool isPresent = false;
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            string sqlCommand = "CustomerOrdersAddwithParm";
            
            DbCommand dbCommandWrapper = db.GetStoredProcCommand(sqlCommand);
            this.CreateCommandParameter(db,dbCommandWrapper, "@CustomerID", DbType.Int16, ParameterDirection.Input, 10);
            this.CreateCommandParameter(db,dbCommandWrapper, "@CustomerName", DbType.String, ParameterDirection.Input, "Steve");
            this.CreateCommandParameter(db,dbCommandWrapper, "@ItemId", DbType.Int16, ParameterDirection.Input, 3);
            this.CreateCommandParameter(db,dbCommandWrapper, "@QtyOrdered", DbType.Int16, ParameterDirection.Input, 432);
            db.ExecuteNonQuery(dbCommandWrapper);
            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("select * from CustomersOrders where CustomerName = 'Steve'", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(3, Convert.ToInt32(dr["ItemId"].ToString()));
                Assert.AreEqual(432, Convert.ToInt32(dr["QtyOrdered"].ToString()));
                isPresent = true;
            }

            dr.Close();
            Assert.AreEqual(true, isPresent);
        }

        [TestMethod]
        public void NonQueryDBCmdSPOutParmTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            string sqlCommand = "CustomerOrdersAddwithOutParm";
            DbCommand dbCommandWrapper = db.GetStoredProcCommand(sqlCommand);
            this.CreateCommandParameter(db,dbCommandWrapper, "@CustomerId", DbType.Int16, ParameterDirection.Input, 3);
            this.CreateCommandParameter(db,dbCommandWrapper, "@QtyOrdered", DbType.Int16, ParameterDirection.Output, 16);
            db.ExecuteNonQuery(dbCommandWrapper);
            Assert.AreEqual(234, Convert.ToInt32(dbCommandWrapper.Parameters["@QtyOrdered"].Value));
        }

        [TestMethod]
        [ExpectedException(typeof(OleDbException))]
        public void NonQueryDBCmdNoSPPresentTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            string sqlCommand = "UpdateItem1";
            DbCommand dbCommandWrapper = db.GetStoredProcCommand(sqlCommand);
            db.ExecuteNonQuery(dbCommandWrapper);
            Assert.AreEqual(ConnectionState.Closed, dbCommandWrapper.Connection.State);
        }
        #endregion

        #region "Db Command, Transaction"
        [TestMethod]
        public void NonQueryDBCmdTransSqlNoCommitTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            StringBuilder readerData = new StringBuilder();
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                string sqlCommand = "begin update Items set QtyInHand= 188 where ItemId = 2 end";
                DbCommand dbCommandWrapper = db.GetSqlStringCommand(sqlCommand);
                dbCommandWrapper.Transaction = transaction;
                dbCommandWrapper.Connection = connection;
                db.ExecuteNonQuery(dbCommandWrapper, transaction);
            }
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("select * from Items where ItemId = 2", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(95), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
        }

        [TestMethod]
        public void NonQueryDBCmdTransSqlCommitTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            StringBuilder readerData = new StringBuilder();
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                string sqlCommand = "begin update Items set QtyInHand = 145 where ItemId = 1 end";
                DbCommand dbCommandWrapper = db.GetSqlStringCommand(sqlCommand);
                dbCommandWrapper.Transaction = transaction;
                dbCommandWrapper.Connection = connection;
                db.ExecuteNonQuery(dbCommandWrapper, transaction);
                transaction.Commit();
            }
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId = 1", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(145), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void NonQueryDBCmdTransSqlRollbackTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            StringBuilder readerData = new StringBuilder();
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                string sqlCommand = "begin update Items set QtyInHand = 167 where ItemId = 2 end";
                DbCommand dbCommandWrapper = db.GetSqlStringCommand(sqlCommand);
                dbCommandWrapper.Transaction = transaction;
                dbCommandWrapper.Connection = connection;
                db.ExecuteNonQuery(dbCommandWrapper, transaction);
                transaction.Rollback();
            }
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId = 2", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(95), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void NonQueryDBCmdTransSqlCommitRollbackTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            StringBuilder readerData = new StringBuilder();
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                string sqlCommand = "begin update Items set QtyInHand = 211 where ItemId = 1  end";
                DbCommand dbCommandWrapper = db.GetSqlStringCommand(sqlCommand);
                dbCommandWrapper.Transaction = transaction;
                dbCommandWrapper.Connection = connection;
                db.ExecuteNonQuery(dbCommandWrapper, transaction);
                transaction.Commit();
                transaction = connection.BeginTransaction();
                sqlCommand = "begin update Items set QtyInHand = 255 where ItemId = 2 end";
                dbCommandWrapper = db.GetSqlStringCommand(sqlCommand);
                db.ExecuteNonQuery(dbCommandWrapper, transaction);
                transaction.Rollback();
            }
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId = 1", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(211), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            cmd = new SqlCommand("Select QtyInHand from Items where ItemId = 2", conn);
            dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(95), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void NonQueryDBCmdTransSqlRollbackOnErrTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            StringBuilder readerData = new StringBuilder();
            DbTransaction transaction = null;
            try
            {
                using (DbConnection connection = db.CreateConnection())
                {
                    connection.Open();
                    transaction = connection.BeginTransaction();
                    string sqlCommand = "begin update Items set QtyInHand = 45 where ItemId = 'Test' end";
                    DbCommand dbCommandWrapper = db.GetSqlStringCommand(sqlCommand);
                    db.ExecuteNonQuery(dbCommandWrapper, transaction);
                    transaction.Commit();
                }
            }
            catch
            {
            }
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId = 2", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(95), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }

            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void NonQueryDBCmdTransSPNoCommitTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            StringBuilder readerData = new StringBuilder();
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                string sqlCommand = "ItemQuantityUpdateById";
                DbCommand dbCommandWrapper = db.GetStoredProcCommand(sqlCommand);
                this.CreateCommandParameter(db,dbCommandWrapper, "@ItemId", DbType.Int16, ParameterDirection.Input, 2);
                this.CreateCommandParameter(db,dbCommandWrapper, "@Quantity", DbType.Int16, ParameterDirection.Input, 67);
                dbCommandWrapper.Transaction = transaction;
                dbCommandWrapper.Connection = connection;
                db.ExecuteNonQuery(dbCommandWrapper, transaction);

            }
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId = 2", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(95), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void NonQueryDBCmdTransSPCommitTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            StringBuilder readerData = new StringBuilder();
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                string sqlCommand = "ItemQuantityUpdateById";
                DbCommand dbCommandWrapper = db.GetStoredProcCommand(sqlCommand);
                this.CreateCommandParameter(db,dbCommandWrapper, "@ItemId", DbType.Int16, ParameterDirection.Input, 1);
                this.CreateCommandParameter(db,dbCommandWrapper, "@Quantity", DbType.Int16, ParameterDirection.Input, 43);
                dbCommandWrapper.Transaction = transaction;
                dbCommandWrapper.Connection = connection;
                db.ExecuteNonQuery(dbCommandWrapper, transaction);
                transaction.Commit();
            }
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId = 1", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(43), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void NonQueryDBCmdTransSPRollbackTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            StringBuilder readerData = new StringBuilder();
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                string sqlCommand = "ItemQuantityUpdateById";
                DbCommand dbCommandWrapper = db.GetStoredProcCommand(sqlCommand);
                this.CreateCommandParameter(db,dbCommandWrapper, "@ItemId", DbType.Int16, ParameterDirection.Input, 2);
                this.CreateCommandParameter(db,dbCommandWrapper, "@Quantity", DbType.Int16, ParameterDirection.Input, 65);
                dbCommandWrapper.Transaction = transaction;
                dbCommandWrapper.Connection = connection;
                db.ExecuteNonQuery(dbCommandWrapper, transaction);
                transaction.Rollback();
            }
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId = 2", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(95), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void NonQueryDBCmdTransSPCommitRollbackTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            StringBuilder readerData = new StringBuilder();
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                string sqlCommand = "ItemQuantityUpdateById";
                DbCommand dbCommandWrapper = db.GetStoredProcCommand(sqlCommand);
                this.CreateCommandParameter(db,dbCommandWrapper, "@ItemId", DbType.Int16, ParameterDirection.Input, 1);
                this.CreateCommandParameter(db,dbCommandWrapper, "@Quantity", DbType.Int16, ParameterDirection.Input, 74);
                dbCommandWrapper.Transaction = transaction;
                dbCommandWrapper.Connection = connection;
                db.ExecuteNonQuery(dbCommandWrapper, transaction);
                transaction.Commit();
                transaction = connection.BeginTransaction();
                sqlCommand = "ItemQuantityUpdateById";
                dbCommandWrapper = db.GetStoredProcCommand(sqlCommand);
                this.CreateCommandParameter(db,dbCommandWrapper, "@ItemId", DbType.Int16, ParameterDirection.Input, 2);
                this.CreateCommandParameter(db,dbCommandWrapper, "@Quantity", DbType.Int16, ParameterDirection.Input, 72);
                db.ExecuteNonQuery(dbCommandWrapper, transaction);
                transaction.Rollback();
            }
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId = 1", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(74), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            cmd = new SqlCommand("Select QtyInHand from Items where ItemId = 2", conn);
            dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(95), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        #endregion

        #region "CommandType, CommandText"

        [TestMethod]

        public void NonQueryCommandtypeSqlTest()
        {
            bool isPresent = false;
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            StringBuilder readerDataActual = new StringBuilder();
            string sqlText = "Insert into CustomersOrders Values(14, 'Lee', 1, 233)";
            db.ExecuteNonQuery(CommandType.Text, sqlText);
            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select CustomerName from CustomersOrders where CustomerId = 14", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual("Lee", dr["CustomerName"].ToString());
                isPresent = true;
            }
            dr.Close();
            conn.Close();
            Assert.AreEqual(true, isPresent);
        }


        [TestMethod]
        public void NonQueryCommandtypeSpTest()
        {
            bool isPresent = false;
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            StringBuilder readerDataActual = new StringBuilder();
            string spName = "CustomerOrdersAdd";
            db.ExecuteNonQuery(CommandType.StoredProcedure, spName);
            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("select * from CustomersOrders where CustomerName = 'David'", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(3, Convert.ToInt32(dr["ItemId"].ToString()));
                Assert.AreEqual(555, Convert.ToInt32(dr["QtyOrdered"].ToString()));
                isPresent = true;
            }
            dr.Close();
            cmd = new SqlCommand("delete from CustomersOrders where CustomerName = 'David'", conn);
            dr = cmd.ExecuteReader();
            dr.Close();
            conn.Close();
            Assert.AreEqual(true, isPresent);
        }


        [TestMethod]
        [ExpectedException(typeof(OleDbException))]
        public void NonQueryNotExistingSPTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            StringBuilder readerDataActual = new StringBuilder();
            string spName = "something";
            db.ExecuteNonQuery(CommandType.StoredProcedure, spName);
        }
        #endregion

        #region "Transaction, Command Type, Command Name"
        [TestMethod]
        public void NonQueryTransSQLTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                db.ExecuteNonQuery(transaction, CommandType.Text, "begin update Items set QtyInHand = 175 where ItemId = 1 end ");
                transaction.Commit();
            }
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId=1", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(175), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void NonQueryTransSQLNoCommitTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                db.ExecuteNonQuery(transaction, CommandType.Text, "begin update Items set QtyInHand = 67 where ItemId = 2 end ");
            }

            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId=2", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(95), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void NonQueryTransSQLRollbackTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                db.ExecuteNonQuery(transaction, CommandType.Text, "begin update Items set QtyInHand = 18 where ItemId = 2 select * from Items end "); transaction.Rollback();
            }

            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId=2", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(95), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void NonQueryTransSpCommitTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                db.ExecuteNonQuery(transaction, CommandType.StoredProcedure, "ItemQuantityUpdateById");
                transaction.Commit();
            }

            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where itemid=1", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(50, Convert.ToInt32(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void NonQueryTransSpNoCommitTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                db.ExecuteNonQuery(transaction, CommandType.StoredProcedure, "ItemQuantityIncrease");
            }
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyRequired from Items", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(100, Convert.ToInt32(dr["QtyRequired"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void NonQueryTransSpRollbackTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                db.ExecuteNonQuery(transaction, CommandType.StoredProcedure, "ItemQuantityIncrease");
                transaction.Rollback();
            }

            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyRequired from Items", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(100, Convert.ToInt32(dr["QtyRequired"].ToString()));
            }
            dr.Close();
            conn.Close();
        }
        #endregion

        # region Helper Method

        private void CreateCommandParameter(Database db, DbCommand comm, string Parametername, DbType dbType, ParameterDirection dir, object value)
        {
            if (dir == ParameterDirection.Input)
            {
                db.AddInParameter(comm, Parametername, dbType, value);
            }
            else if (dir == ParameterDirection.Output)
            {
                db.AddOutParameter(comm, Parametername, dbType, (int)value);
            }
        }
        # endregion
    }
}
