using System;
using System.Text;
using System.Collections.Generic;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Data;
using System.Data;
using System.Data.Common;
namespace DataFuncTests.OracleDatabaseFixtures
{
    /// <summary>
    /// Summary description for UpdateDataSetTestFixture
    /// </summary>
    [TestClass]
    public class UpdateDataSetTestFixture
    {
        public UpdateDataSetTestFixture()
        {
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
         [TestCleanup()]
         public void Cleanup() 
         {
             Database db = DatabaseFactory.CreateDatabase("OracleTest");
             db.ExecuteNonQuery(CommandType.Text, "delete from Country where CountryCode in ('HOL','FRA','FIN','SWI','SIN','JAP', 'RUS')");   
         }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void TransactionCommitTest()
        {
            Database db = DatabaseFactory.CreateDatabase("OracleTest");
            DataSet ds = new DataSet();

            DbCommand dbAddCountry = db.GetStoredProcCommand("AddCountry");
            db.AddInParameter(dbAddCountry, "vCountryCode", DbType.String,"CountryCode",DataRowVersion.Current );
            db.AddInParameter(dbAddCountry, "vCountryName", DbType.String,"CountryName",DataRowVersion.Current);    
            DbCommand dbUpdateCountry = db.GetStoredProcCommand("UpdateCountry");
            db.AddInParameter(dbUpdateCountry, "vCountryCode", DbType.String,"CountryCode", DataRowVersion.Current);
            db.AddInParameter(dbUpdateCountry, "vCountryName", DbType.String,"CountryName", DataRowVersion.Current);
            DbCommand dbDeleteCountry = db.GetStoredProcCommand("DeleteCountry");
            db.AddInParameter(dbDeleteCountry, "vCountryCode", DbType.String,"CountryCode", DataRowVersion.Current);

            db.LoadDataSet(CommandType.Text, "select CountryCode,CountryName from Country", ds, new string[] { "Country" });

            ds.Tables[0].PrimaryKey = new DataColumn[] { ds.Tables[0].Columns["CountryCode"] };

            DataRow row = ds.Tables[0].NewRow();
            row["CountryCode"] = "HOL";
            row["CountryName"] = "Holland";

            ds.Tables[0].Rows.Add(row);
            
            DataRow modifyRow = ds.Tables[0].Rows.Find("HOL");
            modifyRow["CountryName"] = "HOLLAND";
 
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                try
                {
                    db.UpdateDataSet(ds, "Country", dbAddCountry, dbUpdateCountry, dbDeleteCountry, transaction);
                    transaction.Commit();  
                }
                catch (Exception e)
                {
                    transaction.Rollback();
                    Assert.Fail("Transaction Rolled back : " + e.Message);   
                }
            }

            DataRow result = ds.Tables[0].Rows.Find("HOL");
            Assert.IsNotNull(result);
            Assert.AreEqual("HOLLAND", result["CountryName"].ToString());
        }

        [TestMethod]
        public void TransactionRollbackTest()
        {
            Database db = DatabaseFactory.CreateDatabase("OracleTest");
            DataSet ds = new DataSet();

            DbCommand dbAddCountry = db.GetStoredProcCommand("AddCountry");
            db.AddInParameter(dbAddCountry, "vCountryCode", DbType.String, "CountryCode", DataRowVersion.Current);
            db.AddInParameter(dbAddCountry, "vCountryName", DbType.String, "CountryName", DataRowVersion.Current);
            DbCommand dbUpdateCountry = db.GetStoredProcCommand("UpdateCountry");
            db.AddInParameter(dbUpdateCountry, "vCountryCode", DbType.String, "CountryCode", DataRowVersion.Current);
            db.AddInParameter(dbUpdateCountry, "vCountryName", DbType.String, "CountryName", DataRowVersion.Current);
            DbCommand dbDeleteCountry = db.GetStoredProcCommand("DeleteCountry");
            db.AddInParameter(dbDeleteCountry, "vCountryCode", DbType.String, "CountryCode", DataRowVersion.Current);

            DbCommand dbAddEmployee = db.GetStoredProcCommand("AddEmployees");
 
            db.LoadDataSet(CommandType.Text, "select CountryCode,CountryName from Country", ds, new string[] { "Country" });

            ds.Tables[0].PrimaryKey = new DataColumn[] { ds.Tables[0].Columns["CountryCode"] };

            DataRow row = ds.Tables[0].NewRow();
            row["CountryCode"] = "JAP";
            row["CountryName"] = "Japan";

            ds.Tables[0].Rows.Add(row);

            DataRow modifyRow = ds.Tables[0].Rows.Find("JAP");
            modifyRow["CountryName"] = "JAPAN";

            DataRow deleteRow = ds.Tables[0].Rows.Find("IN");
            deleteRow.Delete();    

            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                try
                {
                    db.UpdateDataSet(ds, "Country", dbAddCountry, dbUpdateCountry, dbDeleteCountry, transaction);
                    db.UpdateDataSet(ds, "Employees", dbAddEmployee, null, null, transaction);
                    transaction.Commit();
                }
                catch (Exception )
                {
                    transaction.Rollback();
                }
            }

            using (DataSet dsResult = db.ExecuteDataSet(CommandType.Text, "select * from Country where CountryCode in ('IN','JAP')"))
            {
                dsResult.Tables[0].PrimaryKey = new DataColumn[] { dsResult.Tables[0].Columns["CountryCode"] };
                DataRow result = dsResult.Tables[0].Rows.Find("IN");
                Assert.IsNotNull(result);
                result = dsResult.Tables[0].Rows.Find("JAP");
                Assert.IsNull(result);
            }
        }

        [TestMethod]
        public void UpdateBehaviorContinueOnErrorTest()
        {
            Database db = DatabaseFactory.CreateDatabase("OracleTest");
            DataSet ds = new DataSet();

            DbCommand dbAddCountry = db.GetStoredProcCommand("AddCountry");
            db.AddInParameter(dbAddCountry, "vCountryCode", DbType.String, "CountryCode", DataRowVersion.Current);
            db.AddInParameter(dbAddCountry, "vCountryName", DbType.String, "CountryName", DataRowVersion.Current);
            DbCommand dbUpdateCountry = db.GetStoredProcCommand("UpdateCountry");
            db.AddInParameter(dbUpdateCountry, "vCountryCode", DbType.String, "CountryCode", DataRowVersion.Current);
            db.AddInParameter(dbUpdateCountry, "vCountryName", DbType.String, "CountryName", DataRowVersion.Current);
            DbCommand dbDeleteCountry = db.GetStoredProcCommand("DeleteCountry");
            db.AddInParameter(dbDeleteCountry, "vCountryCode", DbType.String, "CountryCode", DataRowVersion.Current);

            DbCommand dbAddEmployee = db.GetStoredProcCommand("AddEmployees");
 
            db.LoadDataSet(CommandType.Text, "select CountryCode,CountryName from Country", ds, new string[] { "Country" });

            DataRow row = ds.Tables[0].NewRow();
            row["CountryCode"] = "SIN";
            row["CountryName"] = "Singapore";

            ds.Tables[0].Rows.Add(row);
            row = ds.Tables[0].NewRow();
            ds.Tables[0].Rows.Add(row);

            db.UpdateDataSet(ds, "Country", dbAddCountry, dbUpdateCountry, dbDeleteCountry, UpdateBehavior.Continue);

            using (DataSet dsResult = db.ExecuteDataSet(CommandType.Text, "select * from Country where CountryCode in ('SIN')"))
            {
                dsResult.Tables[0].PrimaryKey = new DataColumn[] { dsResult.Tables[0].Columns["CountryCode"] };
                DataRow result = dsResult.Tables[0].Rows.Find("SIN");
                Assert.IsNotNull(result);
                Assert.AreEqual("Singapore", Convert.ToString(result["CountryName"]));    
            }
        }

        [TestMethod]
        public void UpdateBehaviorTransactionRollbackTest()
        {
            Database db = DatabaseFactory.CreateDatabase("OracleTest");
            DataSet ds = new DataSet();

            DbCommand dbAddCountry = db.GetStoredProcCommand("AddCountry");
            db.AddInParameter(dbAddCountry, "vCountryCode", DbType.String, "CountryCode", DataRowVersion.Current);
            db.AddInParameter(dbAddCountry, "vCountryName", DbType.String, "CountryName", DataRowVersion.Current);
            DbCommand dbUpdateCountry = db.GetStoredProcCommand("UpdateCountry");
            db.AddInParameter(dbUpdateCountry, "vCountryCode", DbType.String, "CountryCode", DataRowVersion.Current);
            db.AddInParameter(dbUpdateCountry, "vCountryName", DbType.String, "CountryName", DataRowVersion.Current);
            DbCommand dbDeleteCountry = db.GetStoredProcCommand("DeleteCountry");
            db.AddInParameter(dbDeleteCountry, "vCountryCode", DbType.String, "CountryCode", DataRowVersion.Current);

            DbCommand dbAddEmployee = db.GetStoredProcCommand("AddEmployees");

            db.LoadDataSet(CommandType.Text, "select CountryCode,CountryName from Country", ds, new string[] { "Country" });

            DataRow row = ds.Tables[0].NewRow();
            row["CountryCode"] = "HOL";
            row["CountryName"] = "Holland";

            ds.Tables[0].Rows.Add(row);
            row = ds.Tables[0].NewRow();
            ds.Tables[0].Rows.Add(row);
            try
            {
                db.UpdateDataSet(ds, "Country", dbAddCountry, dbUpdateCountry, dbDeleteCountry, UpdateBehavior.Transactional);
            }
            catch
            {
            }

            using (DataSet dsResult = db.ExecuteDataSet(CommandType.Text, "select * from Country where CountryCode in ('HOL')"))
            {
                Assert.IsNotNull(0 == dsResult.Tables[0].Rows.Count);
            }
        }

        [TestMethod]
        public void UpdateBehaviorStandardTest()
        {
            Database db = DatabaseFactory.CreateDatabase("OracleTest");
            DataSet ds = new DataSet();

            DbCommand dbAddCountry = db.GetStoredProcCommand("AddCountry");
            db.AddInParameter(dbAddCountry, "vCountryCode", DbType.String, "CountryCode", DataRowVersion.Current);
            db.AddInParameter(dbAddCountry, "vCountryName", DbType.String, "CountryName", DataRowVersion.Current);
            DbCommand dbUpdateCountry = db.GetStoredProcCommand("UpdateCountry");
            db.AddInParameter(dbUpdateCountry, "vCountryCode", DbType.String, "CountryCode", DataRowVersion.Current);
            db.AddInParameter(dbUpdateCountry, "vCountryName", DbType.String, "CountryName", DataRowVersion.Current);
            DbCommand dbDeleteCountry = db.GetStoredProcCommand("DeleteCountry");
            db.AddInParameter(dbDeleteCountry, "vCountryCode", DbType.String, "CountryCode", DataRowVersion.Current);

            DbCommand dbAddEmployee = db.GetStoredProcCommand("AddEmployees");

            db.LoadDataSet(CommandType.Text, "select CountryCode,CountryName from Country", ds, new string[] { "Country" });
            ds.Tables[0].PrimaryKey = new DataColumn[] { ds.Tables[0].Columns["CountryCode"] };

            DataRow row = ds.Tables[0].NewRow();
            row["CountryCode"] = "FIN";
            row["CountryName"] = "Finland";

            ds.Tables[0].Rows.Add(row);
            DataRow modifyRow = ds.Tables[0].Rows.Find("FIN");
            modifyRow["CountryName"] = "FINLAND";
            try
            {
                db.UpdateDataSet(ds, "Country", dbAddCountry, dbUpdateCountry, dbDeleteCountry, UpdateBehavior.Standard);
            }
            catch
            {
            }
            using (DataSet dsResult = db.ExecuteDataSet(CommandType.Text, "select * from Country where CountryCode in ('FIN')"))
            {
                Assert.IsTrue(1 == dsResult.Tables[0].Rows.Count);
                Assert.AreEqual("FINLAND", Convert.ToString(dsResult.Tables[0].Rows[0]["CountryName"]));
            }
        }

        [TestMethod]
        public void UpdateBehaviorStandardOnErrorTest()
        {
            Database db = DatabaseFactory.CreateDatabase("OracleTest");
            DataSet ds = new DataSet();

            DbCommand dbAddCountry = db.GetStoredProcCommand("AddCountry");
            db.AddInParameter(dbAddCountry, "vCountryCode", DbType.String, "CountryCode", DataRowVersion.Current);
            db.AddInParameter(dbAddCountry, "vCountryName", DbType.String, "CountryName", DataRowVersion.Current);
            DbCommand dbUpdateCountry = db.GetStoredProcCommand("UpdateCountry");
            db.AddInParameter(dbUpdateCountry, "vCountryCode", DbType.String, "CountryCode", DataRowVersion.Current);
            db.AddInParameter(dbUpdateCountry, "vCountryName", DbType.String, "CountryName", DataRowVersion.Current);
            DbCommand dbDeleteCountry = db.GetStoredProcCommand("DeleteCountry");
            db.AddInParameter(dbDeleteCountry, "vCountryCode", DbType.String, "CountryCode", DataRowVersion.Current);

            DbCommand dbAddEmployee = db.GetStoredProcCommand("AddEmployees");

            db.LoadDataSet(CommandType.Text, "select CountryCode,CountryName from Country", ds, new string[] { "Country" });

            DataRow row = ds.Tables[0].NewRow();
            row["CountryCode"] = "SWI";
            row["CountryName"] = "SwitzerLand";

            ds.Tables[0].Rows.Add(row);
            row = ds.Tables[0].NewRow();
            ds.Tables[0].Rows.Add(row);

            try
            {
                db.UpdateDataSet(ds, "Country", dbAddCountry, dbUpdateCountry, dbDeleteCountry, UpdateBehavior.Standard);
            }
            catch
            {
            }

            using (DataSet dsResult = db.ExecuteDataSet(CommandType.Text, "select * from Country where CountryCode in ('SWI')"))
            {
                Assert.IsTrue(1 == dsResult.Tables[0].Rows.Count);
                Assert.AreEqual("SwitzerLand", Convert.ToString(dsResult.Tables[0].Rows[0]["CountryName"]));
            }
        }

        [TestMethod]
        public void UpdateBehaviorTransactionCommitTest()
        {
            Database db = DatabaseFactory.CreateDatabase("OracleTest");
            DataSet ds = new DataSet();

            DbCommand dbAddCountry = db.GetStoredProcCommand("AddCountry");
            db.AddInParameter(dbAddCountry, "vCountryCode", DbType.String, "CountryCode", DataRowVersion.Current);
            db.AddInParameter(dbAddCountry, "vCountryName", DbType.String, "CountryName", DataRowVersion.Current);
            DbCommand dbUpdateCountry = db.GetStoredProcCommand("UpdateCountry");
            db.AddInParameter(dbUpdateCountry, "vCountryCode", DbType.String, "CountryCode", DataRowVersion.Current);
            db.AddInParameter(dbUpdateCountry, "vCountryName", DbType.String, "CountryName", DataRowVersion.Current);
            DbCommand dbDeleteCountry = db.GetStoredProcCommand("DeleteCountry");
            db.AddInParameter(dbDeleteCountry, "vCountryCode", DbType.String, "CountryCode", DataRowVersion.Current);

            DbCommand dbAddEmployee = db.GetStoredProcCommand("AddEmployees");

            db.LoadDataSet(CommandType.Text, "select CountryCode,CountryName from Country", ds, new string[] { "Country" });

            DataRow row = ds.Tables[0].NewRow();
            row["CountryCode"] = "FRA";
            row["CountryName"] = "France";

            ds.Tables[0].Rows.Add(row);

            db.UpdateDataSet(ds, "Country", dbAddCountry, dbUpdateCountry, dbDeleteCountry, UpdateBehavior.Transactional);

            using (DataSet dsResult = db.ExecuteDataSet(CommandType.Text, "select * from Country where CountryCode in ('FRA')"))
            {
                Assert.AreEqual("France", Convert.ToString(dsResult.Tables[0].Rows[0]["CountryName"]));
            }
        }

        [TestMethod]
        public void CommandWrapperWithSourceColumnTest()
        {
            Database db = DatabaseFactory.CreateDatabase("OracleTest");
            DataSet ds = new DataSet();

            DbCommand dbAddCountry = db.GetStoredProcCommandWithSourceColumns("AddCountry", new string[] {"CountryCode","CountryName" });
            DbCommand dbUpdateCountry = db.GetStoredProcCommandWithSourceColumns("UpdateCountry", new string[] { "CountryCode", "CountryName" });
            DbCommand dbDeleteCountry = db.GetStoredProcCommandWithSourceColumns("DeleteCountry", new string[] { "CountryCode"});

            db.LoadDataSet(CommandType.Text, "select * from Country", ds, new string[] { "Country" });

            ds.Tables[0].PrimaryKey = new DataColumn[] { ds.Tables[0].Columns["CountryCode"] };

            DataRow row = ds.Tables[0].NewRow();
            row["CountryCode"] = "RUS";
            row["CountryName"] = "Russia";

            ds.Tables[0].Rows.Add(row);

            DataRow modifyRow = ds.Tables[0].Rows.Find("RUS");
            modifyRow["CountryName"] = "RUSSIA";

            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                try
                {
                    db.UpdateDataSet(ds, "Country", dbAddCountry, dbUpdateCountry, dbDeleteCountry, transaction);
                    transaction.Commit();
                }
                catch (Exception )
                {
                    transaction.Rollback();
                }
            }

            using (DataSet dsResult = db.ExecuteDataSet(CommandType.Text, "select * from Country where CountryCode in ('RUS')"))
            {
                DataRow result = dsResult.Tables[0].Rows[0];
                Assert.IsNotNull(result);
                Assert.AreEqual("RUSSIA", Convert.ToString(result["CountryName"]));    
            }
        }
    }
}
