using System;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Data.Sql;
using System.Data;
using System.Data.SqlClient;
using System.Text;
using System.Data.Common;
using System.Configuration;

namespace DataFuncTests.SqlDatabaseFixtures
{
    /// <summary>
    /// Tests the ExecuteScalar Method of the Database Class
    /// </summary>
    [TestClass]
    public class ExecuteScalarTestFixture
    {
        string connStr;
        String itemsXMLfile;
        DataSet dsExpectedResult = new DataSet();
        public ExecuteScalarTestFixture()
        {
        }
        /// <summary>
        /// Initialize() is called once during test execution before
        /// test methods in this test class are executed.
        /// </summary>
        [TestInitialize()]
        public void Initialize()
        {
           connStr = ConfigurationManager.ConnectionStrings["DataSQLTest"].ConnectionString;
            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("begin Delete from Items where itemId > 3 " +
                                                  "Update Items set ItemDescription='Digital Image Pro',QtyInHand = 25, QtyRequired=100 where ItemId = 1 " +
                                                   "Update Items set ItemDescription='Excel 2003',QtyInHand = 95, QtyRequired=100 where ItemId = 2 " +
                                                   "Update Items set ItemDescription='Infopath',QtyInHand = 34, QtyRequired=100 where ItemId = 3 " +
                                                   "delete from CustomersOrders where CustomerId > 3 " +
                                                   "Update CustomersOrders set QtyOrdered=100 " +
                                            "end", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            dr.Close();
            conn.Close();
            itemsXMLfile = "Items.xml";
            dsExpectedResult = new DataSet();
            dsExpectedResult.ReadXml(itemsXMLfile);
        }
        /// <summary>
        /// Cleanup() is called once during test execution after
        /// test methods in this class have executed unless the
        /// corresponding Initialize() call threw an exception.
        /// </summary>
        [TestCleanup()]
        public void Cleanup()
        {
            //  TODO: Add test cleanup code
        }
        #region "DB Command"
        [TestMethod]
        public void ExecuteScalarDBCommandSqlTest()
        {
            Database db = DatabaseFactory.CreateDatabase("DataSQLTest");
            string sqlCommand = "select ItemDescription from items where ItemID=1";
            DbCommand dbCommandWrapper = db.GetSqlStringCommand(sqlCommand);
            object actualResult = db.ExecuteScalar(dbCommandWrapper);
            Assert.AreEqual(actualResult.ToString().Trim(), dsExpectedResult.Tables[0].Rows[0][0].ToString());
        }
        [TestMethod]
        public void ExecuteScalarDBCommandSPTest()
        {
            Database db = DatabaseFactory.CreateDatabase("DataSQLTest");
            string spName = "ItemsDescriptionGet";
            DbCommand dbCommandWrapper = db.GetStoredProcCommand(spName);
            object actualResult = db.ExecuteScalar(dbCommandWrapper);
            Assert.AreEqual(actualResult.ToString().Trim(), dsExpectedResult.Tables[0].Rows[0][0].ToString());
        }

        [TestMethod]
        public void ExecuteScalarDBCommandSPINSqlTypeTest()
        {
            string spName = "CustomerOrdersAddwithParm";
            SqlDatabase db = (SqlDatabase)DatabaseFactory.CreateDatabase("DataSQLTest");
            DbCommand dbCommandWrapper = db.GetStoredProcCommand(spName);
            db.AddInParameter(dbCommandWrapper, "@CustomerId", SqlDbType.Int , "19");
            db.AddInParameter(dbCommandWrapper, "@CustomerName",SqlDbType.VarChar , "google");
            db.AddInParameter(dbCommandWrapper, "@ItemId",SqlDbType.Int,   3);
            db.AddInParameter(dbCommandWrapper, "@QtyOrdered", SqlDbType.Int, 300);
            object actualResult = db.ExecuteScalar(dbCommandWrapper);
            Assert.AreEqual(Convert.ToInt32(actualResult.ToString().Trim()), 300);
        }

        [TestMethod]
        [ExpectedException(typeof(SqlException))]
        public void ExecuteScalarDBCmdNoSPPresentTest()
        {
            Database db = DatabaseFactory.CreateDatabase("DataSQLTest");
            string sqlCommand = "UpdateItem1";
            DbCommand dbCommandWrapper = db.GetStoredProcCommand(sqlCommand);
            object actualResult = db.ExecuteScalar(dbCommandWrapper);
            { }
            Assert.AreEqual(ConnectionState.Closed, dbCommandWrapper.Connection.State);
        }
        #endregion

        #region "Command Type, Command Text"
        [TestMethod]
        public void ExecuteScalarTypeTextSqlTest()
        {
            Database db = DatabaseFactory.CreateDatabase("DataSQLTest");
            string sqlCommand = "select ItemDescription from items where ItemID=1";
            DbCommand dbCommandWrapper = db.GetSqlStringCommand(sqlCommand);
            object actualResult = db.ExecuteScalar(CommandType.Text, sqlCommand);
            Assert.AreEqual(actualResult.ToString().Trim(), dsExpectedResult.Tables[0].Rows[0][0].ToString());
        }
        [TestMethod]
        public void ExecuteScalarTypeTextSPtest()
        {
            Database db = DatabaseFactory.CreateDatabase("DataSQLTest");
            string spName = "ItemsDescriptionGet";
            DbCommand dbCommandWrapper = db.GetSqlStringCommand(spName);
            object actualResult = db.ExecuteScalar(CommandType.StoredProcedure, spName);
            Assert.AreEqual(actualResult.ToString().Trim(), dsExpectedResult.Tables[0].Rows[0][0].ToString());
        }

        #endregion

        #region "Command, Transaction"
        [TestMethod]
        public void ExecuteScalarDBCmdTransCommitTest()
        {
            Database db = DatabaseFactory.CreateDatabase("DataSQLTest");
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                DbCommand dbCommandWrapper = db.GetStoredProcCommand("CustomerOrdersAddwithParm");
                this.CreateCommandParameter(db,dbCommandWrapper, "@CustomerId", DbType.Int32, ParameterDirection.Input, "20");
                this.CreateCommandParameter(db,dbCommandWrapper, "@CustomerName", DbType.String, ParameterDirection.Input, "MSoft");
                this.CreateCommandParameter(db,dbCommandWrapper, "@ItemId", DbType.Int32, ParameterDirection.Input, "2");
                this.CreateCommandParameter(db,dbCommandWrapper, "@QtyOrdered", DbType.Int32, ParameterDirection.Input, "500");
                object actualResult = db.ExecuteScalar(dbCommandWrapper, transaction);
                transaction.Commit();
                Assert.AreEqual(Convert.ToInt32(actualResult.ToString().Trim()), 500);
                connection.Close();
            }
        }
        [TestMethod]
        public void ExecuteScalarDBCmdTransRollbackTest()
        {
            Database db = DatabaseFactory.CreateDatabase("DataSQLTest");
            using (DbConnection connection = db.CreateConnection())
            {
                bool value = true;
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                DbCommand dbCommandWrapper = db.GetStoredProcCommand("CustomerOrdersAddwithParm");
                this.CreateCommandParameter(db,dbCommandWrapper, "@CustomerId", DbType.Int32, ParameterDirection.Input, "21");
                this.CreateCommandParameter(db,dbCommandWrapper, "@CustomerName", DbType.String, ParameterDirection.Input, "oracle");
                this.CreateCommandParameter(db,dbCommandWrapper, "@ItemId", DbType.Int32, ParameterDirection.Input, "2");
                this.CreateCommandParameter(db,dbCommandWrapper, "@QtyOrdered", DbType.Int32, ParameterDirection.Input, "500");
                db.ExecuteScalar(dbCommandWrapper, transaction);
                transaction.Rollback();
                connection.Close();
                SqlConnection conn = new SqlConnection();
                StringBuilder readerDataExpected = new StringBuilder();
                conn.ConnectionString = connStr;
                conn.Open();
                SqlCommand cmd = new SqlCommand("Select QtyOrdered from CustomersOrders where customerName='oracle'", conn);
                SqlDataReader dr = cmd.ExecuteReader();
                while (dr.Read())
                {
                    readerDataExpected.Append(dr["QtyOrdered"]);
                    value = false;
                }
                conn.Close();
                Assert.IsTrue(value);
            }
        }
        #endregion

        #region "SP, Parm"
        [TestMethod]
        public void ExecuteScalarSpParamTest()
        {
            Database db = DatabaseFactory.CreateDatabase("DataSQLTest");
            string spName = "CustomerOrdersAddwithParm";
            object actualResult = db.ExecuteScalar(spName, new object[] { 22, "apple", 1, 800 });
            Assert.AreEqual(Convert.ToInt32(actualResult.ToString().Trim()), 800);
        }
        #endregion

        #region "Transaction, SP, Parm"
        [TestMethod]
        public void ExecuteScalarTransSpParamCommitTest()
        {
            Database db = DatabaseFactory.CreateDatabase("DataSQLTest");
            string spName = "CustomerOrdersAddwithParm";
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                object actualResult = db.ExecuteScalar(transaction, spName, new object[] { 23, "airtel", 1, 900 });
                transaction.Commit();
                Assert.AreEqual(Convert.ToInt32(actualResult.ToString().Trim()), 900);
                connection.Close();
            }
        }
        [TestMethod]
        public void ExecuteScalarTransSpParamRollbackTest()
        {
            Database db = DatabaseFactory.CreateDatabase("DataSQLTest");
            string spName = "CustomerOrdersAddwithParm";
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                db.ExecuteScalar(transaction, spName, new object[] { 24, "aircel", 1, 1000 });
                transaction.Rollback();
                connection.Close();
                SqlConnection conn = new SqlConnection();
                StringBuilder readerDataExpected = new StringBuilder();
                conn.ConnectionString = connStr;
                conn.Open();
                SqlCommand cmd = new SqlCommand("Select QtyOrdered from CustomersOrders where customerName='aircel'", conn);
                SqlDataReader dr = cmd.ExecuteReader();
                while (dr.Read())
                {
                    readerDataExpected.Append(dr["QtyOrdered"]);
                }
                conn.Close();
                Assert.AreEqual(readerDataExpected.ToString(), "");
            }
        }

        #endregion

        #region "Transaction, CommandType, Command Text"
        [TestMethod]
        public void ExecuteScalarTransSqlCommitTest()
        {
            Database db = DatabaseFactory.CreateDatabase("DataSQLTest");
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                object actualResult = db.ExecuteScalar(transaction, CommandType.Text, "begin Insert into CustomersOrders values(5,'orange',3,600) Select count(*) from CustomersOrders where CustomerID='5' end");
                transaction.Commit();
                Assert.AreEqual(Convert.ToInt32(actualResult.ToString().Trim()), 1);
                connection.Close();
            }
        }
        [TestMethod]
        public void ExecuteScalarTransSqlRollbackTest()
        {
            Database db = DatabaseFactory.CreateDatabase("DataSQLTest");
            using (DbConnection connection = db.CreateConnection())
            {
                bool value = true;
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                db.ExecuteScalar(transaction, CommandType.Text, "Insert into CustomersOrders values(6,'Hutch',3,200) ");
                transaction.Rollback();
                connection.Close();
                SqlConnection conn = new SqlConnection();
                StringBuilder readerDataActual = new StringBuilder();
                conn.ConnectionString = connStr;
                conn.Open();
                SqlCommand cmd = new SqlCommand("Select CustomerName from CustomersOrders where CustomerID='6' ", conn);
                SqlDataReader dr = cmd.ExecuteReader();
                while (dr.Read())
                {
                    readerDataActual.Append(dr["CustomerName"]);
                    value = false;
                }
                conn.Close();
                Assert.IsTrue(value);
            }
        }
        [TestMethod]
        public void ExecuteScalarTransSpCommitTest()
        {
            Database db = DatabaseFactory.CreateDatabase("DataSQLTest");
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                string spName = "CustomerOrdersAdd";
                DbTransaction transaction = connection.BeginTransaction();
                object actualResult = db.ExecuteScalar(transaction, CommandType.StoredProcedure, spName);
                transaction.Commit();
                Assert.AreEqual(actualResult.ToString().Trim(), "David");
                connection.Close();
            }
        }
        [TestMethod]
        public void ExecuteScalarTransSpRollbackTest()
        {
            Database db = DatabaseFactory.CreateDatabase("DataSQLTest");
            using (DbConnection connection = db.CreateConnection())
            {
                bool value = true;
                string spName = "OrderUpdate";
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                db.ExecuteScalar(transaction, CommandType.StoredProcedure, spName);
                transaction.Rollback();
                connection.Close();
                SqlConnection conn = new SqlConnection();
                StringBuilder readerDataActual = new StringBuilder();
                conn.ConnectionString = connStr;
                conn.Open();
                SqlCommand cmd = new SqlCommand("Select CustomerName from CustomersOrders where QtyOrdered=150", conn);
                SqlDataReader dr = cmd.ExecuteReader();
                while (dr.Read())
                {
                    readerDataActual.Append(dr["QtyOrdered"]);
                    value = false;
                }
                conn.Close();
                Assert.IsTrue(value);
            }
        }
        #endregion

        # region Helper Method
        private void CreateCommandParameter(Database db, DbCommand comm, string Parametername, DbType dbType, ParameterDirection dir, object value)
        {

            if (dir == ParameterDirection.Input)
            {
                db.AddInParameter(comm, Parametername, dbType, value);
            }
            else if (dir == ParameterDirection.Output)
            {
                db.AddOutParameter(comm, Parametername, dbType, (int)value);
            }

            //DbParameter cmd = comm.CreateParameter();
            //cmd.ParameterName = Parametername;
            //cmd.Value = value;
            //cmd.Direction = dir;
            //cmd.DbType = dbType;
            //return (cmd);
        }

        # endregion
        
    }
}

