using System;
using System.IO;
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Configuration;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling;
using System.Data.SqlClient;
using System.Collections.Specialized;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;

namespace ExceptionHandlingCoreTests
{
    /// <summary>
    /// Summary description for ExceptionHandlingTestFixture
    /// </summary>
    [TestClass]
    public class ExceptionHandlingTestFixture
    {
        public ExceptionHandlingTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        /// <summary>
        /// Initialize() is called once during test execution before
        /// test methods in this test class are executed.
        /// </summary>
        [TestInitialize()]
        public void Initialize()
        {
            //  TODO: Add test initialization code
        }

        /// <summary>
        /// Cleanup() is called once during test execution after
        /// test methods in this class have executed unless the
        /// corresponding Initialize() call threw an exception.
        /// </summary>
        [TestCleanup()]
        public void Cleanup()
        {
            //  TODO: Add test cleanup code
        }



        [TestMethod]
        public void NoExceptionTypeTest()
        {
            Exception originalException = new System.Security.SecurityException("Test No ExceptionType in Config File");

            bool rethrow = ExceptionPolicy.HandleException(originalException, "No Exception Type");
            Assert.IsTrue(rethrow);
        }

        [TestMethod]
        public void NoHandlerwithRethrowTest()
        {
            Exception originalException = new System.Security.SecurityException("Test No Handler with Posthandling Rethrow in Config");
            bool rethrow = ExceptionPolicy.HandleException(originalException, "No Handler with Rethrow");
            Assert.IsTrue(rethrow);
        }

        [TestMethod]
        [ExpectedException(typeof(System.Security.SecurityException))]
        public void NoHandlerwithThrowNewTest()
        {
            Exception originalException = new System.Security.SecurityException("No handlers with Posthandling ThrowNewException in Config");

            bool rethrow = ExceptionPolicy.HandleException(originalException, "No Handler with Throw New");
            Assert.IsTrue(rethrow);
        }

        [TestMethod]
        public void DerivedExceptionTest()
        {
            bool exceptionRaised = false;
            bool rethrow = false;
            try
            {
                Exception originalException = new System.IO.FileNotFoundException("Original Exception: IO Exception");
                rethrow = ExceptionPolicy.HandleException(originalException, "Derived Exception Test");
                if (rethrow) throw originalException;

            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Replaced Exception: IO Exception Replaced with Security Exception", e.Message);
                Assert.AreEqual("SecurityException", e.GetType().Name);
            }

            Assert.AreEqual(true, exceptionRaised); 
            exceptionRaised = false;
            try
            {
                Exception originalException = new System.ArithmeticException("Original Exception: Arithmetic Exception");
                rethrow = ExceptionPolicy.HandleException(originalException, "Derived Exception Test");
                if (rethrow) throw originalException;

            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Replaced Exception: Arithmetic Exception Replaced with Verification Exception", e.Message);
                Assert.AreEqual("VerificationException", e.GetType().Name);
            }

            Assert.AreEqual(true, exceptionRaised);
        }

        [TestMethod]
        public void NoHandlerwithNoneTest()
        {
            Exception originalException = new System.Security.SecurityException("No Handlers with Posthandling none in config");
            bool rethrow = ExceptionPolicy.HandleException(originalException, "No Handler with None");
            Assert.IsFalse(rethrow);

        }

        [TestMethod]
        [ExpectedException(typeof(System.Configuration.ConfigurationErrorsException))]
        public void ExceptionHandlingExTest()
        {
            Exception originalException = new System.Security.SecurityException("No Handlers with Posthandling none in config");
            bool rethrow = ExceptionPolicy.HandleException(originalException, "Policy does not exist");
            

        }

        [TestMethod]
        [ExpectedException(typeof(System.Security.SecurityException))]
        public void ExceptionHandlingExceptionTest()
        {
            Exception originalException = new System.IO.FileNotFoundException("Original Exception: IO Exception");
            ExceptionPolicy.HandleException(originalException, "Derived Exception Test");
        }
        


        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void NoExceptionTest()
        {
            Exception originalException = null;
            bool rethrow = ExceptionPolicy.HandleException(originalException, "Wrap ThrowNew Different Assembly");


        }

        [TestMethod]
        public void ExceptionTypeNotFoundTest()
        {
            Exception originalException = new System.Data.DataException();
            bool rethrow = ExceptionPolicy.HandleException(originalException, "Wrap ThrowNew Different Assembly");
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void NoPolicyTest()
        {
            Exception originalException = new System.Security.SecurityException("No Policy defined"); 
            bool rethrow = ExceptionPolicy.HandleException(originalException, null);


        }

       
    }

}
