using System;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling;
using System.Data.SqlClient;
using System.IO;
using System.Diagnostics;
using System.Configuration;
using System.Messaging;

namespace ExceptionHandlingLoggingTests
{
    /// <summary>
    /// Summary description for WrapHandlerLogTestFixture
    /// </summary>
    [TestClass]
    public class WrapHandlerLogTestFixture
    {
        string msmqPath;
        public WrapHandlerLogTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        /// <summary>
        /// Initialize() is called once during test execution before
        /// test methods in this test class are executed.
        /// </summary>
        [TestInitialize()]
        public void Initialize()
        {
            msmqPath = ".\\Private$\\myQueue";
            if (!MessageQueue.Exists(msmqPath))
                MessageQueue.Create(msmqPath);
        }

        /// <summary>
        /// Cleanup() is called once during test execution after
        /// test methods in this class have executed unless the
        /// corresponding Initialize() call threw an exception.
        /// </summary>
        [TestCleanup()]
        public void Cleanup()
        {
            //  TODO: Add test cleanup code
        }

        #region "Multiple Handlers"


        [TestMethod]
        public void WrapLogToFileThrowNewTest()
        {
            bool exceptionRaised = false;
            bool isPresent = false;
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception Occurred");

            bool rethrow = false;
            try
            {
                rethrow = ExceptionPolicy.HandleException(originalException, "Wrap And Log To File ThrowNew");
                if (rethrow) throw originalException;


            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Security exception 1 wrapped by Verification exception", e.Message);
                Assert.AreEqual("Original Exception: Security Exception Occurred", e.InnerException.Message);
                Assert.AreEqual("VerificationException", e.GetType().Name);
                Assert.AreEqual("SecurityException", e.InnerException.GetType().Name);
            }
            Assert.IsFalse(rethrow);
            if (File.Exists("FlatFileTestOutput\\TextWriter.txt") == true)
            {
                isPresent = true;
                //File.Delete("FlatFileTestOutput\\TextWriter.txt");
            }
            Assert.AreEqual(true, isPresent);
            Assert.AreEqual(true, exceptionRaised);




        }
        [TestMethod]
        public void WrapLogToFileNoneTest()
        {
            bool isPresent = false;
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception Occurred");
            
            bool rethrow = false;
            rethrow = ExceptionPolicy.HandleException(originalException, "Wrap And Log To File NoneAction");
            Assert.IsFalse(rethrow);

            if (File.Exists("FlatFileTestOutput\\SimpleEntLibLog.txt") == true)
            {
                isPresent = true;
                //File.Delete("FlatFileTestOutput\\SimpleEntLibLog.txt");
            }
            Assert.AreEqual(true, isPresent);

        }

        [TestMethod]
        public void WrapLogToEventSinkNoneActionTest()
        {
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception wrapped by IO Exception which is logged into event sink");

            bool rethrow = ExceptionPolicy.HandleException(originalException, "Wrap And Log To Event Sink NoneAction");

            Assert.IsFalse(rethrow);

            string logName = EventLog.LogNameFromSourceName("ExceptionHandlingLogging", ".");
            using (EventLog eventlog = new EventLog(logName))
            {

                int entryCount = eventlog.Entries.Count;

                Assert.AreEqual("ExceptionHandlingLogging", eventlog.Entries[eventlog.Entries.Count - 1].Source);
                Assert.AreEqual(Convert.ToInt64(130), eventlog.Entries[eventlog.Entries.Count - 1].InstanceId);

            }

        }

        [TestMethod]
        public void WrapLogToEventSinkThrowNewTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception wrapped by IO Exception which is logged into event sink");

            bool rethrow = false;
            try
            {
                rethrow = ExceptionPolicy.HandleException(originalException, "Wrap And Log To Event Sink ThrowNew");
                if (rethrow) throw originalException;

            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Security exception 4 wrapped by IO exception", e.Message);
                Assert.AreEqual("Original Exception: Security Exception wrapped by IO Exception which is logged into event sink", e.InnerException.Message);
                Assert.AreEqual("IOException", e.GetType().Name);
                Assert.AreEqual("SecurityException", e.InnerException.GetType().Name);
            }
            Assert.IsFalse(rethrow);
            string logName = EventLog.LogNameFromSourceName("Entlib Tests", ".");
            using (EventLog eventlog = new EventLog(logName))
            {

                int entryCount = eventlog.Entries.Count;

                Assert.AreEqual("Entlib Tests", eventlog.Entries[eventlog.Entries.Count - 1].Source);
                Assert.AreEqual(Convert.ToInt64(131), eventlog.Entries[eventlog.Entries.Count - 1].InstanceId);

            }
            Assert.AreEqual(true, exceptionRaised);

        }


        [TestMethod]
        public void WrapLogToEventSinkThrowNewTestXmlFormat()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception wrapped by IO Exception which is logged into event sink");

            bool rethrow = false;
            try
            {
                rethrow = ExceptionPolicy.HandleException(originalException, "Wrap And Log To Event Sink ThrowNew in XmlFormat");
                if (rethrow) throw originalException;

            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Security exception 4 wrapped by IO exception", e.Message);
                Assert.AreEqual("Original Exception: Security Exception wrapped by IO Exception which is logged into event sink", e.InnerException.Message);
                Assert.AreEqual("IOException", e.GetType().Name);
                Assert.AreEqual("SecurityException", e.InnerException.GetType().Name);
            }
            Assert.IsFalse(rethrow);
            string logName = EventLog.LogNameFromSourceName("ExceptionHandlingLogging", ".");
            using (EventLog eventlog = new EventLog(logName))
            {

                int entryCount = eventlog.Entries.Count;

                Assert.AreEqual("ExceptionHandlingLogging", eventlog.Entries[eventlog.Entries.Count - 1].Source);
                Assert.AreEqual(Convert.ToInt64(131), eventlog.Entries[eventlog.Entries.Count - 1].InstanceId);

            }
            Assert.AreEqual(true, exceptionRaised);

        }

        [TestMethod]
        public void WrapLogToFileThrowNewTestXmlFormat()
        {
            bool exceptionRaised = false;
            bool isPresent = false;
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception Occurred");

            bool rethrow = false;
            try
            {
                rethrow = ExceptionPolicy.HandleException(originalException, "Wrap And Log To File ThrowNew in XmlFormat");
                if (rethrow) throw originalException;
            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Security exception 1 wrapped by Verification exception", e.Message);
                Assert.AreEqual("Original Exception: Security Exception Occurred", e.InnerException.Message);
                Assert.AreEqual("VerificationException", e.GetType().Name);
                Assert.AreEqual("SecurityException", e.InnerException.GetType().Name);
            }
            Assert.IsFalse(rethrow);
            if (File.Exists("FlatFileTestOutput\\EntLibLog.xml") == true)
            {
                isPresent = true;
                //File.Delete("FlatFileTestOutput\\EntLibLog.xml");
            }
            Assert.AreEqual(true, isPresent);
            Assert.AreEqual(true, exceptionRaised);

        }

        #endregion
    }
}
