using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.IO;
using System.Diagnostics;
using System.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Security;
using Microsoft.Practices.EnterpriseLibrary.Security.AzMan;
using System.Security.Principal;

namespace FileSourceAppSecurityTests
{
    /// <summary>
    /// Summary description for SecurityAppTestFixture
    /// </summary>
    [TestClass]
    public class SecurityAppTestFixture
    {
        public SecurityAppTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void SecurityAzManTaskAuthorizeAppTest()
        {
            IAuthorizationProvider azManProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultAzManProvider");
            WindowsIdentity identity = WindowsIdentity.GetAnonymous();

            bool isAuthorized = azManProvider.Authorize(new WindowsPrincipal(identity), "Authorize Purchase");
            Assert.IsTrue(isAuthorized);

        }


        [TestMethod]
        public void SecurityInMemoryAddIdentityAppTest()
        {
            ISecurityCacheProvider securityCache = SecurityCacheFactory.GetSecurityCacheProvider("CacheProvider");
            Assert.IsNotNull(securityCache);
            IIdentity identity = new GenericIdentity("testuser", "SecurityCache");
            IToken token = securityCache.SaveIdentity(identity);
            Assert.IsNotNull(token);
            Assert.IsNotNull(token.Value);

            IIdentity cachedIdentity = securityCache.GetIdentity(token);
            Assert.IsNotNull(cachedIdentity);
            Assert.AreEqual(cachedIdentity.Name, "testuser");
        }


        [TestMethod]
        public void SecurityIsolatedAddIdentityAppTest()
        {
            ISecurityCacheProvider securityCache = SecurityCacheFactory.GetSecurityCacheProvider("CacheProviderIsolated");
            Assert.IsNotNull(securityCache);

            IIdentity identity = new GenericIdentity("testuser", "SecurityCache");
            IToken token = securityCache.SaveIdentity(identity);
            Assert.IsNotNull(token);
            Assert.IsNotNull(token.Value);

            IIdentity cachedIdentity = securityCache.GetIdentity(token);
            Assert.IsNotNull(cachedIdentity);
            Assert.AreEqual(cachedIdentity.Name, "testuser");
        }

        [TestMethod]
        public void SecurityDBAddIdentityAppTest()
        {
            ISecurityCacheProvider securityCache = SecurityCacheFactory.GetSecurityCacheProvider("CacheProviderDB");
            Assert.IsNotNull(securityCache);

            IIdentity identity = new GenericIdentity("testuser", "SecurityCache");
            IToken token = securityCache.SaveIdentity(identity);
            Assert.IsNotNull(token);
            Assert.IsNotNull(token.Value);

            IIdentity cachedIdentity = securityCache.GetIdentity(token);
            Assert.IsNotNull(cachedIdentity);
            Assert.AreEqual(cachedIdentity.Name, "testuser");
        }

        [TestMethod]
        public void SecurityAuthRuleIdentityAppTest()
        {
            IIdentity identity = new GenericIdentity("TestUser");
            string[] roles = new string[] { "Manager" };
            IPrincipal principal = new GenericPrincipal(identity, roles);

            AuthorizationRuleProvider authRuleProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultRuleProvider") as AuthorizationRuleProvider;
            bool authorized = authRuleProvider.Authorize(principal, "TestIdentityRule");
            Assert.IsTrue(authorized);
        }
    }
}
