using System;
using System.Text;
using System.Collections.Generic;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using System.Messaging;
using System.Diagnostics;
using System.Configuration;
using System.Management;
using System.Net;
using System.IO;
using System.Data.Common;
using Microsoft.Practices.EnterpriseLibrary.Logging;
using Microsoft.Practices.EnterpriseLibrary.Logging.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Logging.Formatters;
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Data.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Logging.Database;
using Microsoft.Practices.EnterpriseLibrary.Logging.Database.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Logging.TraceListeners;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration.ObjectBuilder;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Win32;
using CustomClasses;
using CustomClasses.Configuration;  

namespace ConfigurationSourceTests
{
    /// <summary>
    /// Summary description for DictionarySourceFormatterTestFixture
    /// </summary>
    [TestClass]
    public class DictionarySourceFormatterTestFixture
    {
        private DictionaryConfigurationSource source = null;

        public DictionarySourceFormatterTestFixture()
        {
            source = LoggingConfigurationSource.BuildConfigurationSource(); 
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion


        #region TextFormatter Tests
        [TestMethod]
        public void TextFormatterEventLogTest()
        {
            LogEntry log = new LogEntry();
            log.Title = "TextFormatterEventLogTest";
            log.Priority = 3;
            log.EventId = 100;
            log.AddErrorMessage("AdditionalInformation");
            log.Categories.Add("General");
            log.Severity = System.Diagnostics.TraceEventType.Critical;
            string loggedMessage = "";
            using (EventLog eventLog = new EventLog(HelperClass.GetEventLogName()))
            {
                int initialCount = eventLog.Entries.Count;
                LogWriterFactory factory = new LogWriterFactory(source);
                using (LogWriter writer = factory.Create())
                {
                    writer.Write(log);
                }

                int finalCount = eventLog.Entries.Count;
                Assert.IsTrue(finalCount - initialCount == 1);
                loggedMessage = eventLog.Entries[finalCount - 1].Message;
            }
            Assert.IsTrue(VerifyTextFormat(log, loggedMessage));
        }

        [TestMethod]
        public void TextFormatterMsmqTest()
        {
            LogEntry log = new LogEntry();
            log.Title = "TextFormatterMsmqTest";
            log.Message = "To test Text formatter for Msmq";
            log.Priority = 3;
            log.EventId = 100;
            log.AddErrorMessage("AdditionalInformation");
            log.Categories.Add("MsmqCategory");
            log.Severity = System.Diagnostics.TraceEventType.Critical;
            string loggedMessage = "";

            using (MessageQueue queue = new MessageQueue(@".\Private$\EntlibTest"))
            {
                queue.Purge();
                queue.Formatter = new XmlMessageFormatter();
                string[] types = { "System.String" };
                ((XmlMessageFormatter)queue.Formatter).TargetTypeNames = types;
                LogWriterFactory factory = new LogWriterFactory(source);
                using (LogWriter writer = factory.Create())
                {
                    writer.Write(log);
                }
                Message message = queue.Receive(new TimeSpan(0,0,5));
                Assert.IsNotNull(message);
                loggedMessage = (string)message.Body;
            }
            Assert.IsTrue(VerifyTextFormat(log, loggedMessage));
        }

        [TestMethod]
        public void TextFormatterDbTest()
        {
            LogEntry log = new LogEntry();
            log.Title = "TextFormatterDbTest";
            log.Message = "To test Text formatter for Databae Listener";
            log.Priority = 3;
            log.EventId = 100;
            log.AddErrorMessage("AdditionalInformation");
            log.Categories.Add("DatabaseCategory");
            log.Severity = System.Diagnostics.TraceEventType.Critical;
            string loggedMessage = "";
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                writer.Write(log);
            }
            DatabaseProviderFactory dbFactory = new DatabaseProviderFactory(source);
            Database db = dbFactory.Create("SqlServer"); 
            DbCommand command = db.GetSqlStringCommand("SELECT TOP 1 FormattedMessage FROM Log ORDER BY LogId DESC");
            loggedMessage = Convert.ToString(db.ExecuteScalar(command));
            Assert.IsTrue(VerifyTextFormat(log, loggedMessage));
        }

        [TestMethod]
        public void TextFormatterCustomListenerTest()
        {
            LogEntry log = new LogEntry();
            log.Title = "TextFormatterCustomListenerTest";
            log.Message = "To test Text formatter for Custom Registry Listener";
            log.Priority = 3;
            log.EventId = 100;
            log.Categories.Add("RegistryCategory");
            log.AddErrorMessage("AdditionalInformation");
            log.Severity = System.Diagnostics.TraceEventType.Critical;
            string loggedMessage = "";
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                writer.Write(log);
            }
            RegistryKey key = Registry.LocalMachine.OpenSubKey(@"Software\EntlibTestApp");
            loggedMessage = (string)key.GetValue("LogEntry");
            key.Close();
            Assert.IsTrue(VerifyTextFormat(log, loggedMessage));
        }

        #endregion

        #region BinaryFormatter Tests
        [TestMethod]
        public void BinaryFormatterEventLogTest()
        {
            LoggingSettings setting = (LoggingSettings)source.GetSection("loggingConfiguration");
            FormattedEventLogTraceListenerData listener = (FormattedEventLogTraceListenerData) setting.TraceListeners.Get("Formatted EventLog TraceListener");
            string actualFormatter = listener.Formatter;  
            listener.Formatter = "Binary Formatter";  
            LogEntry log = new LogEntry();
            log.Title = "BinaryFormatterEventLogTest";
            log.Priority = 3;
            log.EventId = 100;
            log.AddErrorMessage("AdditionalInformation");
            log.Categories.Add("General");
            log.Severity = System.Diagnostics.TraceEventType.Critical;
            string loggedMessage = "";
            using (EventLog eventLog = new EventLog(HelperClass.GetEventLogName()))
            {
                int initialCount = eventLog.Entries.Count;
                LogWriterFactory factory = new LogWriterFactory(source);
                using (LogWriter writer = factory.Create())
                {
                    writer.Write(log);
                }

                int finalCount = eventLog.Entries.Count;
                Assert.IsTrue(finalCount - initialCount == 1);
                loggedMessage = eventLog.Entries[finalCount - 1].Message;
            }
            Assert.IsTrue(VerifyBinaryFormat(log, loggedMessage));
            listener.Formatter = actualFormatter;  
        }


        [TestMethod]
        public void BinaryFormatterMsmqTest()
        {
            LogEntry log = new LogEntry();
            log.Title = "BinaryFormatterMsmqTest";
            log.Message = "To test Binary formatter for Msmq";
            log.Priority = 3;
            log.EventId = 100;
            log.AddErrorMessage("AdditionalInformation");
            log.Categories.Add("FormatterCategory");
            log.Severity = System.Diagnostics.TraceEventType.Critical;
            string loggedMessage = "";

            using (MessageQueue queue = new MessageQueue(@".\Private$\EntlibTest"))
            {
                queue.Purge();
                queue.Formatter = new XmlMessageFormatter();
                string[] types = { "System.String" };
                ((XmlMessageFormatter)queue.Formatter).TargetTypeNames = types;
                LogWriterFactory factory = new LogWriterFactory(source);
                using (LogWriter writer = factory.Create())
                {
                    writer.Write(log);
                }
                Message message = queue.Receive(new TimeSpan(0,0,5));
                Assert.IsNotNull(message);
                loggedMessage = (string)message.Body;
            }
            Assert.IsTrue(VerifyBinaryFormat(log, loggedMessage));
        }

        [TestMethod]
        public void BinaryFormatterDbTest()
        {
            LoggingSettings setting = (LoggingSettings)source.GetSection("loggingConfiguration");
            FormattedDatabaseTraceListenerData listener = (FormattedDatabaseTraceListenerData)setting.TraceListeners.Get("Database Trace Listener");
            string actualFormatter = listener.Formatter;
            listener.Formatter = "Binary Formatter";  

            LogEntry log = new LogEntry();
            log.Title = "BinaryFormatterDbTest";
            log.Message = "To test Binary formatter for Databae Listener";
            log.Priority = 3;
            log.EventId = 100;
            log.AddErrorMessage("AdditionalInformation");
            log.Categories.Add("DatabaseCategory");
            log.Severity = System.Diagnostics.TraceEventType.Critical;
            string loggedMessage = "";
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                writer.Write(log);
            }
            DatabaseProviderFactory dbFactory = new DatabaseProviderFactory(source);
            Database db = dbFactory.Create("SqlServer");
            DbCommand command = db.GetSqlStringCommand("SELECT TOP 1 FormattedMessage FROM Log ORDER BY LogId DESC");
            loggedMessage = Convert.ToString(db.ExecuteScalar(command));
            Assert.IsTrue(VerifyBinaryFormat(log, loggedMessage));
            listener.Formatter = actualFormatter;  
        }

        [TestMethod]
        public void BinaryFormatterCustomListenerTest()
        {
            LoggingSettings setting = (LoggingSettings)source.GetSection("loggingConfiguration");
            CustomTraceListenerData listener = (CustomTraceListenerData)setting.TraceListeners.Get("Registry Listener");
            string actualFormatter = listener.Formatter;
            listener.Formatter = "Binary Formatter";  

            LogEntry log = new LogEntry();
            log.Title = "BinaryFormatterCustomListenerTest";
            log.Message = "To test Binary formatter for Custom Registry Listener";
            log.Priority = 3;
            log.EventId = 100;
            log.Categories.Add("RegistryCategory");
            log.AddErrorMessage("AdditionalInformation");
            log.Severity = System.Diagnostics.TraceEventType.Critical;
            string loggedMessage = "";
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                writer.Write(log);
            }
            RegistryKey key = Registry.LocalMachine.OpenSubKey(@"Software\EntlibTestApp");
            loggedMessage = (string)key.GetValue("LogEntry");
            key.Close();
            Assert.IsTrue(VerifyBinaryFormat(log, loggedMessage));
            listener.Formatter = actualFormatter;  
        }
        #endregion

        #region CustomFormatter Tests
        [TestMethod]
        public void CustomFormatterEventLogTest()
        {
            CustomLogEntry log = new CustomLogEntry();
            log.Title = "TextFormatterEventLogTest";
            log.Message = "To test Custom Formatter for event Log";
            log.Priority = 3;
            log.EventId = 100;
            log.CategoryID = 25;
            log.AddErrorMessage("AdditionalInformation");
            log.Categories.Add("FormatterCategory");
            log.Severity = System.Diagnostics.TraceEventType.Critical;
            string loggedMessage = "";
            using (EventLog eventLog = new EventLog(HelperClass.GetEventLogName()))
            {
                int initialCount = eventLog.Entries.Count;
                LogWriterFactory factory = new LogWriterFactory(source);
                using (LogWriter writer = factory.Create())
                {
                    writer.Write(log);
                }
                int finalCount = eventLog.Entries.Count;
                Assert.IsTrue(finalCount - initialCount == 1);
                loggedMessage = eventLog.Entries[finalCount - 1].Message;
            }
            Assert.IsTrue(VerifyCustomFormat(log, loggedMessage));
        }

        [TestMethod]
        public void CustomFormatterMsmqTest()
        {
            LoggingSettings setting = (LoggingSettings)source.GetSection("loggingConfiguration");
            MsmqTraceListenerData listener = (MsmqTraceListenerData)setting.TraceListeners.Get("Msmq TraceListener");
            string actualFormatter = listener.Formatter;
            listener.Formatter = "CustomFormatter";  

            LogEntry log = new LogEntry();
            log.Title = "CustomFormatterMsmqTest";
            log.Message = "To test Custom formatter for Msmq";
            log.Priority = 3;
            log.EventId = 100;
            log.AddErrorMessage("AdditionalInformation");
            log.Categories.Add("MsmqCategory");
            log.Severity = System.Diagnostics.TraceEventType.Critical;
            string loggedMessage = "";

            using (MessageQueue queue = new MessageQueue(@".\Private$\EntlibTest"))
            {
                queue.Purge();
                queue.Formatter = new XmlMessageFormatter();
                string[] types = { "System.String" };
                ((XmlMessageFormatter)queue.Formatter).TargetTypeNames = types;
                LogWriterFactory factory = new LogWriterFactory(source);
                using (LogWriter writer = factory.Create())
                {
                    writer.Write(log);
                }
                Message message = queue.Receive(new TimeSpan(0,0,5));
                Assert.IsNotNull(message);
                loggedMessage = (string)message.Body;
            }
            Assert.IsTrue(VerifyCustomFormat(log, loggedMessage));
            listener.Formatter = actualFormatter;  
        }

        [TestMethod]
        public void CustomFormatterDbTest()
        {
            LoggingSettings setting = (LoggingSettings)source.GetSection("loggingConfiguration");
            FormattedDatabaseTraceListenerData listener = (FormattedDatabaseTraceListenerData)setting.TraceListeners.Get("Database Trace Listener");
            string actualFormatter = listener.Formatter;
            listener.Formatter = "CustomFormatter";

            LogEntry log = new LogEntry();
            log.Title = "CustomFormatterDbTest";
            log.Message = "To test Custom formatter for Databae Listener";
            log.Priority = 3;
            log.EventId = 100;
            log.AddErrorMessage("AdditionalInformation");
            log.Categories.Add("DatabaseCategory");
            log.Severity = System.Diagnostics.TraceEventType.Critical;
            string loggedMessage = "";
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                writer.Write(log);
            }
            DatabaseProviderFactory dbFactory = new DatabaseProviderFactory(source);
            Database db = dbFactory.Create("SqlServer");
            DbCommand command = db.GetSqlStringCommand("SELECT TOP 1 FormattedMessage FROM Log ORDER BY LogId DESC");
            loggedMessage = Convert.ToString(db.ExecuteScalar(command));
            Assert.IsTrue(VerifyCustomFormat(log, loggedMessage));
            listener.Formatter = actualFormatter;
        }


        [TestMethod]
        public void CustomFormatterCustomListenerTest()
        {
            CustomLogEntry log = new CustomLogEntry();
            log.Title = "CustomFormatterCustomListenerTest";
            log.Message = "To test Custom Formatter for Custom Registry Listener";
            log.Priority = 3;
            log.EventId = 100;
            log.CategoryID = 25;
            log.AddErrorMessage("AdditionalInformation");
            log.Categories.Add("FormatterCategory");
            log.Severity = System.Diagnostics.TraceEventType.Critical;
            string loggedMessage = "";
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                writer.Write(log);
            }
            RegistryKey key = Registry.LocalMachine.OpenSubKey(@"Software\EntlibTestApp");
            loggedMessage = (string)key.GetValue("LogEntry");
            key.Close();
            Assert.IsTrue(VerifyCustomFormat(log, loggedMessage));
        }

        #endregion

        #region Helper functions
        private bool VerifyTextFormat(LogEntry log, string loggedMessage)
        {
            TextFormatter formatter = new TextFormatter("EntLibLog{newline}{tab}{message}{newline}{priority}{newline}{tab}{severity}{newline}{tab}{eventid}{newline}{tab}{timestamp}{newline}{tab}{title}{newline}{appDomain}{newline}{processId}{newline}{processName}{newline}{machine}{newline}{threadName}{newline}{activity}{newline}{win32ThreadId}{newline}Extended Properties: {dictionary({key} - {value})}");
            string formattedMessage = formatter.Format(log);
            return loggedMessage.Equals(formattedMessage);
        }

        private bool VerifyBinaryFormat(LogEntry log, string loggedMessage)
        {
            LogEntry loggedEntry = BinaryLogFormatter.Deserialize(loggedMessage);
            Assert.IsTrue(loggedEntry.ActivityId.CompareTo(log.ActivityId) == 0);
            Assert.IsTrue(loggedEntry.ActivityIdString.Equals(log.ActivityIdString));
            Assert.IsTrue(loggedEntry.AppDomainName.Equals(log.AppDomainName));
            for (int i = 0; i < loggedEntry.CategoriesStrings.Length; i++)
            {
                Assert.IsTrue(loggedEntry.CategoriesStrings[i].Equals(log.CategoriesStrings[i]));
            }
            Assert.IsTrue(loggedEntry.ErrorMessages.Equals(log.ErrorMessages));
            Assert.IsTrue(loggedEntry.EventId.ToString().Equals(log.EventId.ToString()));
            Assert.IsTrue(loggedEntry.LoggedSeverity.Equals(log.LoggedSeverity));
            Assert.IsTrue(loggedEntry.MachineName.Equals(log.MachineName));
            Assert.IsTrue(loggedEntry.ManagedThreadName.Equals(log.ManagedThreadName));
            Assert.IsTrue(loggedEntry.Message.Equals(log.Message));
            Assert.IsTrue(loggedEntry.Priority == log.Priority);
            Assert.IsTrue(loggedEntry.ProcessId.Equals(log.ProcessId));
            Assert.IsTrue(loggedEntry.ProcessName.Equals(log.ProcessName));
            Assert.IsTrue(loggedEntry.Severity.ToString().Equals(log.Severity.ToString()));
            Assert.IsTrue(loggedEntry.TimeStamp.Equals(log.TimeStamp));
            Assert.IsTrue(loggedEntry.TimeStampString.Equals(log.TimeStampString));
            Assert.IsTrue(loggedEntry.Title.Equals(log.Title));
            Assert.IsTrue(loggedEntry.Win32ThreadId.Equals(log.Win32ThreadId));
            return true;
        }

        private bool VerifyCustomFormat(LogEntry log, string loggedMessage)
        {
            CustomFormatter formatter = new CustomFormatter();
            return formatter.Format(log).Equals(loggedMessage);
        }
        #endregion
    }
}
