using System;
using System.Text;
using System.Collections.Generic;
using System.Management;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Logging;
using Microsoft.Practices.EnterpriseLibrary.Common.Instrumentation;
using System.Diagnostics;
using System.Configuration; 
using Microsoft.Practices.EnterpriseLibrary.Logging.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Logging.Formatters;
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Data.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Logging.Database;
using Microsoft.Practices.EnterpriseLibrary.Logging.Database.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Logging.TraceListeners;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration.ObjectBuilder;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Instrumentation.Configuration;  
using Microsoft.Win32;
using CustomClasses;   

namespace ConfigurationSourceTests
{
    /// <summary>
    /// Summary description for DictionarySourceInstrumentationTestFixture
    /// </summary>
    [TestClass]
    public class DictionarySourceInstrumentationTestFixture
    {
        private DictionaryConfigurationSource source;
        private bool logged = false;
        private string loggedMessage = "";

        public DictionarySourceInstrumentationTestFixture()
        {
            source = LoggingConfigurationSource.BuildConfigurationSource(); 
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        #region Performance Counter tests
        [TestMethod]
        public void LogtoOneListenerandVerifyTraceListenerEntries()
        {
            string counterName = "Trace Listener Entries Written/sec";
            int initialCount = HelperClass.GetCounterValue(counterName, "Total");
            LogEntry entry = new LogEntry();
            entry.Severity = TraceEventType.Error;
            entry.Message = "";
            entry.Categories.Add("DatabaseCategory");
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                writer.Write(entry);
            }
            int loggedCount = HelperClass.GetCounterValue(counterName, "Total");
            Assert.IsTrue((loggedCount - initialCount) == 1);
        }

        [TestMethod]
        public void LogtoOneListenerandVerifyLoggingEventsRaised()
        {
            string counterName = "Logging Events Raised/sec";
            int initialCount = HelperClass.GetCounterValue(counterName, "Total");
            LogEntry entry = new LogEntry();
            entry.Severity = TraceEventType.Error;
            entry.Message = "";
            entry.Categories.Add("DatabaseCategory");
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                writer.Write(entry);
            }
            int loggedCount = HelperClass.GetCounterValue(counterName, "Total");
            Assert.IsTrue((loggedCount - initialCount) == 1);
        }

        [TestMethod]
        public void LogToAllListenersAndVerifyTraceListenerEntries()
        {
            string counterName = "Trace Listener Entries Written/sec";
            int initialCount = HelperClass.GetCounterValue(counterName, "Total");
            LogEntry entry = new LogEntry();
            entry.Severity = TraceEventType.Error;
            entry.Message = "";
            entry.Categories.Add("DatabaseCategory");
            entry.Categories.Add("General");
            entry.Categories.Add("WMICategory");
            entry.Categories.Add("MsmqCategory");
            entry.Categories.Add("FileCategory");
            entry.Categories.Add("SystemLogCategory");
            entry.Categories.Add("RegistryCategory");
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                writer.Write(entry);
            }
            int loggedCount = HelperClass.GetCounterValue(counterName, "Total");
            Assert.IsTrue((loggedCount - initialCount) == 10);
        }

        [TestMethod]
        public void LogToAllListenersAndVerifyLoggingEventsRaised()
        {
            string counterName = "Logging Events Raised/sec";
            int initialCount = HelperClass.GetCounterValue(counterName, "Total");
            LogEntry entry = new LogEntry();
            entry.Severity = TraceEventType.Error;
            entry.Message = "";
            entry.Categories.Add("DatabaseCategory");
            entry.Categories.Add("General");
            entry.Categories.Add("WMICategory");
            entry.Categories.Add("MsmqCategory");
            entry.Categories.Add("FileCategory");
            entry.Categories.Add("SystemLogCategory");
            entry.Categories.Add("RegistryCategory");
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                writer.Write(entry);
            }
            int loggedCount = HelperClass.GetCounterValue(counterName, "Total");
            Assert.IsTrue((loggedCount - initialCount) == 1);
        }

        [TestMethod]
        public void LogToSingleTracerAndVerifyTracerOperationsStarted()
        {
            string counterName = "Trace Operations Started/sec";
            int initialCount = HelperClass.GetCounterValue(counterName, "DatabaseCategory");
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                using (Tracer trace = new Tracer("DatabaseCategory", writer,source))
                {

                }
            }
            int finalCount = HelperClass.GetCounterValue(counterName, "DatabaseCategory");
            Assert.IsTrue(finalCount - initialCount == 1);
        }

        [TestMethod]
        public void LogToMultipleTracerAndVerifyTraceOperationsStarted()
        {
            string counterName = "Trace Operations Started/sec";
            int initialDbCount = HelperClass.GetCounterValue(counterName, "DatabaseCategory");
            int initialEventLogCount = HelperClass.GetCounterValue(counterName, "General");
            int initialMsmqCount = HelperClass.GetCounterValue(counterName, "MsmqCategory");
            int finalDbCount = 0, finalEventLogCount = 0, finalMsmqCount = 0;
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                using (Tracer trace = new Tracer("DatabaseCategory",writer,source))
                {
                    finalDbCount = HelperClass.GetCounterValue(counterName, "DatabaseCategory");
                    using (Tracer trace1 = new Tracer("General",writer,source))
                    {
                        finalEventLogCount = HelperClass.GetCounterValue(counterName, "General");
                        InnerMethod(writer);
                        finalMsmqCount = HelperClass.GetCounterValue(counterName, "MsmqCategory");
                    }
                }
            }
            Assert.IsTrue(finalDbCount - initialDbCount == 1);
            Assert.IsTrue(finalEventLogCount - initialEventLogCount == 1);
            Assert.IsTrue(finalMsmqCount - initialMsmqCount == 1);
        }

        [TestMethod]
        public void LogToSingleTracerAndVerifyAvgExecutionTime()
        {
            string counterName = "Avg. Trace Execution Time";
            int initialCount = HelperClass.GetCounterValue(counterName, "DatabaseCategory");
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {

                using (Tracer trace = new Tracer("DatabaseCategory",writer,source))
                {

                }
            }
            int finalCount = HelperClass.GetCounterValue(counterName, "DatabaseCategory");
            Assert.IsTrue(finalCount - initialCount > 0);
        }

        [TestMethod]
        public void LogToMultipleTracerAndVerifyAvgExecutionTime()
        {
            string counterName = "Avg. Trace Execution Time";
            int initialDbCount = HelperClass.GetCounterValue(counterName, "DatabaseCategory");
            int initialEventLogCount = HelperClass.GetCounterValue(counterName, "General");
            int initialMsmqCount = HelperClass.GetCounterValue(counterName, "MsmqCategory");
            int finalDbCount = 0, finalEventLogCount = 0, finalMsmqCount = 0;
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                using (Tracer trace = new Tracer("DatabaseCategory",writer,source))
                {
                    writer.Write(new LogEntry());
                    using (Tracer trace1 = new Tracer("General",writer,source))
                    {
                        writer.Write(new LogEntry());
                        InnerMethod(writer );
                        finalMsmqCount = HelperClass.GetCounterValue(counterName, "MsmqCategory");
                    }
                    finalEventLogCount = HelperClass.GetCounterValue(counterName, "General");
                }
            }
            finalDbCount = HelperClass.GetCounterValue(counterName, "DatabaseCategory");
            Assert.IsTrue((finalDbCount - initialDbCount) > 0);
            Assert.IsTrue((finalEventLogCount - initialEventLogCount) > 0);
            Assert.IsTrue((finalMsmqCount - initialMsmqCount) > 0);
            Assert.IsTrue(finalEventLogCount > finalMsmqCount);
            Assert.IsTrue(finalDbCount > finalEventLogCount);

        }


        private void InnerMethod(LogWriter writer)
        {
            using (Tracer trace = new Tracer("MsmqCategory", new Guid(),writer,source))
            {
                writer.Write(new LogEntry());
            }
        }
        #endregion

        #region WMI Tests
        [TestMethod]
        public void LoggingFailureLoggingErrorEvent()
        {
            LoggingSettings setting = (LoggingSettings)source.GetSection("loggingConfiguration");
            FormattedDatabaseTraceListenerData dbListener = (FormattedDatabaseTraceListenerData)setting.TraceListeners.Get("Database Trace Listener");
            string dbInstanceName = dbListener.DatabaseInstanceName;
            dbListener.DatabaseInstanceName = "InvalidInstance";
            TraceSourceData actualSource = setting.SpecialTraceSources.ErrorsTraceSource;
            setting.SpecialTraceSources.ErrorsTraceSource = setting.TraceSources.Get("DatabaseCategory");  
  
            LogEntry entry = new LogEntry();
            entry.Categories.Add("DatabaseCategory");
            entry.Message = "To test the LoggingFailureLoggingErrorEvent";
            ManagementScope scope = new ManagementScope(@"\\.\root\EnterpriseLibrary");
            scope.Options.EnablePrivileges = true;
            EventQuery eq = new EventQuery("SELECT * FROM LoggingFailureLoggingErrorEvent");
            ManagementEventWatcher watcher = new ManagementEventWatcher(scope, eq);
            watcher.EventArrived += new EventArrivedEventHandler(WatcherEventArrived);
            watcher.Start();
            logged = false;
            loggedMessage = "";
            LogWriterFactory factory = new LogWriterFactory(source);
            using (LogWriter writer = factory.Create())
            {
                writer.Write(entry);
            }
            System.Threading.Thread.Sleep(1000);
            watcher.Stop();
            watcher.Dispose();
            Assert.IsTrue(logged);
            Assert.IsTrue(loggedMessage.Contains("Failure while sending log entry to the TraceListeners"));     
            dbListener.DatabaseInstanceName = dbInstanceName;
            setting.SpecialTraceSources.ErrorsTraceSource = actualSource;     
        }

        private void WatcherEventArrived(object sender, EventArrivedEventArgs args)
        {
            loggedMessage = args.NewEvent.GetText(TextFormat.Mof);
            logged = true;
        }

        #endregion

        #region EventLog Instrumentation Tests
        [TestMethod] 
        public void VerifyErrorinEventLogErrorinTracing()
        {
            LoggingSettings setting = (LoggingSettings)source.GetSection("loggingConfiguration");
            FormattedDatabaseTraceListenerData dbListener = (FormattedDatabaseTraceListenerData)setting.TraceListeners.Get("Database Trace Listener");
            string dbInstanceName = dbListener.DatabaseInstanceName;
            dbListener.DatabaseInstanceName = "InvalidInstance";
            TraceSourceData actualSource = setting.SpecialTraceSources.ErrorsTraceSource;
            setting.SpecialTraceSources.ErrorsTraceSource = setting.TraceSources.Get("DatabaseCategory");  
  
            LogEntry entry = new LogEntry();
            entry.Categories.Add("DatabaseCategory");
            entry.Message = "To test the LoggingFailureLoggingErrorEvent";
            LogWriterFactory factory = new LogWriterFactory(source);
            using (EventLog log = new EventLog("Application"))
            {
                int initialCount = log.Entries.Count; 
                using (LogWriter writer = factory.Create())
                {
                    writer.Write(entry);
                }
                int finalCount = log.Entries.Count;
                Assert.IsTrue(finalCount - initialCount == 1);
                Assert.IsTrue(log.Entries[finalCount - 1].Message.Contains("Failure while sending log entry to the TraceListeners"));
                Assert.IsTrue(log.Entries[finalCount - 1].Source.Equals("Enterprise Library Logging"));
            }
            dbListener.DatabaseInstanceName = dbInstanceName;
            setting.SpecialTraceSources.ErrorsTraceSource = actualSource;     
        }

        [TestMethod]
        public void VerifyLogErrorMissingCategories()
        {
            LogEntry entry = new LogEntry();
            entry.Categories.Add("MissingCategory");
            entry.Message = "To test if  error message is logged to event Log when specified category is not found";
            LogWriterFactory factory = new LogWriterFactory(source);
            using (EventLog log = new EventLog(HelperClass.GetEventLogName()))
            {
                int initialCount = log.Entries.Count;
                using (LogWriter writer = factory.Create())
                {
                    writer.Write(entry);
                }
                int finalCount = log.Entries.Count;
                Assert.IsTrue(finalCount - initialCount == 1);
                Assert.IsTrue(log.Entries[finalCount - 1].Message.Contains("There is no explicit mapping for the categories"));
                Assert.IsTrue(log.Entries[finalCount - 1].Message.Contains(entry.Message));
            }
        }

        [TestMethod]
        public void VerifyLogErrorCheckingFilter()
        {
            LoggingSettings setting = (LoggingSettings)source.GetSection("loggingConfiguration");
            CustomLogFilterData sevFilter = (CustomLogFilterData)setting.LogFilters.Get("CustomSevFilter");
            sevFilter.Attributes.Clear();     
            LogEntry entry = new LogEntry();
            entry.Message = "To test if  error message is logged to event Log when there is error checking filters";
            LogWriterFactory factory = new LogWriterFactory(source);
            using (EventLog log = new EventLog(HelperClass.GetEventLogName()))
            {
                int initialCount = log.Entries.Count;
                using (LogWriter writer = factory.Create())
                {
                    Assert.IsTrue(writer.ShouldLog(entry));
                }
                int finalCount = log.Entries.Count;
                Assert.IsTrue(finalCount - initialCount == 1);
                Assert.IsTrue(log.Entries[finalCount - 1].Message.Contains("Evaluation of filter 'CustomSeverityFilter' failed and the filter will be ignored"));
                Assert.IsTrue(log.Entries[finalCount - 1].Message.Contains(entry.Message));
            }
            sevFilter.Attributes.Add("key1", "Information");
            sevFilter.Attributes.Add("key2", "Error");
            sevFilter.Attributes.Add("key3", "Warning");
            sevFilter.Attributes.Add("key4", "Start");
            sevFilter.Attributes.Add("key5", "Stop");
            sevFilter.Attributes.Add("key6", "Critical");
        }

#endregion


    }
}
