using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using Microsoft.Practices.EnterpriseLibrary.Logging.TraceListeners;    
using Microsoft.Practices.EnterpriseLibrary.Logging.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Logging.Formatters; 
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using CustomClasses.Configuration; 
      
namespace CustomClasses
{
    [ConfigurationElementType(typeof(ExtendedFlatFileTraceListenerData))] 
    public class ExtendedFlatFileListener:FlatFileTraceListener 
    {
        int fileSize;
        string fileName;
        public ExtendedFlatFileListener()
        {

        }
        
        public ExtendedFlatFileListener(string fileName,string header,string footer,ILogFormatter formatter,int fileSize)
            : base(fileName,header,footer,formatter )
        {
            this.fileSize = fileSize;
            this.fileName = fileName; 
        }

        public override void TraceData(System.Diagnostics.TraceEventCache eventCache, string source, System.Diagnostics.TraceEventType eventType, int id, object data)
        {
            if (File.Exists(fileName))
            {
                using (FileStream fs = new FileStream(fileName, FileMode.Open, FileAccess.Read))
                {
                    if (fs.Length >= (fileSize * 1024))
                    {
                        throw new ApplicationException("The file size has exceeded the limit specified in the configuration. Please increase the size limit or point to a different file for logging");
                    }
                }
            }
            base.TraceData(eventCache, source, eventType, id, data);
        }
    }
}
