using System;
using System.Text;
using System.Collections.Generic;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using System.Threading;
using System.Diagnostics;
using System.Management;
using System.Messaging; 
using System.Data.Common;
using System.IO; 
using Microsoft.Practices.EnterpriseLibrary.Data;     
using Microsoft.Practices.EnterpriseLibrary.Common;
using Microsoft.Practices.EnterpriseLibrary.Logging;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration.ObjectBuilder;
using Microsoft.Practices.EnterpriseLibrary.Logging.TraceListeners;
using Microsoft.Practices.EnterpriseLibrary.Logging.Formatters; 
using Microsoft.Practices.EnterpriseLibrary.Logging.Database;
using CustomClasses; 

namespace LoggerFacade
{
    /// <summary>
    /// Summary description for MultiThreadingTestFixture
    /// </summary>
    [TestClass]
    public class MultiThreadingTestFixture
    {
        private string category = "SystemLogCategory";
        private int threadCount = 1;
        private object obj = new object();
        private int loggedCount = 0;

        public MultiThreadingTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        #region Multi Threading Tests for various Listener

        [TestMethod]
        public void FormattedEventLogListenerTest()
        {
            using (EventLog eventlog = new EventLog(HelperClass.GetEventLogName()))
            {
                int initialCount = eventlog.Entries.Count;   
                category = "General";
                InitiateThread(); 
                int finalCount = eventlog.Entries.Count;
                Assert.IsTrue((finalCount - initialCount) == threadCount, "Count : " + (finalCount - initialCount));
            }
        }

        [TestMethod]
        public void SystemEventLogListenerTest()
        {
            using (EventLog eventlog = new EventLog(HelperClass.GetEventLogName()))
            {
                int initialCount = eventlog.Entries.Count;
                category = "SystemLogCategory";
                InitiateThread();
                int finalCount = eventlog.Entries.Count;
                Assert.IsTrue((finalCount - initialCount) == threadCount, "Count : " + (finalCount - initialCount));
            }
        }


        [TestMethod]
        public void FormattedEventLogListenerUsingTrace()
        {
            using (EventLog eventlog = new EventLog(HelperClass.GetEventLogName()))
            {
                int initialCount = eventlog.Entries.Count;
                category = "General";
                InitiateThreadUsingTrace();
                int finalCount = eventlog.Entries.Count;
                Assert.IsTrue((finalCount - initialCount) == (threadCount*2), "Count : " + (finalCount - initialCount));
            }

        }

        [TestMethod]
        public void SystemFlatFileListenerTest()
        {
            category = "SystemLogCategory";
            int initialCount = GetMessageCount("trace2.log", "Test: Thread Test");  
            InitiateThread();
            int finalCount = GetMessageCount("trace2.log", "Test: Thread Test");
            Assert.IsTrue((finalCount - initialCount) == threadCount, "Count : " + (finalCount - initialCount));
        }

        [TestMethod]
        public void FormattedFlatFileListenerTest()
        {
            category = "FileCategory";
            int initialCount = GetMessageCount("trace.log", category);
            InitiateThread();
            int finalCount = GetMessageCount("trace.log", category);
            Assert.IsTrue((finalCount - initialCount) == threadCount, "Count : " + (finalCount - initialCount));
        }

        [TestMethod]
        public void DatabaseListenerTest()
        {
            Database db = DatabaseFactory.CreateDatabase("SqlServer");
            DbCommand command = db.GetSqlStringCommand("SELECT count(logId) FROM Log");
            category = "DatabaseCategory";
            int initialCount = Convert.ToInt32(db.ExecuteScalar(command));
            InitiateThread();
            int finalCount = Convert.ToInt32(db.ExecuteScalar(command));
            Assert.IsTrue((finalCount - initialCount) == threadCount, "Count : " + (finalCount - initialCount));
            
        }

        [TestMethod]
        public void WMIListenerTest()
        {
            category = "WMICategory";
            ManagementScope scope = new ManagementScope(@"\\.\root\EnterpriseLibrary");
            scope.Options.EnablePrivileges = true;
            EventQuery eq = new EventQuery("SELECT * FROM LogEntryV20");
            ManagementEventWatcher watcher = new ManagementEventWatcher(scope, eq);
            watcher.EventArrived += new EventArrivedEventHandler(WatcherEventArrived);
            watcher.Start();
            InitiateThread();
            Thread.Sleep(1000);
            watcher.Stop();
            watcher.Dispose();
            Assert.IsTrue(loggedCount == threadCount,loggedCount.ToString() );
        }


        [TestMethod]
        public void MsmqListenerTest()
        {
            using (MessageQueue queue = new MessageQueue(@".\Private$\EntlibTest"))
            {
                queue.Purge();
                queue.Formatter = new XmlMessageFormatter();
                string[] types = { "System.String" };
                ((XmlMessageFormatter)queue.Formatter).TargetTypeNames = types;
                category = "MsmqCategory";
                InitiateThread();
                int finalCount = queue.GetAllMessages().Length;
                Assert.IsTrue(finalCount  == threadCount, "Count : " + finalCount);
            }

        }
        #endregion

        #region Additional Tests
        [TestMethod]
        public void AreListenersThreadSafe()
        {
            EventLogTraceListener listener = new EventLogTraceListener("Entlib Tests");
            FormattedEventLogTraceListener formattedListener = new FormattedEventLogTraceListener();
            FlatFileTraceListener filelistener = new FlatFileTraceListener();
            TextWriterTraceListener systemListener = new TextWriterTraceListener();
            EmailTraceListener emailListener = new EmailTraceListener();
            WmiTraceListener wmiTraceListener = new WmiTraceListener();
            MsmqTraceListener msmqListener = new MsmqTraceListener("MsmqListener", ".\\Private$\\EntlibTest", new TextFormatter(), MessagePriority.Normal, false, Message.InfiniteTimeout, Message.InfiniteTimeout, false, false, false, MessageQueueTransactionType.None, new MsmqSendInterfaceFactory());
            FormattedDatabaseTraceListener dbListener = new FormattedDatabaseTraceListener(null, "", "", new TextFormatter());   

            Assert.IsFalse(listener.IsThreadSafe);
            Assert.IsTrue(formattedListener.IsThreadSafe);
            Assert.IsFalse(filelistener.IsThreadSafe);
            Assert.IsFalse(systemListener.IsThreadSafe);
            Assert.IsTrue(emailListener.IsThreadSafe);
            Assert.IsFalse(wmiTraceListener.IsThreadSafe);
            Assert.IsTrue(msmqListener.IsThreadSafe);
            Assert.IsTrue(dbListener.IsThreadSafe);
        }
        #endregion

        #region Helper Methods
        private void InitiateThread()
        {
            Thread[] th = new Thread[threadCount];
            for (int i = 0; i < threadCount; i++)
            {
                th[i] = new Thread(new ThreadStart(LogToEventLog));
                th[i].Start();
            }

            for (int i = 0; i < threadCount; i++)
            {
                th[i].Join();
            }
        }

        private void InitiateThreadUsingTrace()
        {
            Thread[] th = new Thread[threadCount];
            for (int i = 0; i < threadCount; i++)
            {
                th[i] = new Thread(new ThreadStart(CallTracer));
                th[i].Start();
            }

            for (int i = 0; i < threadCount; i++)
            {
                th[i].Join();
            }
        }

        private void LogToEventLog()
        {
            LogEntry entry = new LogEntry();
            entry.Message = "Test: Thread Test";
            entry.Categories.Add(category);
            Logger.Write(entry);
        }

        private void WatcherEventArrived(object sender, EventArrivedEventArgs args)
        {
            lock (obj)
            {
                string loggedMessage = args.NewEvent.GetText(TextFormat.Mof);
                if (loggedMessage != null && loggedMessage.Length != 0 && loggedMessage.Contains("WMICategory"))
                    loggedCount++;
            }
        }

        private void CallTracer()
        {
            using (Tracer trace = new Tracer("General"))
            {

            }
        }

        private int GetMessageCount(string fileName,string message)
        {
            int noOfLogs = 0;
            if (File.Exists(fileName))
            {
                using (System.IO.FileStream logReader = new System.IO.FileStream(fileName, System.IO.FileMode.Open, System.IO.FileAccess.Read, System.IO.FileShare.ReadWrite))
                {
                    long fileSize = logReader.Length;
                    byte[] fileBytes = new byte[fileSize];
                    int amountOfBytes = logReader.Read(fileBytes, 0, (int)fileSize);
                    System.Text.UTF7Encoding utf7 = new System.Text.UTF7Encoding();
                    System.String fileContent = utf7.GetString(fileBytes, 0, amountOfBytes);
                    int startindex;
                    while((startindex= fileContent.IndexOf(message)) != -1)  
                    {
                        fileContent = fileContent.Substring(startindex + 1);
                        noOfLogs++; 
                    }
                }
            }
            return noOfLogs;
        }
        #endregion
    }
}
