using System;
using System.Text;
using System.Collections.Generic;
using System.Configuration;
using System.Security.Principal;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Common;
using Microsoft.Practices.EnterpriseLibrary.Security;
using Microsoft.Practices.EnterpriseLibrary.Security.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Security.AzMan;
using Microsoft.Practices.EnterpriseLibrary.Common.Instrumentation;
using System.Diagnostics;
using System.Threading;
using System.Management;

namespace SecurityAzManFuncTests
{
    /// <summary>
    /// Summary description for InstrumentationTestFixture
    /// </summary>
    [TestClass]
    public class AzManInstrumentationTestFixture
    {
        String wmiText = "";
        bool wmiLogged = false;
        public AzManInstrumentationTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void AzmanSystemConfigurationFailureWMITest()
        {
            wmiLogged = false;
            wmiText = "";
            bool exceptionThrown = false;
            ManagementScope scope = new ManagementScope(@"\\.\root\EnterpriseLibrary");
            scope.Options.EnablePrivileges = true;
            EventQuery eq = new EventQuery("SELECT * FROM SecurityConfigurationFailureEvent");
            ManagementEventWatcher watcher = new ManagementEventWatcher(scope, eq);
            watcher.EventArrived += new EventArrivedEventHandler(WatcherEventArrived);
            watcher.Start();
            try
            {
                IAuthorizationProvider azManProvider = AuthorizationFactory.GetAuthorizationProvider("InvalidProvider");

            }
            catch (ConfigurationErrorsException)
            {
                exceptionThrown = true;
            }
            Assert.AreEqual(true, exceptionThrown);

            int timeoutCount = 10;
            int count = 0;
            bool loopTimed = false;
            while (!wmiLogged && (loopTimed == false))
            {
                Thread.Sleep(100);
                count++;
                if (count == timeoutCount)
                {
                    loopTimed = true;
                }
            }
            watcher.Stop();
            Assert.AreEqual(true, wmiLogged);
            if (wmiText.IndexOf("SecurityConfigurationFailureEvent") == -1)
            {
                Assert.Fail();
            }
        }

        [TestMethod]
        public void AzmanSystemConfigFailureEventLogTest()
        {
            bool exceptionThrown = false;
            try
            {
                IAuthorizationProvider azManProvider = AuthorizationFactory.GetAuthorizationProvider("InvalidProvider");

            }
            catch (ConfigurationErrorsException)
            {
                exceptionThrown = true;
            }
            Assert.AreEqual(true, exceptionThrown);

            string logName = EventLog.LogNameFromSourceName("Application", ".");
            using (EventLog eventlog = new EventLog(logName))
            {

                int entryCount = eventlog.Entries.Count;
                if (!(eventlog.Entries[eventlog.Entries.Count - 1].Message.Contains("InvalidProvider")))
                    Assert.Fail("Event Not logged");
            }
        }

        [TestMethod]
        public void AzmanAuthCheckPerformedTest()
        {
            IAuthorizationProvider azManProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultAzManProvider");
            WindowsIdentity identity = WindowsIdentity.GetAnonymous();
            string counterName = "Authorization Requests/sec";
            int initialCount = GetCounterValue(counterName);
            int loggedCount = 0;
            bool isAuthorized = azManProvider.Authorize(new WindowsPrincipal(identity), "Authorize Purchase");
            Assert.IsTrue(isAuthorized);
            loggedCount = GetCounterValue(counterName);
            Assert.AreEqual(loggedCount - 1, initialCount);  

        }

        [TestMethod]
        public void AzmanAuthCheckFailedTest()
        {
            string counterName = "Authorization Requests Denied/sec";
            int initialCount = GetCounterValue(counterName);
            int loggedCount = 0;
            IAuthorizationProvider azManProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultAzManProvider");
            WindowsIdentity identity = WindowsIdentity.GetAnonymous();
            bool isAuthorized = azManProvider.Authorize(new WindowsPrincipal(identity), "Fire Employee");
            Assert.IsFalse(isAuthorized);
            loggedCount = GetCounterValue(counterName);
            Assert.AreEqual(loggedCount - 1, initialCount);  
        }

        [TestMethod]
        public void AzmanAuthCheckPerformedWMITest()
        {
            wmiLogged = false;
            wmiText = "";
            ManagementScope scope = new ManagementScope(@"\\.\root\EnterpriseLibrary");
            scope.Options.EnablePrivileges = true;
            EventQuery eq = new EventQuery("SELECT * FROM AuthorizationCheckPerformedEvent");
            ManagementEventWatcher watcher = new ManagementEventWatcher(scope, eq);
            watcher.EventArrived += new EventArrivedEventHandler(WatcherEventArrived);
            watcher.Start(); 
            
            IAuthorizationProvider azManProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultAzManProvider");
            WindowsIdentity identity = WindowsIdentity.GetAnonymous();
            
            bool isAuthorized = azManProvider.Authorize(new WindowsPrincipal(identity), "Authorize Purchase");
            Assert.IsTrue(isAuthorized);
            
            int timeoutCount = 10;
            int count = 0;
            bool loopTimed = false;
            while (!wmiLogged && (loopTimed == false))
            {
                Thread.Sleep(100);
                count++;
                if (count == timeoutCount)
                {
                    loopTimed = true;
                }
            }
            watcher.Stop();
            Assert.AreEqual(true, wmiLogged);
            if (wmiText.IndexOf("AuthorizationCheckPerformedEvent") == -1)
            {
                Assert.Fail();
            }

        }

        [TestMethod]
        public void AzmanAuthCheckFailedWMITest()
        {
            wmiLogged = false;
            wmiText = "";
            ManagementScope scope = new ManagementScope(@"\\.\root\EnterpriseLibrary");
            scope.Options.EnablePrivileges = true;
            EventQuery eq = new EventQuery("SELECT * FROM AuthorizationCheckFailedEvent");
            ManagementEventWatcher watcher = new ManagementEventWatcher(scope, eq);
            watcher.EventArrived += new EventArrivedEventHandler(WatcherEventArrived);
            watcher.Start(); 
            
            IAuthorizationProvider azManProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultAzManProvider");
            WindowsIdentity identity = WindowsIdentity.GetAnonymous();
            bool isAuthorized = azManProvider.Authorize(new WindowsPrincipal(identity), "Fire Employee");
            Assert.IsFalse(isAuthorized);

            int timeoutCount = 10;
            int count = 0;
            bool loopTimed = false;
            while (!wmiLogged && (loopTimed == false))
            {
                Thread.Sleep(100);
                count++;
                if (count == timeoutCount)
                {
                    loopTimed = true;
                }
            }
            watcher.Stop();
            Assert.AreEqual(true, wmiLogged);
            if (wmiText.IndexOf("AuthorizationCheckFailedEvent") == -1)
            {
                Assert.Fail();
            }
        }
        private int GetCounterValue(string counterName)
        {
            //Verify the perf counter
            string categoryName = "Enterprise Library Security Counters";
            string instanceName = new AppDomainNameFormatter().CreateName("Total");
            if (PerformanceCounterCategory.InstanceExists(instanceName, categoryName))
            {
                using (PerformanceCounter counter = new PerformanceCounter())
                {
                    counter.CategoryName = categoryName;
                    counter.CounterName = counterName;
                    counter.InstanceName = instanceName;
                    return (int)counter.RawValue;
                }
            }
            return 0;
        }

        private void WatcherEventArrived(object sender, EventArrivedEventArgs args)
        {
            wmiLogged = true;
            /// Validate the message here
            wmiText = args.NewEvent.GetText(TextFormat.Mof);
        }
    }
}
