using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Security;
using Microsoft.Practices.EnterpriseLibrary.Security.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Security.Cache.CachingStore.Configuration;
using System.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Caching.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Caching.Database.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Caching.BackingStoreImplementations;
using Microsoft.Practices.EnterpriseLibrary.Data.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Instrumentation.Configuration;
namespace SecurityCachingFuncTests
{
    public class SecurityCachingDictSource
    {
        public DictionaryConfigurationSource BuildDictionarySourceSection()
        {
            DictionaryConfigurationSource section = new DictionaryConfigurationSource();

            #region "Security Configuration"
            SecuritySettings secSettings = new SecuritySettings();
            secSettings.DefaultSecurityCacheProviderName = "CacheProviderDic";

            CachingStoreProviderData scpd = new CachingStoreProviderData();
            scpd.Name = "CacheProviderDic";
            scpd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Security.Cache.CachingStore.CachingStoreProvider, Microsoft.Practices.EnterpriseLibrary.Security.Cache.CachingStore");
            scpd.CacheManager = "DefaultCacheManagerDic";
            scpd.AbsoluteExpiration = 5;
            scpd.SlidingExpiration = 120;
            secSettings.SecurityCacheProviders.Add(scpd);

            scpd = new CachingStoreProviderData();
            scpd.Name = "CacheProviderIsolatedDic";
            scpd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Security.Cache.CachingStore.CachingStoreProvider, Microsoft.Practices.EnterpriseLibrary.Security.Cache.CachingStore");
            scpd.CacheManager = "IsolatedCacheManagerDic";
            scpd.AbsoluteExpiration = 5;
            scpd.SlidingExpiration = 120;
            secSettings.SecurityCacheProviders.Add(scpd);

            scpd = new CachingStoreProviderData();
            scpd.Name = "CacheProviderDBDic";
            scpd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Security.Cache.CachingStore.CachingStoreProvider, Microsoft.Practices.EnterpriseLibrary.Security.Cache.CachingStore");
            scpd.CacheManager = "DBCacheManagerDic";
            scpd.AbsoluteExpiration = 5;
            scpd.SlidingExpiration = 120;
            secSettings.SecurityCacheProviders.Add(scpd);

            CustomSecurityCacheProviderData customData = new CustomSecurityCacheProviderData("MyCache", Type.GetType("SecurityCachingFuncTests.CustomSecurityCacheProvider,SecurityCachingFuncTests"));
            customData.Attributes.Add("key1", "value1");
            secSettings.SecurityCacheProviders.Add(customData);    

            section.Add("securityConfiguration", secSettings);
            #endregion

            CacheManagerSettings cacheSettings = new CacheManagerSettings();
            cacheSettings.DefaultCacheManager = "DefaultCacheManagerDic";

            #region "Backing Stores"
            IsolatedStorageCacheStorageData isbs = new IsolatedStorageCacheStorageData();
            isbs.Name = "Isolated Storage Dic";
            isbs.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Caching.BackingStoreImplementations.IsolatedStorageBackingStore, Microsoft.Practices.EnterpriseLibrary.Caching");
            isbs.PartitionName = "EntLib";
            cacheSettings.BackingStores.Add(isbs);

            CacheStorageData nbs = new CacheStorageData();
            nbs.Name = "In Memory Dic";
            nbs.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Caching.BackingStoreImplementations.NullBackingStore, Microsoft.Practices.EnterpriseLibrary.Caching");
            cacheSettings.BackingStores.Add(nbs);

            DataCacheStorageData dcsd = new DataCacheStorageData();
            dcsd.Name = "Data Cache Storage Dic";
            dcsd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Caching.Database.DataBackingStore, Microsoft.Practices.EnterpriseLibrary.Caching.Database");
            dcsd.DatabaseInstanceName = "CachingDatabase";
            dcsd.PartitionName = "EntLib";
            cacheSettings.BackingStores.Add(dcsd);

            #endregion

            #region "Cache Managers"
            CacheManagerData cmd = new CacheManagerData();
            cmd.Name = "DefaultCacheManagerDic";
            cmd.ExpirationPollFrequencyInSeconds = 1;
            cmd.MaximumElementsInCacheBeforeScavenging = 10;
            cmd.NumberToRemoveWhenScavenging = 3;
            cmd.CacheStorage = "In Memory Dic";

            cacheSettings.CacheManagers.Add(cmd);

            cmd = new CacheManagerData();
            cmd.Name = "IsolatedCacheManagerDic";
            cmd.ExpirationPollFrequencyInSeconds = 1;
            cmd.MaximumElementsInCacheBeforeScavenging = 10;
            cmd.NumberToRemoveWhenScavenging = 3;
            cmd.CacheStorage = "Isolated Storage Dic";

            cacheSettings.CacheManagers.Add(cmd);

            cmd = new CacheManagerData();
            cmd.Name = "DBCacheManagerDic";
            cmd.ExpirationPollFrequencyInSeconds = 1;
            cmd.MaximumElementsInCacheBeforeScavenging = 10;
            cmd.NumberToRemoveWhenScavenging = 3;
            cmd.CacheStorage = "Data Cache Storage Dic";

            cacheSettings.CacheManagers.Add(cmd);

            #endregion

            section.Add("cachingConfiguration",cacheSettings);

            #region "Data Configuration"
            DatabaseSettings dataSettings = new DatabaseSettings();
            dataSettings.DefaultDatabase = "CachingDatabase";

            ConnectionStringSettings connstring = new ConnectionStringSettings();
            connstring.Name = "CachingDatabase";
            connstring.ProviderName = "System.Data.SqlClient";
            connstring.ConnectionString = "server=.\\sqlexpress;database=Caching;Integrated Security=true";


            ConnectionStringsSection connsection = new ConnectionStringsSection();
            connsection.ConnectionStrings.Add(connstring);

            section.Add("dataConfiguration", dataSettings);
            section.Add("connectionStrings", connsection);
            #endregion
            InstrumentationConfigurationSection ics = new InstrumentationConfigurationSection(true, true, true);
            section.Add("instrumentationConfiguration", ics);

            return section;
        }
    }
}
