using System;
using System.Text;
using System.Collections.Generic;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Security;
using System.Security.Principal;
using Microsoft.Practices.EnterpriseLibrary.Common.Instrumentation;
using System.Diagnostics;
using System.Configuration;
using System.Threading;
using System.Management;

namespace SecurityCachingFuncTests
{
    /// <summary>
    /// Summary description for SecurityCachingInstrumentationTestFixture
    /// </summary>
    [TestClass]
    public class SecurityCachingInstrumentationTestFixture
    {
        private static IPrincipal principal;
        String wmiText = "";
        bool wmiLogged = false;
        public SecurityCachingInstrumentationTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void SecurityCacheReadPerformedTest()
        {
            ISecurityCacheProvider securityCache = SecurityCacheFactory.GetSecurityCacheProvider("CacheProvider");
            Assert.IsNotNull(securityCache);
            
            IIdentity identity = new GenericIdentity("testuser", "SecurityCache");
            string[] roles = new string[] { "admin", "managers" };
            principal = new GenericPrincipal(identity, roles);

            IToken token = securityCache.SavePrincipal(principal);
            Assert.IsNotNull(token);
            Assert.IsNotNull(token.Value);
            string counterName = "Security Cache Reads/sec";
            int initialAuthDeniedCount = GetCounterValue("Authorization Requests Denied/sec");
            int initialAuthReqCount = GetCounterValue("Authorization Requests/sec");
            int initialCount = GetCounterValue(counterName);
            int loggedCount = 0;
            IPrincipal cachedPrincipal = securityCache.GetPrincipal(token);
            loggedCount = GetCounterValue(counterName);
            int loggedAuthDeniedCount = GetCounterValue("Authorization Requests Denied/sec");
            int loggedAuthReqCount = GetCounterValue("Authorization Requests/sec");
            
            Assert.AreEqual(loggedCount - 1, initialCount);
            Assert.AreEqual(initialAuthDeniedCount, loggedAuthDeniedCount);
            Assert.AreEqual(initialAuthReqCount, loggedAuthReqCount);

        }

        [TestMethod]
        public void SecurityCacheReadPerformedWMITest()
        {
            wmiLogged = false;
            wmiText = "";
            bool exceptionThrown = false;
            ManagementScope scope = new ManagementScope(@"\\.\root\EnterpriseLibrary");
            scope.Options.EnablePrivileges = true;
            EventQuery eq = new EventQuery("SELECT * FROM SecurityCacheReadPerformedEvent");
            ManagementEventWatcher watcher = new ManagementEventWatcher(scope, eq);
            watcher.EventArrived += new EventArrivedEventHandler(WatcherEventArrived);
            watcher.Start(); 
            
            ISecurityCacheProvider securityCache = SecurityCacheFactory.GetSecurityCacheProvider("CacheProvider");
            Assert.IsNotNull(securityCache);

            IIdentity identity = new GenericIdentity("testuser", "SecurityCache");
            string[] roles = new string[] { "admin", "managers" };
            principal = new GenericPrincipal(identity, roles);

            IToken token = securityCache.SavePrincipal(principal);
            Assert.IsNotNull(token);
            Assert.IsNotNull(token.Value);
            
            IPrincipal cachedPrincipal = securityCache.GetPrincipal(token);

            int timeoutCount = 10;
            int count = 0;
            bool loopTimed = false;
            while (!wmiLogged && (loopTimed == false))
            {
                Thread.Sleep(100);
                count++;
                if (count == timeoutCount)
                {
                    loopTimed = true;
                }
            }
            watcher.Stop();
            Assert.AreEqual(true, wmiLogged);
            if (wmiText.IndexOf("SecurityCacheReadPerformedEvent") == -1)
            {
                Assert.Fail();
            }

        }


        [TestMethod]
        public void SecurityCacheConfigFailureWMITest()
        {
            wmiLogged = false;
            wmiText = "";
            bool exceptionThrown = false;
            ManagementScope scope = new ManagementScope(@"\\.\root\EnterpriseLibrary");
            scope.Options.EnablePrivileges = true;
            EventQuery eq = new EventQuery("SELECT * FROM SecurityConfigurationFailureEvent");
            ManagementEventWatcher watcher = new ManagementEventWatcher(scope, eq);
            watcher.EventArrived += new EventArrivedEventHandler(WatcherEventArrived);
            watcher.Start();
            try
            {
                ISecurityCacheProvider cacheProvider = SecurityCacheFactory.GetSecurityCacheProvider("InvalidProvider");

            }
            catch (ConfigurationErrorsException)
            {
                exceptionThrown = true;
            }
            Assert.AreEqual(true, exceptionThrown);

            int timeoutCount = 10;
            int count = 0;
            bool loopTimed = false;
            while (!wmiLogged && (loopTimed == false))
            {
                Thread.Sleep(100);
                count++;
                if (count == timeoutCount)
                {
                    loopTimed = true;
                }
            }
            watcher.Stop();
            Assert.AreEqual(true, wmiLogged);
            if (wmiText.IndexOf("SecurityConfigurationFailureEvent") == -1)
            {
                Assert.Fail();
            }
        }


        [TestMethod]
        public void SecurityCacheConfigFailureEventLogTest()
        {
            bool exceptionThrown = false;
            try
            {
                ISecurityCacheProvider cacheProvider = SecurityCacheFactory.GetSecurityCacheProvider("InvalidProvider");

            }
            catch (ConfigurationErrorsException)
            {
                exceptionThrown = true;
            }
            Assert.AreEqual(true, exceptionThrown);

            string logName = EventLog.LogNameFromSourceName("Application", ".");
            using (EventLog eventlog = new EventLog(logName))
            {

                int entryCount = eventlog.Entries.Count;
                if (!(eventlog.Entries[eventlog.Entries.Count - 1].Message.Contains("InvalidProvider")))
                    Assert.Fail("Event Not logged");
            }
        }

        private int GetCounterValue(string counterName)
        {
            //Verify the perf counter
            string categoryName = "Enterprise Library Security Counters";
            string instanceName = new AppDomainNameFormatter().CreateName("Total");
            if (PerformanceCounterCategory.InstanceExists(instanceName, categoryName))
            {
                using (PerformanceCounter counter = new PerformanceCounter())
                {
                    counter.CategoryName = categoryName;
                    counter.CounterName = counterName;
                    counter.InstanceName = instanceName;
                    return (int)counter.RawValue;
                }
            }
            return 0;
        }
        private void WatcherEventArrived(object sender, EventArrivedEventArgs args)
        {
            wmiLogged = true;
            /// Validate the message here
            wmiText = args.NewEvent.GetText(TextFormat.Mof);
        }
        
    }
}
