using System;
using System.Text;
using System.Collections.Generic;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using System.Security.Principal;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Security;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration.ObjectBuilder;

namespace SecurityCachingFuncTests
{
    /// <summary>
    /// Summary description for SecurityCachingIsolatedStorageDictTestFixture
    /// </summary>
    [TestClass]
    public class SecurityCachingIsolatedStorageDictTestFixture
    {
        IIdentity identity;
        private static IPrincipal principal;
        private static object profile;
        private DictionaryConfigurationSource configSource;
        SecurityCacheProvider securityCache;
        public SecurityCachingIsolatedStorageDictTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestInitialize()]
        public void SetUp()
        {
            identity = new GenericIdentity("testuser", "SecurityCache");
            string[] roles = new string[] { "admin", "managers" };
            principal = new GenericPrincipal(identity, roles);
            profile = "New Profile";

            SecurityCachingDictSource sourceSection = new SecurityCachingDictSource();
            configSource = sourceSection.BuildDictionarySourceSection();

            SecurityCacheProviderFactory securityCacheFactory = new SecurityCacheProviderFactory(configSource);
            securityCache = securityCacheFactory.Create("CacheProviderIsolatedDic") as SecurityCacheProvider;
        }

        #region Principal
        [TestMethod]
        public void IsolatedAddPrincipalDicTest()
        {

            Assert.IsNotNull(securityCache);

            IToken token = securityCache.SavePrincipal(principal);
            Assert.IsNotNull(token);
            Assert.IsNotNull(token.Value);

            IPrincipal cachedPrincipal = securityCache.GetPrincipal(token);
            Assert.IsNotNull(cachedPrincipal);
            Assert.AreEqual(cachedPrincipal.Identity.Name, "testuser");
            Assert.IsTrue(cachedPrincipal.IsInRole("admin"));
            Assert.IsTrue(cachedPrincipal.IsInRole("managers"));
            Assert.IsFalse(cachedPrincipal.IsInRole("users"));

        }

        [TestMethod]
        public void IsolatedGetExpiredPrincipalDicTest()
        {

            Assert.IsNotNull(securityCache);

            IToken token = securityCache.SavePrincipal(principal);
            Assert.IsNotNull(token);
            Assert.IsNotNull(token.Value);

            securityCache.ExpirePrincipal(token);

            IPrincipal cachedPrincipal = securityCache.GetPrincipal(token);
            Assert.IsNull(cachedPrincipal);

        }


        #endregion

        #region Identity
        [TestMethod]
        public void IsolatedAddIdentityDicTest()
        {

            Assert.IsNotNull(securityCache);

            IToken token = securityCache.SaveIdentity(identity);
            Assert.IsNotNull(token);
            Assert.IsNotNull(token.Value);

            IIdentity cachedIdentity = securityCache.GetIdentity(token);
            Assert.IsNotNull(cachedIdentity);
            Assert.AreEqual(cachedIdentity.Name, "testuser");
        }

        [TestMethod]
        public void IsolatedGetExpiredIdentityDicTest()
        {

            Assert.IsNotNull(securityCache);

            IToken token = securityCache.SaveIdentity(identity);
            Assert.IsNotNull(token);
            Assert.IsNotNull(token.Value);

            securityCache.ExpireIdentity(token);

            IIdentity cachedIdentity = securityCache.GetIdentity(token);
            Assert.IsNull(cachedIdentity);
        }
        #endregion

        #region Profile
        [TestMethod]
        public void IsolatedAddProfileDicTest()
        {

            Assert.IsNotNull(securityCache);

            IToken token = securityCache.SaveProfile(profile);
            Assert.IsNotNull(token);
            Assert.IsNotNull(token.Value);

            object retrievedProfile = securityCache.GetProfile(token);
            Assert.IsNotNull(retrievedProfile);
            Assert.AreEqual(retrievedProfile.ToString(), "New Profile");
        }

        [TestMethod]
        public void IsolatedGetExpiredProfileDicTest()
        {

            Assert.IsNotNull(securityCache);

            IToken token = securityCache.SaveProfile(profile);
            Assert.IsNotNull(token);
            Assert.IsNotNull(token.Value);

            securityCache.ExpireProfile(token);

            object retrievedProfile = securityCache.GetProfile(token);
            Assert.IsNull(retrievedProfile);
        }

        #endregion

        [TestMethod]
        public void IsolatedGetNonExistingTestFixture()
        {

            Assert.IsNotNull(securityCache);

            IToken token = securityCache.SaveIdentity(identity);
            Assert.IsNotNull(token);
            Assert.IsNotNull(token.Value);

            securityCache.ExpireIdentity(token);

            IPrincipal cachedPrincipal = securityCache.GetPrincipal(token);
            Assert.IsNull(cachedPrincipal);
        }


        [TestMethod]
        public void IsolatedAddIdentityPrincipalProfileDicTest()
        {

            Assert.IsNotNull(securityCache);

            IToken token = securityCache.SaveIdentity(identity);
            Assert.IsNotNull(token);
            Assert.IsNotNull(token.Value);

            securityCache.SavePrincipal(principal, token);

            securityCache.SaveProfile(profile, token);

            IPrincipal cachedPrincipal = securityCache.GetPrincipal(token);
            Assert.IsNotNull(cachedPrincipal);
            Assert.AreEqual(cachedPrincipal.Identity.Name, "testuser");
            Assert.IsTrue(cachedPrincipal.IsInRole("admin"));
            Assert.IsTrue(cachedPrincipal.IsInRole("managers"));
            Assert.IsFalse(cachedPrincipal.IsInRole("users"));

            IIdentity cachedIdentity = securityCache.GetIdentity(token);
            Assert.IsNotNull(cachedIdentity);
            Assert.AreEqual(cachedIdentity.Name, "testuser");

            object retrievedProfile = securityCache.GetProfile(token);
            Assert.IsNotNull(retrievedProfile);
            Assert.AreEqual(retrievedProfile.ToString(), "New Profile");

        }
    }
}
