using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Common;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Security;
using Microsoft.Practices.EnterpriseLibrary.Security.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Instrumentation.Configuration;

namespace SecurityCoreFuncTests
{
    public class AuthorizationRuleDictSource
    {
        public DictionaryConfigurationSource BuildDictionarySourceSection()
        {
            DictionaryConfigurationSource section = new DictionaryConfigurationSource();
            SecuritySettings secSettings = new SecuritySettings();
            secSettings.DefaultAuthorizationProviderName = "DefaultRuleProviderDic";

            AuthorizationRuleProviderData arpd = new AuthorizationRuleProviderData("DefaultRuleProviderDic");
            arpd.Rules.Add(new AuthorizationRuleData("TestIdentityRuleDic", "I:TestUser"));
            arpd.Rules.Add(new AuthorizationRuleData("TestRoleRuleDic", "R:Manager"));
            arpd.Rules.Add(new AuthorizationRuleData("TestIdentityORRoleRuleDic", "I:TestUser OR R:Manager"));
            arpd.Rules.Add(new AuthorizationRuleData("TestIdentityAndRoleRuleDic", "I:TestUser And R:Manager"));
            arpd.Rules.Add(new AuthorizationRuleData("TestIdentityORIdentityRuleDic", "I:TestUser OR I:Me"));
            arpd.Rules.Add(new AuthorizationRuleData("TestIdentityAndIdentityRuleDic", "I:TestUser And I:Me"));
            arpd.Rules.Add(new AuthorizationRuleData("TestRoleORRoleRuleDic", "R:Manager Or R:Employee"));
            arpd.Rules.Add(new AuthorizationRuleData("TestRoleAndRoleRuleDic", "R:Manager And R:Employee"));
            arpd.Rules.Add(new AuthorizationRuleData("TestRoleOrIdentityRuleDic", "R:Manager Or I:TestUser"));
            arpd.Rules.Add(new AuthorizationRuleData("TestRuleName With Spaces Dic", "R:Manager"));
            arpd.Rules.Add(new AuthorizationRuleData("TestRule With Spaces Dic", "R:Test Manager"));
            arpd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Security.AuthorizationRuleProvider, Microsoft.Practices.EnterpriseLibrary.Security");
            secSettings.AuthorizationProviders.Add(arpd);

            CustomAuthorizationProviderData customAuthProvider = new CustomAuthorizationProviderData("MyAuthorizationProvider", Type.GetType("SecurityCoreFuncTests.CustomAuthorizationProvider,SecurityCoreFuncTests"));
            customAuthProvider.Attributes.Add("key1", "value1");
            secSettings.AuthorizationProviders.Add(customAuthProvider);    
            
            section.Add("securityConfiguration", secSettings);
            InstrumentationConfigurationSection ics = new InstrumentationConfigurationSection(true, true, true);
            section.Add("instrumentationConfiguration", ics);

            return section;
        }
    }
}
